/*----------------------------------------------------------------------------
 *      Name:    extEeprom.c
 *      Purpose: External EEPROM
 *
 *	A2dDriver  shall sample the cold block temperature, and the base plate 
 *	temperature. The sample rate and running average shall be attributes that can 
 *	modified with commands, and saved in NV memory.      
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#include <LPC214X.H>                        /* LPC214x definitions */
#include <stdio.h>
#include "system.h"
#include "type.h"
#include "serial.h"
#include "i2c.h"

void a2d0_isr (void)__irq; 
void a2d1_isr (void)__irq; 

static ushort myA2dVal = 0;
static uchar a2dChnlSelected = 0;
uchar isA2d0ConvDone = FALSE;

// initilaize the internal A/D variables, hardware pins, and interrupts
void a2dInit(void)
{
	PINSEL1 |= 0x01000000;					// P0.28 - 25:24 = 01 for  ADC0.1		A/D Cold Block Temp
	PINSEL1 |= 0x04000000;					// P0.29 - 27:26 = 01 for  ADC0.2   A/D Base Plate temperature

	isA2d0ConvDone = FALSE;

	VICVectAddr7 = (unsigned long)a2d0_isr;   
	VICVectCntl7 = 0x20 | 18;     // A/D AD0 Interrupt -> IRQ Slot 6
	VICIntEnable = 1 << 18;       // Enable A/D  AD0 Interrupt
}

// is the A/D done with the conversion?
uchar isA2dDone(void)
{
	uchar myStatus = FALSE;

	if(AD0STAT & (1 << a2dChnlSelected))
		myStatus = TRUE;
	
	return (myStatus); 
}



// start a conversion for the internal A/D, channel 0 - channel 7
void a2dChnlStart(uchar myChnl)		// A/D Channel 0 - 7
{
	a2dChnlSelected = myChnl & 0xFF;

	if(myChnl == 1 || myChnl == 2)
		isA2d0ConvDone = FALSE;
	
	AD0CR = 0x01203000 | (1 << a2dChnlSelected);		// Single channel start converting with a 1MHz clock
}

// read the A/D channel that was previously started (end of conversion should be completed)
ushort a2dChnlRead(void)
{
	switch(a2dChnlSelected)
	{
		case 0:
			myA2dVal = AD0DR0;
			break;

		case 1:

			myA2dVal = AD0DR1;
			break;
		case 2:

			myA2dVal = AD0DR2;
			break;
		case 3:

			myA2dVal = AD0DR3;
			break;

		case 4:
			myA2dVal = AD0DR4;
			break;

		case 5:
			myA2dVal = AD0DR5;
			break;

		case 6:
			myA2dVal = AD0DR6;
			break;

		case 7:
			myA2dVal = AD0DR7;
			break;

		default:
			swError(__FILE__, __LINE__);
 	}

	return((myA2dVal >> 6) & 0x03FF);
}

// Read the Cold Block Temperature, Base Plate Temperature
void readAllA2dChnls(uchar myPort)
{
	ushort myVal;
	char myBuf[20];
	volatile short myTicks = 0;

	// get the Cold Block Value
	a2dChnlStart(A2D_CHNL_COLD_BLOCK);

 	timer0Init(MAX_TIMEOUT_A2D);
	timer0Start();

	while(isA2d0ConvDone == FALSE)
	{
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			swError(__FILE__, __LINE__);
		 	break;
		}
	}

	timer0Stop();

	myVal = a2dChnlRead();


	sprintf(myBuf, "A2D Cold Blk: %d\n", myVal);
	putString(myPort, myBuf);


  // get the Base Plate value
	a2dChnlStart(A2D_CHNL_BASE_PLATE);

	timer0Init(MAX_TIMEOUT_A2D);
	timer0Start();

	while(isA2d0ConvDone == FALSE)
	{
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			swError(__FILE__, __LINE__);
		 	break;
		}
	}

	timer0Stop();

	myVal = a2dChnlRead();

	sprintf(myBuf, "A2D Base Plate: %d\n", myVal);
	putString(myPort, myBuf);
}

// Diagnostics to test and verify the A/D																								
void a2dDiags(void)
{
	PINSEL1 |= 0x01000000;					// 25:24 = 01 for A2D 0
	AD0CR = 0x01203002;		// Single channel start converting with a 1MHz clock using P0.28 as an input
	//AD0CR = 0x01203004;		// Single channel start converting with a 1MHz clock using P0.29 as an input

  while(1) 	// continually read A/D 1
	{
		delayMs(30);
		myA2dVal = AD0DR1;
		AD0CR = 0x01203002;	
	}
}

// A/D0 isr
void a2d0_isr (void) __irq
{
	AD0CR &= 0xF8FFFFFF;		// Single channel stop converting

	isA2d0ConvDone = TRUE;
		  
	VICVectAddr = 0;                 /* Acknowledge Interrupt */	   
}

// A/D1 isr
void a2d1_isr (void) __irq
{
	AD1CR &= 0xF8FFFFFF;		// Single channel stop converting
															  
	VICVectAddr = 0;                 /* Acknowledge Interrupt */	   
}

