/*----------------------------------------------------------------------------
 *      Name:    elapsedTime.c
 *      Purpose: Elapsed Time
 *
 *	ElapsedTime  record the amount of time that has occurred when the laser is on
 *	with a 1 minute resolution. Use a shadow register to record and increment the 
 *	elapsed time, and store the elapsed time in NVRAM when a power down occurs.      
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/
#include <LPC214X.H>                        /* LPC214x definitions */
#include <stdio.h>
#include "system.h"
#include "type.h"
#include "serial.h"
#include "sysCfg.h"
#include "queue.h"

extern void displayPrompt(uchar myPort);
extern void cHandlerProcess(ushort myMode);


static ulong myStartTime[STOP_WATCH_MAX_SEL] = { 0, 0, 0, 0};
static ulong myStopTime[STOP_WATCH_MAX_SEL] = { 0, 0, 0, 0};
static ulong myCurLsrOnTimeInSecs[STOP_WATCH_MAX_SEL] = { 0, 0, 0, 0};
static STOP_WATCH_TYPE stopWatchCurState[STOP_WATCH_MAX_SEL] = { RESET, RESET, RESET, RESET}; 

extern char powerUpInitCurr;
extern char statLedFunction;

// keep track of how long the laser is on.
// The clock runs for error codes 0, 4, 8 only.
// The stop watch can be reset, started, stopped and also logged
// the time is maintained even when after the stop, so that restarting the 
// laser stop watch maintains previous "on" time.   
void lsrOnStopWatch(STOP_WATCH_SEL mySelection, STOP_WATCH_TYPE myMode)
{
	switch(myMode)
	{
		case RESET:
			myStartTime[mySelection] = 0;
			myCurLsrOnTimeInSecs[mySelection] = 0;
			stopWatchCurState[mySelection] = RESET;
			break;

	 	case STOP:
			if(stopWatchCurState[mySelection] != STOP)
			{
				myStopTime[mySelection] = oneSecTicks;
				myCurLsrOnTimeInSecs[mySelection] += (myStopTime[mySelection] - myStartTime[mySelection]);
				myStartTime[mySelection] = myStopTime[mySelection];
				stopWatchCurState[mySelection] = STOP;
			}
			break;

	 	case START:
			if(stopWatchCurState[mySelection] != START)		 // if we are already running (in START mode), than just keep running
			{
				myStartTime[mySelection] = oneSecTicks;
				stopWatchCurState[mySelection] = START;
			}
			break;

	 	case LOG:
			if(mySelection == LSR_STOP_WATCH)
			{
				// Save the laser on time to non volatile
				sysCfgShadowRegs.laserHours += ((myCurLsrOnTimeInSecs[mySelection] / 60.0) / 60.0);
				saveSysCfgDelta(EE_AREA_SYS_CFG); 
			}
			myCurLsrOnTimeInSecs[mySelection] = 0;
			break;
		case TRIGGER:
			break;
		default:
			swError(__FILE__, __LINE__);				
	}

}

// if the laser has been on for 12 mins than log the time
void logStopWatch12Mins(void)
{
	if(stopWatchCurState[LSR_STOP_WATCH] == START)
	{
		if((myCurLsrOnTimeInSecs[LSR_STOP_WATCH] + (oneSecTicks - myStartTime[LSR_STOP_WATCH])) >= (12 * 60))
		{
			myCurLsrOnTimeInSecs[LSR_STOP_WATCH] += (oneSecTicks - myStartTime[LSR_STOP_WATCH]);
			myStartTime[LSR_STOP_WATCH] = oneSecTicks;
			lsrOnStopWatch(LSR_STOP_WATCH, LOG);
		}	
	}
	else if(stopWatchCurState[LSR_STOP_WATCH] == STOP)
	{
		if(myCurLsrOnTimeInSecs[LSR_STOP_WATCH] >= (12 * 60))
		{
			lsrOnStopWatch(LSR_STOP_WATCH, LOG);
		}																									 
	}
	else if(stopWatchCurState[LSR_STOP_WATCH] == RESET)
		;								
	else
		swError(__FILE__, __LINE__);
}

// has the command from the comm port been inactive for more than 10 seconds?
void isCmdInActiveFor20Secs(void)
{
	uchar mySize;

	if(stopWatchCurState[CMD_COMM0_STOP_WATCH] == START)
	{
		if(((oneSecTicks - myStartTime[CMD_COMM0_STOP_WATCH])) >= 20)
		{
			mySize = qFullIndicator(SER0_Q, IN_Q);
			flushQ(SER0_Q, IN_Q, mySize);
			lsrOnStopWatch(CMD_COMM0_STOP_WATCH, RESET);
			displayPrompt(COMM_0);
		}	
	}
	else if(stopWatchCurState[CMD_COMM0_STOP_WATCH] == RESET)
		;
	else if(stopWatchCurState[CMD_COMM0_STOP_WATCH] == STOP)
		;
	else									
		swError(__FILE__, __LINE__);
}

// has the initial current power-up waiting period lapsed? 
void isInitialCurrPowerUpLapsed(void)
{
	if(stopWatchCurState[INIT_CURR_PWRUP_STOP_WATCH] == START)
	{
		if((oneSecTicks - myStartTime[INIT_CURR_PWRUP_STOP_WATCH]) >= 10)
		{
			stopWatchCurState[INIT_CURR_PWRUP_STOP_WATCH] = TRIGGER;
		}
	}
	else 
		if(stopWatchCurState[INIT_CURR_PWRUP_STOP_WATCH] == TRIGGER)
		{
			//Set to C=0 with power at LP=sysCfgShadowRegs.lp
			powerUpInitCurr = TRUE;
			cHandlerProcess(NO_PARSER); 
			lsrOnStopWatch(INIT_CURR_PWRUP_STOP_WATCH, RESET);		
		}
		else					
			swError(__FILE__, __LINE__);
}


