/*----------------------------------------------------------------------------
 *      Name:    errLog.c
 *      Purpose: Error Log
 *
 *	ErrLog  the error log shall record a number of errors into a circular buffer.
 *	Bit encoded field which states which errors have occurred.
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#include <LPC214X.H>                        /* LPC214x definitions */
#include <stdio.h>
#include <string.h>

#include "system.h"
#include "syscfg.h"
#include "serial.h"
#include "errLog.h"
#include "type.h"
#include "extEeprom.h"

// prototypes
void saveSwErrChkSum(void);

extern void leHandlerProcess(ushort myMode);

// Error faults are translated from the Vort error to the Brandx error
// Ex: Vort Val Out of Range = pos 3, and BrandX uses position 8
const long vort2BrandX[16] = { 0, INVALID_ERR, INVALID_ERR, 8, 1, 6, INVALID_ERR, 12, 5, 4, 15, 3, 7, INVALID_ERR, INVALID_ERR, 16 };

const char errDescriptionVort[][MAX_ERR_RESPONSE] = {
	"LASER EMISSION ACTIVE",
	"STANDBY",			// errors start here and below, Bit 0 error
	"WARMUP",
	"VALUE OUT OF RANGE",
	"INVALID COMMAND",
	"INTERLOCK OPEN",
	"TEC OFF",
	"DIODE OVER CURRENT",
	"DIODE TEMPERATURE FAULT",
	"BASE PLATE TEMPERATURE FAULT",
	"BUFFER OVERFLOW",
	"EEPROM ERROR",
	"I2C ERROR",
	"COMMAND TIME OUT",
	"WATCH DOG ERROR",
	"FATAL ERROR",
	"END OF LIFE",

	""
};

const char errDescriptionBrandX[][MAX_ERR_RESPONSE] = {
	"SYSTEM OK",
	"BAD COMMAND",			// errors start here and below, Bit 0 error
	"BAD CODE",
	"EEPROM CHECKSUM ERROR",
	"BASE PLATE TEMPERATURE",
	"LASER DIODE TEMPERATURE",
	"INTERLOCK FAULT",
	"I2C ERROR",
	"VALUE OUT OF RANGE",
	"",
	"",
	"FLASH RESET",
	"OVER CURRENT",
	"RESET CLOCK",
	"SYSTEM TIMEOUT",
	"BUFFER OVERFLOW",
	"FATAL ERROR",

	""
};

uchar myTraceNum = 0;

static ulong errLogFlags = 0;
static ulong prevErrLogFlags = 0;	// used for ?FC, and ?FD
static ulong storedFaults = 0;
static SYS_HW_ERRS sysHwStatus;		// bit encoded status of hardware devices

char myErrBuf[MAX_SW_ERRORS][MAX_SW_ERR_LEN];
char myTraceBuf[MAX_SW_TRACES][MAX_SW_ERR_LEN];

void errLogInit(void)
{
	errLogFlagClearAll();
	storedFaultsClearAll();
	clearSysHwErr();
}

void errLogFlagSet(ulong myError)
{

	errLogFlags |= (1 << (myError - 1));

	prevErrLogFlags = errLogFlags;

	storedFaults |= errLogFlags;	

	isErrLogStatGrtr16();
}

ulong errLogFlagGet(void)
{
	return errLogFlags;
}

ulong prevErrLogFlagGet(void)
{
	return prevErrLogFlags;
}

ulong storedFaultsGet(void)
{
	return storedFaults;
}

void errLogDiscriptionGet(uchar myPort)
{
	uchar myPos = 0;
	uchar myErrs = 0;
	char myBuf[MAX_ERR_RESPONSE];
	ulong myErrFlags = 0;

	myErrFlags = errLogFlagGet();//Use current error flags Vortran-DMS 11/19/09

	putString(myPort, "\n");

	// determine the number of bits set
	while(myPos < 32)
	{
		if(myErrFlags & (1 << myPos++))
			myErrs++;
	}

	myPos=0;

	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
	{
		if(myErrFlags)		// errors are bit encoded
		{
			while(errDescriptionVort[myPos + 1][0] != 0)	// the first error description is not an error " LASER EMISSION ACTIVE "
			{
				if(myErrFlags & (1 << myPos))
				{
					if(myErrs-- == 1)	// last error, no CR needed
						sprintf(myBuf, "?FD=%s", &errDescriptionVort[myPos + 1][0]);
					else
						sprintf(myBuf, "?FD=%s\n", &errDescriptionVort[myPos + 1][0]);

					putString(myPort, myBuf);
				}

				myPos++;
			}
		}
		else
				putString(myPort, "?FD=LASER EMISSION ACTIVE");

#if 0
	else if (sysCfgShadowRegs.protoMode != EE_DEF_PROTO_VORT)
		{
			while(errDescriptionBrandX[myPos + 1][0] != 0)	// the first error description is not an error " LASER EMISSION ACTIVE "
			{
				if(myErrFlags & (1 << myPos))
				{
					if(myErrs-- == 1)	// last error, no CR needed
						sprintf(myBuf, "?FD=%s", &errDescriptionBrandX[myPos + 1][0]);
					else
						sprintf(myBuf, "?FD=%s\n", &errDescriptionBrandX[myPos + 1][0]);

					putString(myPort, myBuf);
				}

				myPos++;
			}
		}
		else
		{
			putString(myPort, "?FD=SYSTEM OK");
		}
#endif

	}
}

void prevErrLogDiscriptionGet(uchar myPort)
{
	uchar myPos = 0;
	char myBuf[MAX_ERR_RESPONSE];

	putString(myPort, "\n");

	if(prevErrLogFlags)		// errors are bit encoded
	{
		while(errDescriptionVort[myPos + 1][0] != 0)	// the first error description is not an error " LASER EMISSION ACTIVE "
		{
			if(prevErrLogFlags & (1 << myPos))
			{
				sprintf(myBuf, "RFC=%s", &errDescriptionVort[myPos + 1][0]);
				putString(myPort, myBuf);
			}

			myPos++;
		}
	}
	else
	{
		putString(myPort, "RFC=LASER EMISSION ACTIVE");
	}

}

void errLogFlagClearAll(void)
{
	if(errLogFlags == 0)
		prevErrLogFlags = 0;

	errLogFlags = 0;
}

void storedFaultsClearAll(void)
{
	storedFaults = 0;
}

void errLogFlagClear(ulong myError)
{
		errLogFlags &= ~(1 << (myError - 1));
}

/*
short isFaultActive(void)
{
	if(errLogFlags)
		return TRUE;
	else
		return FALSE;
}
*/

// Error Log Status Values of 4 or Greater
short isErrLogStatGrtr4(void)
{
	short myStatus = TRUE;

	if(errLogFlags & 0xFFFF7FFC) //Exclude EOL Flag, Bit 15
	{
		myStatus = TRUE;
	}
	else
		myStatus = FALSE;

	return(myStatus);
}


// Set PO.17 (I/O Fault Pin) High for Values of 16 or Greater
short isErrLogStatGrtr16(void)
{
	short myStatus = TRUE;

	if(errLogFlags & 0xFFFF7FF0) //Exclude EOL Flag, Bit 15
	{
		IO0SET = PIN_FAULT_INDC;
		IO0CLR = PIN_LSR_RDY; ///Deassert Laser Ready Line DMS 11/6/09

		// Stop Laser Emission!!
		IO0CLR = PIN_LSR_ON;
		IO0CLR = PIN_LSR_EMISSION_LED;
		lsrOnStopWatch(LSR_STOP_WATCH, STOP);
		myStatus = TRUE;
	}
	else
	{
		if(errLogFlags & 0x00008000) //Is End of Life (32768)?
		{
			if(calShadowRegs.endOfLifeFaultLine == TRUE)
			{
				IO0SET = PIN_FAULT_INDC;
			}
		}
		else
		{
			IO0CLR = PIN_FAULT_INDC;
		}
		myStatus = FALSE;
	}

	return(myStatus);
}

// Error Log Status Values of 32 or Greater
short isErrLogStatGrtr32(void)
{
	short myStatus = TRUE;

	if(errLogFlags & 0xFFFF7FE0) //Exclude EOL Flag, Bit 15
	{
		myStatus = TRUE;
	}
	else
		myStatus = FALSE;

	return(myStatus);
}

// this API is called when a system hardware error has occurred
void setSysHwErr(SYS_HW_ERRS myError)
{
	sysHwStatus |= myError;		// bit encoded status of hardware devices
}
// this API is called when a system hardware error has occurred
void clearSysHwErr(void)
{
	sysHwStatus = NO_SYS_HW_ERRS;		// bit encoded status of hardware devices
}

// this API is called when a system hardware error has occurred
SYS_HW_ERRS getSysHwErr(void)
{
	return(sysHwStatus);		// bit encoded status of hardware devices
}

// called when a software error occurs
// save 10 errors, use the "vort"
void swError(char * myFile, short myLine)
{
	static uchar myErrNum = 0;
	static short myPrevLine = 0;
	static char myPrevFile[20] = { 0 };
#ifndef SIMULATOR_ACTIVE
	//ushort myEePos = 0;
	//uchar myBytesWritten = 0;
	//char *myPtr;
#endif

	if((myLine != myPrevLine) && (strcmp(myFile, myPrevFile) != 0)) // only save the error if it did not just occur, toss identical consecutive errors
	{
		if(myErrNum < MAX_SW_ERRORS)		// only save the first few errors
		{
			if((strlen(myFile) + 5 ) >= MAX_SW_ERR_LEN)
			{
				;	// string is too large!!!
			}
			else
		  {
				sprintf(myErrBuf[myErrNum], "%s %d\n", myFile, myLine);
				myPrevLine = myLine;

#ifndef SIMULATOR_ACTIVE
#if 0
				// save information to eeprom
				myPtr = &myErrBuf[myErrNum][0];
				myEePos = EE_SW_ERR_START + (MAX_SW_ERR_LEN * myErrNum);	// determine the eeprom offset in the sw errors image area

				while(*myPtr != 0)
				{
					eepromWrite(myEePos++, *myPtr++);

					if(myBytesWritten++ >= (MAX_SW_ERR_LEN - 1))
						break;
				}

				eepromWrite(myEePos, *myPtr);		// write the null character

				myErrNum++;

				// Update the eeprom checksum
				saveSwErrChkSum();
#endif
#endif
			}
		}
	}
}

void displaySwError(void)
{
	uchar myPos = 0;

	for(myPos = 0; myPos < MAX_SW_ERRORS; myPos++)		// display 10 errors
	{
		if(myErrBuf[myPos][0] != 0)
		{
			putString(COMM_0, myErrBuf[myPos]);
			putString(COMM_0, "\n");
		}
	}
}

#if 0
void displayEeSwError(void)
{
	uchar myPos = 0;
	uchar myErrStr[MAX_SW_ERR_LEN];
	uchar *myPtr;
	char myEePos;
	uchar bytesRead = 0;
	ushort myCheckSum = 0;
	uchar myChkSumMsb = 0;
	uchar myChkSumLsb = 0;

	// verify the EEPROM checksum
	myCheckSum = chkSumCalc(EE_SW_ERR_START, EE_SW_ERR_CHKSUM - 1);

	eepromRead(EE_SW_ERR_CHKSUM, &myChkSumMsb);
	eepromRead(EE_SW_ERR_CHKSUM + 1, &myChkSumLsb);

	if(myCheckSum != (myChkSumMsb << 8 | myChkSumLsb))
		;	// FRG Initialize Software Error image?
	else
	{
		for(myPos = 0; myPos < MAX_SW_ERRORS; myPos++)		// display all errors
		{
			myEePos = EE_SW_ERR_START + (MAX_SW_ERR_LEN * myPos);
			myPtr = myErrStr;

			while(bytesRead < MAX_SW_ERR_LEN)
			{
				eepromRead(myEePos, myPtr);
				bytesRead++;

				if(*myPtr == 0)
					break;

				if(myErrStr[0] != 0)
				{
					putString(COMM_0, (char *) myErrStr);
					putString(COMM_0, "\n");
				}
			}
		}
	}
}
#endif

// Configure the EEPROM Image error for Software Errors to Null strings for all errors
void saveSwErrDefaults(void)
{
	uchar myErrNum;
	ushort myEeAddr;

#if VORT_BOARD
	for(myErrNum = 0; myErrNum < MAX_SW_ERRORS; myErrNum++)		// initialize error strings to NULL
	{
		myEeAddr = EE_SW_ERR_START + (MAX_SW_ERR_LEN * myErrNum);

		eepromWrite(myEeAddr, 0);
	}

	saveSwErrChkSum();
#endif
}

void saveSwErrChkSum(void)
{
	ushort myCheckSum;

#if VORT_BOARD
	// calculate the EEPROM checksum
	myCheckSum = chkSumCalc(EE_SW_ERR_START, EE_SW_ERR_CHKSUM - 1);

	// Save the checksum
	eepromWrite(EE_SW_ERR_CHKSUM, (myCheckSum >> 8) & 0xFF);
	eepromWrite(EE_SW_ERR_CHKSUM + 1, myCheckSum & 0xFF);
#endif
}



void swTrace(char * myFile, short myLine)
{
	static short myPrevLine = 0;
	static char myPrevFile[20] = { 0 };

	allInterruptsOff();
#ifndef SIMULATOR_ACTIVE
	//ushort myEePos = 0;
	//uchar myBytesWritten = 0;
	//char *myPtr;
#endif

	if((myLine != myPrevLine) && (strcmp(myFile, myPrevFile) != 0)) // only save the error if it did not just occur, toss identical consecutive errors
	{
		if(myTraceNum >= MAX_SW_TRACES)
			myTraceNum = 0;				// circular buffer

		if(myTraceNum < MAX_SW_TRACES)		// only save the first few errors
		{
			if((strlen(myFile) + 5 ) >= MAX_SW_ERR_LEN)
			{
				;	// string is too large!!!
			}
			else
		  {
				sprintf(myTraceBuf[myTraceNum], "%s %d\n", myFile, myLine);
				myPrevLine = myLine;

#if 0		// do not save to eeprom
#ifndef SIMULATOR_ACTIVE
				// save information to eeprom
				myPtr = &myTraceBuf[myTraceNum][0];
				myEePos = EE_SW_ERR_START + (MAX_SW_ERR_LEN * myTraceNum);	// determine the eeprom offset in the sw errors image area

				while(*myPtr != 0)
				{
					eepromWrite(myEePos++, *myPtr++);

					if(myBytesWritten++ >= (MAX_SW_ERR_LEN - 1))
						break;
				}

				eepromWrite(myEePos, *myPtr);		// write the null character

				// Update the eeprom checksum
				saveSwErrChkSum();
#endif
#endif

				myTraceNum++;
			}
		}
	}

	allInterruptsOn();
}
