/*----------------------------------------------------------------------------
 *      Name:    extA2d.c
 *      Purpose: External A2D
 *
 *	ExtA2dDriver  this 4 channel A2D shall sample the laser current, power output,
 *	and Drive Current IDRIVE. The sample rate and running average shall be attributes that can
 *	modified with commands, and saved in NV memory.
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "system.h"
#include "type.h"
#include "i2c.h"
#include "serial.h"
#include "errLog.h"
#include "sysCfg.h"


#define A2D_CHNL_SEL_NONE		0x00		//  channels
#define A2D_CHNL_SEL_C0			0x10		//  channels
#define A2D_CHNL_SEL_C1			0x20		//  channels
#define A2D_CHNL_SEL_S01		0x30		//  channels
#define A2D_CHNL_SEL_C2			0x40		//  channels
#define A2D_CHNL_SEL_S02		0x50		//  channels
#define A2D_CHNL_SEL_S12		0x60		//  channels
#define A2D_CHNL_SEL_S012		0x70		//  channels
#define A2D_CHNL_SEL_C3			0x80		//  channels
#define A2D_CHNL_SEL_S03		0x90		//  channels
#define A2D_CHNL_SEL_S13		0xA0		//  channels
#define A2D_CHNL_SEL_S013		0xB0		//  channels
#define A2D_CHNL_SEL_S23		0xC0		//  channels
#define A2D_CHNL_SEL_S023		0xD0		//  channels
#define A2D_CHNL_SEL_S123		0xE0		//  channels
#define A2D_CHNL_SEL_ALL		0xF0		//  all channels

#define A2D_REF_SEL_EXT		0x08		//  External Reference select
#define A2D_FILTER_BYPASS	0x04		//  The value written to this bit of the control register
																	// determines whether the filtering on SDA and SCL is
																	// enabled or bypassed. If this bit is set to 0, the
																	// filtering is enabled; if it set to 1, the filtering is bypassed.
#define A2D_BIT_TRIAL			0x02
#define A2D_BIT_SAMPLE		0x01

#define DEFAULT_CFG_VAL	(A2D_CHNL_SEL_S012 | A2D_REF_SEL_EXT)		// sequence between all channels

ushort extA2dValues[3] = {0, 0, 0};
ushort extA2dCurrentValues[3]={0,0,0};
char dbg[80];

void extA2dInit(void)
{
	// write a register in the A/D to verify that it is working
	writeI2C2_Byte(I2C_ADDR_A2D | I2C_WRITE, A2D_CHNL_SEL_S012 | A2D_REF_SEL_EXT);
	delayMs(5);

	if(isrI2cErr == TRUE)		// did the A/D fail?
	{
		setSysHwErr(HW_ERR_A2D);	// yes
		isrI2cErr = FALSE;
	}
}


short extA2dDiags(void)
{
	short myStatus = FAIL;
	uchar myDebug[2];

 	// Test External A/D
	writeI2C2_Byte(I2C_ADDR_A2D | I2C_WRITE, DEFAULT_CFG_VAL);

	delayMs(2);
	//readI2C2_Dual_D2a_BytesBlocking(I2C_ADDR_A2D | I2C_READ, (uchar *) &myDebug, 2);

	delayMs(2);

	if(myDebug[0] == 0x01 && myDebug[1] == 0x02 )
		myStatus = PASS;

	return(myStatus);
}

// read all three channels of the A2D and store the results based on the channel
void getAllExtA2dRegs(void)
{
	uchar samplesToAvg = 4;
	uchar myCnt;
	uchar mySamples[6];	// channel 0 - 2 (2 bytes each)
	ushort myTmpSamples[3] = { 0, 0, 0};	// channel 0 - 2 (2 bytes each)
	ushort myAvgSamples[3] = { 0, 0, 0};	// channel 0 - 2 (2 bytes each)
	uchar currSelect=0;

	//IO1=MUX1,IO3=MUX2
	//IN1,IN2
	//L,L ALL OFF
	//H,L NO0 ON, SOA 
	//L,H NO1 ON, DFB
	//H,H NO2 ON, HEATER

	for(currSelect=0;currSelect<3;currSelect++)
	{
		myCnt = samplesToAvg;

		//Select current sense channel to measure
		if(currSelect==EXT_A2D_SOA_CURR)
		{
			 extPsaSet(1,1);
			 delayMs(5);
			 extPsaSet(3,0);
		}
		else
			if(currSelect==EXT_A2D_DFB_CURR)
			{
				 extPsaSet(1,0);
				 delayMs(5);
				 extPsaSet(3,1);
			}
			else
				if(currSelect==EXT_A2D_HEATER_CURR)
				{
				 	 extPsaSet(1,1);
					 delayMs(5);
					 extPsaSet(3,1);
				}

		delayMs(50); //Allow for analog signal stabilization after switching current measurement channels

		myAvgSamples[0] = 0;
	
		// perform multiple reads and average the results.
		while(myCnt--)
		{		
			writeI2C2_Byte(I2C_ADDR_A2D | I2C_WRITE, A2D_CHNL_SEL_C0 | A2D_REF_SEL_EXT);
			delayMs(5);
	
			readI2C2_Dual_A2d_BytesBlocking(I2C_ADDR_A2D | I2C_READ, (uchar *) &mySamples, 2);
	
			delayMs(5);
	
			myTmpSamples[(mySamples[0] >> 4) & 0x3] = ((mySamples[0] & 0x0F) << 8) | mySamples[1];
					   	
			//sprintf(dbg, "\n0:%d", myTmpSamples[0]);
			//putString(0, dbg);
			
	
			myAvgSamples[0] += myTmpSamples[0];
			
		}
		extA2dCurrentValues[currSelect] = myAvgSamples[0] / samplesToAvg; //SOA Curr, DFB Curr, HTR Curr		
	}

	myCnt = samplesToAvg;
	myAvgSamples[1] = 0;
	myAvgSamples[2] = 0;
	while(myCnt--)
	{
	 	writeI2C2_Byte(I2C_ADDR_A2D | I2C_WRITE, A2D_CHNL_SEL_S12 | A2D_REF_SEL_EXT);

		delayMs(5);
	
		readI2C2_Dual_A2d_BytesBlocking(I2C_ADDR_A2D | I2C_READ, (uchar *) &mySamples, 4);

		delayMs(5);

		myTmpSamples[(mySamples[0] >> 4) & 0x3] = ((mySamples[0] & 0x0F) << 8) | mySamples[1];
		myTmpSamples[(mySamples[2] >> 4) & 0x3] = ((mySamples[2] & 0x0F) << 8) | mySamples[3];

		//sprintf(dbg, "\n1:%d, 2:%d",myTmpSamples[1],myTmpSamples[2]);
		//putString(0, dbg);

		myAvgSamples[1] += myTmpSamples[1];
		myAvgSamples[2] += myTmpSamples[2];		
	}
	extA2dValues[1] = myAvgSamples[1] / samplesToAvg; //SOA Light
	extA2dValues[2] = myAvgSamples[2] / samplesToAvg; //SOA Curr Drive
}

void readAllExtA2dRegs(uchar myPort)
{
	char myBuf[MAX_CMD_SIZE];

	getAllExtA2dRegs();

	sprintf(myBuf, "A2D.0 SOA Cur = %d\n", extA2dCurrentValues[EXT_A2D_SOA_CURR]);
	putString(myPort, myBuf);

	sprintf(myBuf, "A2D.0 DFB Cur = %d\n", extA2dCurrentValues[EXT_A2D_DFB_CURR]);
	putString(myPort, myBuf);

	sprintf(myBuf, "A2D.0 Heater Cur = %d\n", extA2dCurrentValues[EXT_A2D_HEATER_CURR]);
	putString(myPort, myBuf);

	sprintf(myBuf, "A2D.1 SOA Light = %d\n", extA2dValues[1]);
	putString(myPort, myBuf);

	sprintf(myBuf, "A2D.2 SOA Drv Cur = %d", extA2dValues[2]);
	putString(myPort, myBuf);
}

