/*----------------------------------------------------------------------------
 *      Name:    extA2d.c
 *      Purpose: External A2D
 *
 *	ExtA2dDriver  this 4 channel A2D shall sample the laser current, power output,
 *	and Point B. The sample rate and running average shall be attributes that can
 *	modified with commands, and saved in NV memory.
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "system.h"
#include "type.h"
#include "i2c.h"
#include "serial.h"
#include "errLog.h"
#include "sysCfg.h"


#define A2D_CHNL_SEL_NONE		0x00		//  channels
#define A2D_CHNL_SEL_C0			0x10		//  channels
#define A2D_CHNL_SEL_C1			0x20		//  channels
#define A2D_CHNL_SEL_S01		0x30		//  channels
#define A2D_CHNL_SEL_C2			0x40		//  channels
#define A2D_CHNL_SEL_S02		0x50		//  channels
#define A2D_CHNL_SEL_S12		0x60		//  channels
#define A2D_CHNL_SEL_S012		0x70		//  channels
#define A2D_CHNL_SEL_C3			0x80		//  channels
#define A2D_CHNL_SEL_S03		0x90		//  channels
#define A2D_CHNL_SEL_S13		0xA0		//  channels
#define A2D_CHNL_SEL_S013		0xB0		//  channels
#define A2D_CHNL_SEL_S23		0xC0		//  channels
#define A2D_CHNL_SEL_S023		0xD0		//  channels
#define A2D_CHNL_SEL_S123		0xE0		//  channels
#define A2D_CHNL_SEL_ALL		0xF0		//  all channels

#define A2D_REF_SEL_EXT		0x08		//  External Reference select
#define A2D_FILTER_BYPASS	0x04		//  The value written to this bit of the control register
																	// determines whether the filtering on SDA and SCL is
																	// enabled or bypassed. If this bit is set to 0, the
																	// filtering is enabled; if it set to 1, the filtering is bypassed.
#define A2D_BIT_TRIAL			0x02
#define A2D_BIT_SAMPLE		0x01

#define DEFAULT_CFG_VAL	(A2D_CHNL_SEL_S012 | A2D_REF_SEL_EXT)		// sequence between all channels

ushort extA2dDfbValues[3] = {0, 0, 0};

void extA2dDfbInit(void)
{
	// write a register in the A/D to verify that it is working
	writeI2C2_Byte(I2C_ADDR_A2D_DFB | I2C_WRITE, A2D_CHNL_SEL_S012 | A2D_REF_SEL_EXT);
	delayMs(5);

	if(isrI2cErr == TRUE)		// did the A/D fail?
	{
		setSysHwErr(HW_ERR_A2D_DFB);	// yes
		isrI2cErr = FALSE;
	}
}


short extA2dDfbDiags(void)
{
	short myStatus = FAIL;
	uchar myDebug[2];

 	// Test External A/D
	writeI2C2_Byte(I2C_ADDR_A2D_DFB | I2C_WRITE, DEFAULT_CFG_VAL);

	delayMs(2);
	//readI2C2_Dual_D2a_BytesBlocking(I2C_ADDR_A2D | I2C_READ, (uchar *) &myDebug, 2);

	delayMs(2);

	if(myDebug[0] == 0x01 && myDebug[1] == 0x02 )
		myStatus = PASS;

	return(myStatus);
}

// read all three channels of the A2D and store the results based on the channel
void getAllExtA2dDfbRegs(void)
{
	uchar samplesToAvg = 4;
	uchar myCnt;
	uchar mySamples[6];	// channel 0 - 2 (2 bytes each)
	ushort myTmpSamples[3] = { 0, 0, 0};	// channel 0 - 2 (2 bytes each)
	ushort myAvgSamples[3] = { 0, 0, 0};	// channel 0 - 2 (2 bytes each)

	myCnt = samplesToAvg;

	// perform multiple reads and average the results.
	while(myCnt--)
	{
		writeI2C2_Byte(I2C_ADDR_A2D_DFB | I2C_WRITE, A2D_CHNL_SEL_S012 | A2D_REF_SEL_EXT);

		delayMs(5);

		readI2C2_Dual_A2d_BytesBlocking(I2C_ADDR_A2D_DFB | I2C_READ, (uchar *) &mySamples, 6);

		delayMs(5);

		myTmpSamples[(mySamples[0] >> 4) & 0x3] = ((mySamples[0] & 0x0F) << 8) | mySamples[1];
		myTmpSamples[(mySamples[2] >> 4) & 0x3] = ((mySamples[2] & 0x0F) << 8) | mySamples[3];
		myTmpSamples[(mySamples[4] >> 4) & 0x3] = ((mySamples[4] & 0x0F) << 8) | mySamples[5];

		myAvgSamples[0] += myTmpSamples[0];
		myAvgSamples[1] += myTmpSamples[1];
		myAvgSamples[2] += myTmpSamples[2];
	}

	extA2dDfbValues[0] = myAvgSamples[0] / samplesToAvg;
	extA2dDfbValues[1] = myAvgSamples[1] / samplesToAvg;
	extA2dDfbValues[2] = myAvgSamples[2] / samplesToAvg;
}

void readAllExtA2dDfbRegs(uchar myPort)
{
	char myBuf[MAX_CMD_SIZE];

	getAllExtA2dDfbRegs();

	sprintf(myBuf, "A2D.0 DFB Ch0 =   %d\n", extA2dDfbValues[0]);
	putString(myPort, myBuf);

	sprintf(myBuf, "A2D.1 DFB Ch1 = %d\n", extA2dDfbValues[1]);
	putString(myPort, myBuf);

	sprintf(myBuf, "A2D.2 DFB Ch2 =   %d", extA2dDfbValues[2]);
	putString(myPort, myBuf);
}

