/*----------------------------------------------------------------------------
 *      Name:    extDualD2a_DFB.c
 *      Purpose: External Dual D2A for DFB Control
 *
 *	ExtDualD2aDriver  this Dual D2A shall control the DFB
 *	12 Bit D2A.
 *	VOutA = Current
 *	VOutB = Modulation
 *
 * The AD5627R/AD5647R/AD5667R, AD5627/AD5667 members of the nanoDAC family are low
 * power, dual, 12-, 14-, 16-bit buffered voltage-out DACs with/without on-chip
 * reference. All devices operate from a single 2.7 V to 5.5 V supply, are guaranteed
 * monotonic by design, and have an I2C-compatible serial interface.
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "system.h"
#include "type.h"
#include "errLog.h"
#include "i2c.h"
#include "serial.h"
#include "syscfg.h"

// The command byte is the first byte after the I2C address: "Cmd" D5 - D3
typedef enum {
	D2A_IN_REG_N = 0,
	D2A_UPD_DAC_REG_N,
	D2A_WR_IN_REG_N,
	D2A_WR_DAC_CHNL_N,
	D2A_PWR_UP_DWN,
	D2A_RESET,
	D2A_LDAC_REG_SETUP,
	D2A_INTRNL_REF_SETUP
} D2A_CMD_DEF;

// The command byte is the first byte after the I2C address: "Addr" D2 - D0
typedef enum {
	D2A_DAC_A = 0,
	D2A_DAC_B,
	D2A_DAC_BOTH
} D2A_DAC_ADDR;

#define SW_RESET_CMD			0x28
#define PWR_UP_DOWN_CMD		0x20
#define REF_SETUP_CMD			0x38

// prototypes
void extD2aDfbSetCur(ushort myLevel);
void extD2aDfbSetLight(ushort myLevel);
void extD2aDfbLdacSetup(void);
void extD2aDfbRefSetup(void);
void extDualD2aDfbSet(uchar myReg, ushort myLevel);

static ushort d2aDfbCur = 0;
static ushort d2aDfbLight = 0;


// initialize the External D2A
void extDualD2aDfbInit(void)
{
	ushort myLevel=0x0800; //2048 counts
	// reset?

	// Power up/Down Mode?
	//writeI2C2_3Bytes(I2C_ADDR_D_D2A, myReg, (myLevel >> 8) & 0xFF, myLevel & 0xFF);

	extD2aDfbPwrUpDown();
	delayMs(150);

	// use the d2a command above to verify that the D2A is working
	if(isrI2cErr == TRUE)		// did the D/A fail?
	{
		setSysHwErr(HW_ERR_DUAL_D2A_DFB);	// yes
		isrI2cErr = FALSE;
	}

	extD2aDfbLdacSetup();
	delayMs(150);
	extD2aDfbRefSetup();
	delayMs(150);

	extDualD2aDfbSet((D2A_WR_DAC_CHNL_N << 3) | D2A_DAC_A, myLevel); //Set current DAC to 2048 counts
	delayMs(150);
	extDualD2aDfbSet((D2A_WR_DAC_CHNL_N << 3) | D2A_DAC_B, myLevel); //Set light DAC to 2048 counts
	delayMs(150);
}

void extDualD2aDfbSet(uchar myReg, ushort myLevel)
{
	writeI2C2_3Bytes(I2C_ADDR_D_D2A_DFB, myReg, ((myLevel & 0x0F00) >> 4) | ((myLevel & 0x00F0) >> 4), (myLevel & 0x0F) << 4);	 // 12 bit data is in the upper 3 nibbles (Nib 3 - 1, Nib 0 is N/A)
}


void extDualD2aDfbGet(void)
{
	uchar myD2A_A[2];
	uchar myD2A_B[2];

	readI2C2_BytesBlocking(I2C_ADDR_D_D2A_DFB, (D2A_IN_REG_N << 3) | D2A_DAC_A, (uchar *) &myD2A_A, 2);
	readI2C2_BytesBlocking(I2C_ADDR_D_D2A_DFB, (D2A_IN_REG_N << 3) | D2A_DAC_B, (uchar *) &myD2A_B, 2);
}


void extD2aDfbSetCur(ushort myLevel)
{
	d2aDfbCur = myLevel;
	extDualD2aDfbSet((D2A_WR_DAC_CHNL_N << 3) | D2A_DAC_A, myLevel);
}

void extD2aDfbSetLight(ushort myLevel)
{
	d2aDfbLight =  myLevel;
	extDualD2aDfbSet((D2A_WR_DAC_CHNL_N << 3) | D2A_DAC_B, myLevel);
}

void extD2aDfbLdacSetup(void)
{
	writeI2C2_3Bytes(I2C_ADDR_D_D2A_DFB, D2A_LDAC_REG_SETUP << 3, 0x0, 0x3);
}

void extD2aDfbRefSetup(void)
{
	writeI2C2_3Bytes(I2C_ADDR_D_D2A_DFB, D2A_INTRNL_REF_SETUP << 3, 0x0, 0x0);
}

void extD2aDfbPwrUpDown(void)
{
		//writeI2C2_3Bytes(I2C_ADDR_D_D2A, D2A_PWR_UP_DWN << 3, 0x0, 0x13);		// Diags Pull down 1K test
	writeI2C2_3Bytes(I2C_ADDR_D_D2A_DFB, D2A_PWR_UP_DWN << 3, 0x0, 0x03);
}

void readAllExtD2aDfbRegs(uchar myPort)
{
	char myBuf[30];

	sprintf(myBuf, "DFB Current = %d\n", d2aDfbCur);
	putString(myPort, myBuf);
	sprintf(myBuf, "DFB Light = %d\n", d2aDfbLight);
	putString(myPort, myBuf);
}

void writeExtD2aDfbReg(ulong myULong1, ulong myULong2)
{
	if(myULong1 == 100)
		extD2aDfbSetCur(myULong2);
	else if(myULong1 == 101)
		extD2aDfbSetLight(myULong2);
}


short diagsDualD2aDfb(void)
{
	short myStatus = FAIL;
	uchar myDebug[3];

	extD2aDfbPwrUpDown();
	delayMs(150);

	extD2aDfbLdacSetup();
	delayMs(150);
	extD2aDfbRefSetup();
	delayMs(150);

	extD2aDfbSetCur(0x800);
	delayMs(150);
	extD2aDfbSetLight(0xFFF);
	delayMs(150);

 	// Test D2A
	writeI2C2_3Bytes(I2C_ADDR_D_D2A_DFB, D2A_WR_IN_REG_N << 3, 0x01, 0x02);

	delayMs(150);
	//readI2C2_Dual_D2a_BytesBlocking(I2C_ADDR_D_D2A, (uchar *) &myDebug, 3);

	delayMs(150);

	if(myDebug[1] == 0x01 && myDebug[2] == 0x02 )
		myStatus = PASS;

	return(myStatus);
}

