/*----------------------------------------------------------------------------
 *      Name:    extDualPots.c
 *      Purpose: External Dual Pots
 *
 *	ExtDualPots -  this driver shall control two Pots. The last settings for the 
 *	Pot shall be stored in the internal NVRAM of the Pot. The pot shall control the 
 *	laser power P Cal, and laser current I Cal. 
 *
 * The AD5251/AD5252 are dual-channel, I2C, nonvolatile mem-ory, digitally controlled
 * potentiometers with 64/256 positions, respectively. These devices perform the same
 * electronic adjust-ment functions as mechanical potentiometers, trimmers, and variable
 * resistors. The parts versatile programmability allows multiple modes of operation,
 * including read/write access in the RDAC and EEMEM registers, increment/decrement of
 * resistance, resistance changes in 6 dB scales, wiper setting readback, and extra 
 * EEMEM for storing user-defined information, such as memory data for other components,
 * look-up table, or system identification information.     
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "system.h"
#include "type.h"
#include "i2c.h"
#include "serial.h"
#include "extDualPots.h"
#include "errLog.h"



void extDualPotsLsrInit(void)
{
	// read a register in the POT to verify that it is working
#if 0
	extDualPotsGetLtfb();
	delayMs(5);

	if(isrI2cErr == TRUE)		// did the A/D fail?
	{
		setSysHwErr(HW_ERR_POT_LSR);	// yes
		isrI2cErr = FALSE;
	}	
#endif
}

void extDualPotsLsrSet(uchar myReg, uchar myValue)
{
		writeI2C2_2Bytes(I2C_ADDR_POT_LSR, myReg, myValue);
		writeI2C2_Byte(I2C_ADDR_POT_LSR, myReg | D_POT_STORE_CMD);
}


uchar extDualPotsLsrGet(DUAL_DIGITAL_POT myReg)
{
	uchar myValue;

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR, myReg | D_POT_EE_RDAC, (uchar *) &myValue, 1);

	return myValue;
}

void extDualPotsSetLtfb(uchar myValue)
{
	extDualPotsLsrSet(D_POT_RDAC1, myValue);
}

void extDualPotsSetCur(uchar myValue)
{
	extDualPotsLsrSet(D_POT_RDAC3, myValue);
}

uchar extDualPotsGetLtfb(void)
{
	uchar myVal = 0;

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR, D_POT_RDAC1, (uchar *) &myVal, 1);

	return(myVal);
}

uchar extDualPotsGetCur(void)
{
	uchar myVal = 0;

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR, D_POT_RDAC3, (uchar *) &myVal, 1);

	return(myVal);
}

void readAllExtPotsLsrRegs(uchar myPort)
{
	uchar myVal;
	char myBuf[20];

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR, D_POT_RDAC1, (uchar *) &myVal, 1);
	delayMs(2);
	sprintf(myBuf, "RDAC1: %d\n", myVal);
	putString(myPort, myBuf);

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR, D_POT_RDAC3, (uchar *) &myVal, 1);
	delayMs(2);
	sprintf(myBuf, "RDAC3: %d\n", myVal);
	putString(myPort, myBuf);
}

void potsLsrWriteReg(ulong myULong1, ulong myULong2)
{
	if(myULong1 == 50)
 	{
		extDualPotsSetCur(myULong2);
	}
	else if(myULong1 == 51)
	{
		extDualPotsSetLtfb(myULong2);
	}

}

short diagsDualPotLsr(void)
{
	short myStatus = FAIL;
	uchar myDebug;

	extDualPotsSetLtfb(0x3C);
	delayMs(20);
	extDualPotsSetCur(0x2A);
	delayMs(20);

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR, D_POT_RDAC1, (uchar *) &myDebug, 1);
	delayMs(20);

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR, D_POT_RDAC3, (uchar *) &myDebug, 1);
	delayMs(20);

 	// Test Dual Pots
	writeI2C2_2Bytes(I2C_ADDR_POT_LSR, D_POT_STR_EMEM_4 | D_POT_EE_RDAC, 0xC3);

	delayMs(20);
	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR, D_POT_STR_EMEM_4 | D_POT_EE_RDAC, (uchar *) &myDebug, 1);

	delayMs(20);

	if(myDebug & 0xC3)
		myStatus = PASS;
	else
		while(1);

	return(myStatus);
}
