/*----------------------------------------------------------------------------
 *      Name:    extDualPotsLsrDfb.c
 *      Purpose: External Dual Pots
 *
 *	ExtDualPots - DFB Current Drive Channel and SOA Current Drive Channel Calibration Pots
 *
 *  RDAC 1: DFB Drive Current
 *  RDAC 3:	SOA Drive Current
 *
 * The AD5251/AD5252 are dual-channel, I2C, nonvolatile mem-ory, digitally controlled
 * potentiometers with 64/256 positions, respectively. These devices perform the same
 * electronic adjust-ment functions as mechanical potentiometers, trimmers, and variable
 * resistors. The parts versatile programmability allows multiple modes of operation,
 * including read/write access in the RDAC and EEMEM registers, increment/decrement of
 * resistance, resistance changes in 6 dB scales, wiper setting readback, and extra 
 * EEMEM for storing user-defined information, such as memory data for other components,
 * look-up table, or system identification information.     
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "system.h"
#include "type.h"
#include "i2c.h"
#include "serial.h"
#include "extDualPots.h"
#include "errLog.h"





void extDualPotsLsrDfbInit(void)
{
	// read a register in the POT to verify that it is working
#if 0
	extDualPotsDfbGetLtfb();
	delayMs(5);

	if(isrI2cErr == TRUE)		// did the A/D fail?
	{
		setSysHwErr(HW_ERR_POT_LSR_DFB);	// yes
		isrI2cErr = FALSE;
	}	
#endif
}

void extDualPotsLsrDfbSet(uchar myReg, uchar myValue)
{
		writeI2C2_2Bytes(I2C_ADDR_POT_LSR_DFB, myReg, myValue);
		writeI2C2_Byte(I2C_ADDR_POT_LSR_DFB, myReg | D_POT_STORE_CMD);
}


uchar extDualPotsLsrDfbGet(DUAL_DIGITAL_POT myReg)
{
	uchar myValue;

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR_DFB, myReg | D_POT_EE_RDAC, (uchar *) &myValue, 1);

	return myValue;
}

//SOA Drive Current
void extDualPotsSoaSetCur(uchar myValue)
{
	extDualPotsLsrDfbSet(D_POT_RDAC3, myValue);
}
//DFB Drive Current
void extDualPotsDfbSetCur(uchar myValue)
{
	extDualPotsLsrDfbSet(D_POT_RDAC1, myValue);
}

uchar extDualPotsSoaGetCur(void)
{
	uchar myVal = 0;

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR_DFB, D_POT_RDAC3, (uchar *) &myVal, 1);
	delayMs(2);
	return(myVal);
}

uchar extDualPotsDfbGetCur(void)
{
	uchar myVal = 0;

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR_DFB, D_POT_RDAC1, (uchar *) &myVal, 1);
	delayMs(2);
	return(myVal);
}

void readAllExtPotsLsrDfbRegs(uchar myPort)
{
	uchar myVal;
	char myBuf[20];

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR_DFB, D_POT_RDAC1, (uchar *) &myVal, 1);
	delayMs(2);
	sprintf(myBuf, "RDAC1: %d\n", myVal);
	putString(myPort, myBuf);

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR_DFB, D_POT_RDAC3, (uchar *) &myVal, 1);
	delayMs(2);
	sprintf(myBuf, "RDAC3: %d\n", myVal);
	putString(myPort, myBuf);
}

void potsLsrDfbWriteReg(ulong myULong1, ulong myULong2)
{
	if(myULong1 == 90)
 	{
		extDualPotsDfbSetCur(myULong2);
	}
	else if(myULong1 == 91)
	{
		extDualPotsSoaSetCur(myULong2);
	}

}

short diagsDualPotLsrDfb(void)
{
	short myStatus = FAIL;
	uchar myDebug;

	extDualPotsSoaSetCur(0x3C);
	delayMs(20);
	extDualPotsDfbSetCur(0x2A);
	delayMs(20);

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR_DFB, D_POT_RDAC1, (uchar *) &myDebug, 1);
	delayMs(20);

	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR_DFB, D_POT_RDAC3, (uchar *) &myDebug, 1);
	delayMs(20);

 	// Test Dual Pots
	writeI2C2_2Bytes(I2C_ADDR_POT_LSR_DFB, D_POT_STR_EMEM_4 | D_POT_EE_RDAC, 0xC3);

	delayMs(20);
	readI2C2_BytesBlocking(I2C_ADDR_POT_LSR_DFB, D_POT_STR_EMEM_4 | D_POT_EE_RDAC, (uchar *) &myDebug, 1);

	delayMs(20);

	if(myDebug & 0xC3)
		myStatus = PASS;
	else
		while(1);

	return(myStatus);
}
