/*----------------------------------------------------------------------------
 *      Name:    extDualPotsMod.c
 *      Purpose: External Dual Pots Modulation
 *
 *	ExtDualPots -  this driver shall control two Pots. The last settings for the
 *	Pot shall be stored in the internal NVRAM of the Pot. .
 *
 * The AD5251/AD5252 are dual-channel, I2C, nonvolatile mem-ory, digitally controlled
 * potentiometers with 64/256 positions, respectively. These devices perform the same
 * electronic adjust-ment functions as mechanical potentiometers, trimmers, and variable
 * resistors. The parts versatile programmability allows multiple modes of operation,
 * including read/write access in the RDAC and EEMEM registers, increment/decrement of
 * resistance, resistance changes in 6 dB scales, wiper setting readback, and extra
 * EEMEM for storing user-defined information, such as memory data for other components,
 * look-up table, or system identification information.
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "system.h"
#include "type.h"
#include "errLog.h"
#include "i2c.h"
#include "serial.h"
#include "extDualPots.h"
#include "sysCfg.h"



void extDualPotsModInit(void)
{
#if 0
	uchar myVal;

	// read a register in the External Dual Pots to verify that it is working
	readI2C2_BytesBlocking(I2C_ADDR_POT_MOD, D_POT_RDAC1, (uchar *) &myVal, 1);
	delayMs(5);

	if(isrI2cErr == TRUE)		// did the A/D fail?
	{
		setSysHwErr(HW_ERR_POT_MOD);	// yes
		isrI2cErr = FALSE;
	}
#endif

}

void extDualPotsModSet(uchar myReg, uchar myValue)
{
		writeI2C2_2Bytes(I2C_ADDR_POT_MOD, myReg, myValue);
		writeI2C2_Byte(I2C_ADDR_POT_MOD, myReg | D_POT_STORE_CMD);
}


uchar extDualPotsModGet(DUAL_DIGITAL_POT myReg)
{
	uchar myValue;

	readI2C2_BytesBlocking(I2C_ADDR_POT_MOD, myReg | D_POT_EE_RDAC, (uchar *) &myValue, 1);

	return myValue;
}

void modifyModulationPots(uchar myVal)
{
	uchar setting1;
	uchar setting3;

	setting3 = (uchar)(myVal * calShadowRegs.epcScaleFactor/100.0);	//Coarse
	setting1 = (uchar)(myVal * calShadowRegs.epcScaleFactor2/100.0); //Fine


	extDualPotsModSet(SET_MOD_COARSE, setting3);	 //RDAC3
	extDualPotsModSet(SET_MOD_FINE, setting1);  //RDAC1


	//myVal = extDualPotsModGet(SET_MOD_COARSE);
	//myVal = extDualPotsModGet(SET_MOD_FINE);
}

void extDualPotsSet1(uchar myValue)
{
		extDualPotsModSet(D_POT_RDAC1, myValue);
}

void extDualPotsSet3(uchar myValue)
{
		extDualPotsModSet(D_POT_RDAC3, myValue);
}

uchar extDualPotsGet1(void)
{
	uchar myVal = 0;

	readI2C2_BytesBlocking(I2C_ADDR_POT_MOD, D_POT_RDAC1, (uchar *) &myVal, 1);

	return(myVal);
}

uchar extDualPotsGet3(void)
{
	uchar myVal = 0;

	readI2C2_BytesBlocking(I2C_ADDR_POT_MOD, D_POT_RDAC3, (uchar *) &myVal, 1);

	return(myVal);
}

void readAllExtPotsModRegs(uchar myPort)
{
	uchar myVal;
	char myBuf[20];

	readI2C2_BytesBlocking(I2C_ADDR_POT_MOD, D_POT_RDAC1, (uchar *) &myVal, 1);
	delayMs(2);
	sprintf(myBuf, "RDAC1: %d\n", myVal);
	putString(myPort, myBuf);

	readI2C2_BytesBlocking(I2C_ADDR_POT_MOD, D_POT_RDAC3, (uchar *) &myVal, 1);
	delayMs(2);
	sprintf(myBuf, "RDAC3: %d\n", myVal);
	putString(myPort, myBuf);
}

void potsModWriteReg(ulong myULong1, ulong myULong2)
{
	if(myULong1 == 80)
 	{
		extDualPotsSet1(myULong2);
	}
	else if(myULong1 == 81)
	{
		extDualPotsSet3(myULong2);
	}
}


short diagsDualPotMod(void)
{
	short myStatus = FAIL;
	uchar myDebug;

	readI2C2_BytesBlocking(I2C_ADDR_POT_MOD, D_POT_RDAC1, (uchar *) &myDebug, 1);
	delayMs(20);

	readI2C2_BytesBlocking(I2C_ADDR_POT_MOD, D_POT_RDAC3, (uchar *) &myDebug, 1);
	delayMs(20);

 	// Test Dual Pots
	writeI2C2_2Bytes(I2C_ADDR_POT_MOD, D_POT_STR_EMEM_4 | D_POT_EE_RDAC, 0xC3);

	delayMs(20);
	readI2C2_BytesBlocking(I2C_ADDR_POT_MOD, D_POT_STR_EMEM_4 | D_POT_EE_RDAC, (uchar *) &myDebug, 1);

	delayMs(20);

	if(myDebug & 0xC3)
		myStatus = PASS;
	else
		while(1);

	return(myStatus);
}
