/*----------------------------------------------------------------------------
 *      Name:    extEeprom.c
 *      Purpose: External EEPROMs
 *
 *	EEPROM Driver  shall perform reads and writes to the NV memory, and perform
 *	a checksum on the EEPROM image. On power up the EEPROM shall perform a
 *	write verify at a specified diagnostics address to verify the device is working.      
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/
#include "type.h"
#include "system.h"
#include "extEeprom.h"
#include "i2c.h"
#include "errlog.h"
#include "serial.h"

char diagExtEeprom(void);

char isEepromGood(void);
static uchar isEepromWorking = FALSE;

// prototypes
short checkIfEepromAliveAndWell(void);

void eepromInit(void)
{
	// is the eeprom working?
	if(checkIfEepromAliveAndWell() == PASS)
		isEepromWorking = TRUE;
	else
	{
		isEepromWorking = FALSE;		// no,
		setSysHwErr(HW_ERR_EEPROM);
	}	
}

char eepromRead(ushort myAddr, uchar* myData)
{
	short myStatus = PASS;
	static short doOnce = FALSE;

	if(isEepromGood())
	{
		#if VORT_BOARD
			readI2C2_16BitAddr_BytesBlocking(I2C_ADDR_EEPROM, myAddr, (uchar *) myData, 1);
		#else
			readI2C2_BytesBlocking(I2C_ADDR_EEPROM, myAddr, (uchar *) myData, 1);
		#endif
	}
	else
	{
		myStatus = FAIL;

		if(doOnce == FALSE)
		{
			doOnce = TRUE;
			// FRG remove later
			putString(COMM_0, "*EEPROM FAILURE!!\n");

#if ACTIVATE_COMM1
			putString(COMM_1, "*EEPROM FAILURE!!\n");
#endif

			swError(__FILE__, __LINE__);
		}
	}

	return (myStatus);
}

void eepromWrite(ushort myAddr, uchar myByte)
{
	if(isEepromGood())
	{
		#if VORT_BOARD
			writeI2C2_3Bytes(I2C_ADDR_EEPROM, (myAddr & 0xFF00) >> 8, myAddr & 0xFF, myByte);
		#else
			writeI2C2_2Bytes(I2C_ADDR_EEPROM, myAddr, myByte);
		#endif
	}
}


// write a byte to a "diagnostic" memory address to verify that the eeprom is working and active
short checkIfEepromAliveAndWell(void)
{
	short myStatus = PASS;
	uchar myDataMsb = 0;
	uchar myDataLsb = 0;
	uchar myTestMsb = 0xA5;
	uchar myTestLsb = 0xC3;

	isEepromWorking = TRUE;	// must allow low level eeprom read/write commands to attempt to access the EEPROM chip

	// verify the data written
	eepromRead(EEPROM_DIAGS, &myDataMsb);

#if VORT_BOARD
		DelayUs(20);
#else
		delayMs(5);
#endif

	// verify the data written
	eepromRead(EEPROM_DIAGS + 1, &myDataLsb);

#if VORT_BOARD
		DelayUs(20);
#else
		delayMs(5);
#endif

	if(myDataMsb != myTestMsb)	// has the data been initialized previously?
	{
		eepromWrite(EEPROM_DIAGS, myTestMsb);	// no, write the diagnostics address
#if VORT_BOARD
		//DelayUs(20);
#else
		delayMs(5);
#endif
		eepromRead(EEPROM_DIAGS, &myDataMsb);
#if VORT_BOARD
		DelayUs(20);
#else
		delayMs(5);
#endif
	}

	if(myDataLsb != myTestLsb)	// has the data been initialized previously?
	{
		eepromWrite(EEPROM_DIAGS + 1, myTestLsb);	// no, write the diagnostics address
#if VORT_BOARD
		//DelayUs(20);
#else
		delayMs(5);
#endif
		eepromRead(EEPROM_DIAGS + 1, &myDataLsb);
#if VORT_BOARD
		DelayUs(20);
#else
		delayMs(5);
#endif
	}

	// verify the data
	if(myDataMsb != myTestMsb && myDataLsb != myTestLsb)
	{
		myStatus = FAIL;
		errLogFlagSet(ERR_EEPROM_ERROR);
	} 

	return myStatus;
}

char isEepromGood(void)
{
	return isEepromWorking;
}

char diagExtEeprom(void)
{	
	char myStatus = PASS;

	uchar myDebug = 1;

 	// Test EEPROM
	//writeI2C2_2Bytes(I2C_ADDR_EEPROM, 0, 0x3C);

		#if VORT_BOARD
			writeI2C2_3Bytes(I2C_ADDR_EEPROM, 0, 0, 0x3C);
		#else
			writeI2C2_2Bytes(I2C_ADDR_EEPROM, 0, 0x3C);
		#endif

	delayMs(10);
	//readI2C2_BytesBlocking(I2C_ADDR_EEPROM, 0, (uchar *) &myDebug, 1);

		#if VORT_BOARD
			readI2C2_16BitAddr_BytesBlocking(I2C_ADDR_EEPROM, 0, (uchar *) &myDebug, 1);
		#else
			readI2C2_BytesBlocking(I2C_ADDR_EEPROM, 0, (uchar *) &myDebug, 1);
		#endif

#if VORT_BOARD
		DelayUs(20);
#else
		delayMs(5);
#endif

	if(myDebug != 0x3C)
		myStatus = (char) FAIL;


	return myStatus;
}

