/*----------------------------------------------------------------------------
 *      Name:    extPsa.c
 *      Purpose: External 4Bit I/O Port
 *
 *
 * Digital Modulation DFB Channel and/or SOA Channel may be enabled simultaneously
 * "IO0" enables/disables SOA Channel
 * "IO2" enables/disables DFB Channel  
 *
 * The PCA9536 is an 8-pin CMOS device that provides 4 bits of General Purpose parallel
 * Input/Output (GPIO) expansion for I2C-bus/SMBus applications and was developed to
 * enhance the NXP Semiconductors family of I2C-bus I/O expanders. 
 * The PCA9536 consists of a 4-bit Configuration register (input or output selection), 4-bit
 * Input Port register, 4-bit Output Port register and a 4-bit Polarity Inversion register
 * (active HIGH or active LOW operation). The system master can enable the I/Os as either
 * inputs or outputs by writing to the I/O configuration bits. The data for each input or output
 * is kept in the corresponding Input Port or Output Port register. The polarity of the read
 * register can be inverted with the Polarity Inversion register. All registers can be read by the
 * system master.   
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "system.h"
#include "type.h"
#include "i2c.h"
#include "errLog.h"	 	

#define PSA_CMD_IN_PORT 	0x00
#define PSA_CMD_OUT_PORT 	0x01
#define PSA_CMD_POLARITY 	0x02
#define PSA_CMD_CONFIG	 	0x03

#define PSA_DEFAULT_OUT_VALS		0x00

short extPsaDiags(void);
void extPsaControl(uchar myVal);
void extPsaSet(uchar myPort, uchar myVal);

static short psaOutVal = 0;

void extPsaInit(void)
{
	// write to the configuration register in the PSA 
	writeI2C2_2Bytes(I2C_ADDR_PSA | I2C_WRITE, PSA_CMD_CONFIG, 0x0);	// all ports are configured as outputs
	delayMs(1);

	if(isrI2cErr == TRUE)		// did the PSA fail?
	{
		setSysHwErr(HW_ERR_PSA);	// yes
		isrI2cErr = FALSE;
	}
	
	extPsaControl(PSA_DEFAULT_OUT_VALS);	
}


short extPsaDiags(void)
{
	short myStatus = PASS;
	//uchar myValue;

#if 0
	readI2C2_Dual_D2a_BytesBlocking(I2C_ADDR_POT_MOD, PSA_CMD_CONFIG, (uchar *) &myValue, 1);
	delayMs(1);

	if(isrI2cErr == TRUE)		// did the PSA fail?
	{
		myStatus = FAIL;
		setSysHwErr(HW_ERR_PSA);	// yes
		isrI2cErr = FALSE;
	}	
#endif

	// write a register in the PSA to verify that it is working
	writeI2C2_2Bytes(I2C_ADDR_PSA | I2C_WRITE, PSA_CMD_CONFIG, 0x0);	// all ports are configured as outputs
	delayMs(5);

	if(isrI2cErr == TRUE)		// did the PSA fail?
	{
		myStatus = FAIL;
		setSysHwErr(HW_ERR_PSA);	// yes
		isrI2cErr = FALSE;
	}	

#if 0
	readI2C2_Dual_D2a_BytesBlocking(I2C_ADDR_POT_MOD, PSA_CMD_CONFIG, (uchar *) &myValue, 1);
	delayMs(1);

	if(isrI2cErr == TRUE)		// did the PSA fail?
	{
		myStatus = FAIL;
		setSysHwErr(HW_ERR_PSA);	// yes
		isrI2cErr = FALSE;
	}
#endif	

	return(myStatus);
}

void extPsaControl(uchar myVal) //Write Raw Binary Value To Port Control Register
{
	psaOutVal = myVal;

	// write to the Output Port register in the PSA 
	writeI2C2_2Bytes(I2C_ADDR_PSA | I2C_WRITE, PSA_CMD_OUT_PORT, myVal);	// all ports are configured as outputs
	delayMs(1);

	if(isrI2cErr == TRUE)		// did the PSA fail?
	{
		setSysHwErr(HW_ERR_PSA);	// yes
		isrI2cErr = FALSE;
		psaOutVal = 0;
	}	
}

void extPsaSet(uchar myPort, uchar myVal) //Toggle a specific port on device
{
	//Use existing value of psaOutVal so other bits are not affected by writes to a desired bit
	if(myVal==1)
	{
		myVal= psaOutVal | (0x01 << myPort); //Ports [3:0]
	}
	else
	{
		myVal= psaOutVal & ~(0x01 << myPort); //Ports [3:0]
	}

	psaOutVal = myVal;

	// write to the Output Port register in the PSA
	writeI2C2_2Bytes(I2C_ADDR_PSA | I2C_WRITE, PSA_CMD_OUT_PORT, myVal);	// all ports are configured as outputs
	delayMs(1);

	if(isrI2cErr == TRUE)		// did the PSA fail?
	{
		setSysHwErr(HW_ERR_PSA);	// yes
		isrI2cErr = FALSE;
		psaOutVal = 0;
	}
}

 

short extPsaGet(void)
{
	return psaOutVal;
}


