/*----------------------------------------------------------------------------
 *      Name:    extQuadD2a.c
 *      Purpose: External Quad D2A
 *
 *	ExtQuadD2aDriver  this Quad D2A shall control the following channels:
 *
 *	VOutA = SOA Drive Current
 *	VOutB = DFB Drive Current
 *	VOutC = QD Module Heater
 *	VoutD =	SOA Light Regulation
 *
 *  AD5325 Quad DAC: The AD5305/AD5315/AD53251 are quad 8-, 10-, and 12-bit buffered voltage output 
 *	DACs in a 10-lead MSOP that operate from a single 2.5 V to 5.5 V supply, consuming 500 A at 3 V. 
 *	Their on-chip output amplifiers allow rail-to-rail output swing with a slew rate of 0.7 V/s. A 
 *	2-wire serial interface that operates at clock rates up to 400 kHz is used. This interface is SMBus 
 *	compatible at VDD < 3.6 V. Multiple devices can be placed on the same bus.
 *
 *---------------------------------------------------------------------------*/

 #include <stdio.h>
#include "system.h"
#include "type.h"
#include "errLog.h"
#include "i2c.h"
#include "serial.h"
#include "syscfg.h"


// The pointer byte is the first byte after the I2C address
//XX00DCBA - D7:D6 Don't Care, D5:D4 Zeros, D3:D0 DAC Channel Select

//4 DAC Channels A, B, C, D
typedef enum {
	D2A_DAC_A = 1,
	D2A_DAC_B = 2,
	D2A_DAC_C = 4,
	D2A_DAC_D = 8
} D2A_DAC_ADDR;

//Operating Modes, LDAC, Shutdown
#define CLR_NORMAL_OP 0x20

//LDAC is LOW = DAC registers updated on every write
 

static ushort d2aSoaCur = 0;
static ushort d2aDfbCur = 0;
static ushort d2aHeater = 0;
static ushort d2aSoaLight = 0;


// initialize the External D2A
void extQuadD2aInit(void)
{
	ushort myLevel=0x0000;

	// use the d2a command above to verify that the D2A is working
	if(isrI2cErr == TRUE)		// did the D/A fail?
	{
		setSysHwErr(HW_ERR_QUAD_D2A);	// yes
		isrI2cErr = FALSE;
	}

	//Set all DAC channels to 0 counts
	extQuadD2aSet(D2A_DAC_A | D2A_DAC_B | D2A_DAC_C | D2A_DAC_D, myLevel); 
}



void extQuadD2aSet(uchar myReg, ushort myLevel)
{
	writeI2C2_3Bytes(I2C_ADDR_D_D2A, myReg, CLR_NORMAL_OP | (myLevel >>8), myLevel);
}


void extDualD2aDfbGet(void)
{
	uchar myD2A_A[2];
	uchar myD2A_B[2];
	uchar myD2A_C[2];
	uchar myD2A_D[2];

	readI2C2_BytesBlocking(I2C_ADDR_D_D2A, D2A_DAC_A, (uchar *) &myD2A_A, 2);
	delayMs(5);
	readI2C2_BytesBlocking(I2C_ADDR_D_D2A, D2A_DAC_B, (uchar *) &myD2A_B, 2);
	delayMs(5);
	readI2C2_BytesBlocking(I2C_ADDR_D_D2A, D2A_DAC_C, (uchar *) &myD2A_C, 2);
	delayMs(5);
	readI2C2_BytesBlocking(I2C_ADDR_D_D2A, D2A_DAC_D, (uchar *) &myD2A_D, 2);
}

void extQuadD2aSoaSetCur(ushort myLevel)
{
	extQuadD2aSet(D2A_DAC_A, myLevel);
}

void extQuadD2aDfbSetCur(ushort myLevel)
{
	extQuadD2aSet(D2A_DAC_B, myLevel);
}

void extQuadD2aSetHeater(ushort myLevel)
{
	extQuadD2aSet(D2A_DAC_C, myLevel);
}

void extQuadD2aSoaSetLight(ushort myLevel)
{
	extQuadD2aSet(D2A_DAC_D, myLevel);
}

ushort extQuadD2aSoaGetCur()
{
	readAllExtD2aRegs(0);
	delayMs(2);
	return d2aSoaCur;
}
ushort extQuadD2aDfbGetCur()
{
	readAllExtD2aRegs(0);
	delayMs(2);
	return d2aDfbCur;
}
ushort extQuadD2aGetHeater()
{
   readAllExtD2aRegs(0);
   delayMs(2);
   return d2aHeater;
}
ushort extQuadD2aSoaGetLight()
{
   readAllExtD2aRegs(0);
   delayMs(2);
   return d2aSoaLight;
}

void readAllExtD2aRegs(uchar myPort)
{
	char myBuf[30];

	uchar myD2A_A[2]={0};
	uchar myD2A_B[2]={0};
	uchar myD2A_C[2]={0};
	uchar myD2A_D[2]={0};

	d2aSoaCur=0;
	d2aDfbCur=0;
	d2aHeater=0;
	d2aSoaLight=0;

	readI2C2_BytesBlocking(I2C_ADDR_D_D2A, D2A_DAC_A, (uchar *) &myD2A_A, 2);
	delayMs(10);
	readI2C2_BytesBlocking(I2C_ADDR_D_D2A, D2A_DAC_B, (uchar *) &myD2A_B, 2);
	delayMs(10);
	readI2C2_BytesBlocking(I2C_ADDR_D_D2A, D2A_DAC_C, (uchar *) &myD2A_C, 2);
	delayMs(10);
	readI2C2_BytesBlocking(I2C_ADDR_D_D2A, D2A_DAC_D, (uchar *) &myD2A_D, 2);
	delayMs(10);

	d2aSoaCur = (myD2A_A[0] & 0x0F) << 8;
	d2aSoaCur |= myD2A_A[1];

	d2aDfbCur = (myD2A_B[0] & 0x0F) << 8;
	d2aDfbCur |= myD2A_B[1];

	d2aHeater = (myD2A_C[0] & 0x0F) << 8;
	d2aHeater |= myD2A_C[1];

	d2aSoaLight = (myD2A_D[0] & 0x0F) << 8;
	d2aSoaLight |= myD2A_D[1];

	if(myPort>0)
	{
		sprintf(myBuf, "SOA Curr:  %d\n", d2aSoaCur);
		putString(myPort, myBuf);
		sprintf(myBuf, "DFB Curr:  %d\n", d2aDfbCur);
		putString(myPort, myBuf);
		sprintf(myBuf, "Heater:    %d\n", d2aHeater);
		putString(myPort, myBuf);
		sprintf(myBuf, "SOA Light: %d\n", d2aSoaLight);
		putString(myPort, myBuf);
	}
}

void writeExtD2aReg(ulong myULong1, ulong myULong2)
{
	if(myULong1 == 85)
		extQuadD2aSoaSetCur(myULong2);
	else if(myULong1 == 86)
		extQuadD2aDfbSetCur(myULong2);
	else if(myULong1 == 87)
		extQuadD2aSetHeater(myULong2);
	else if(myULong1==88)
		extQuadD2aSoaSetLight(myULong2);
		
}

short diagsQuadD2a(void)
{
	short myStatus = FAIL;
	uchar myDebug[3];

	delayMs(150);

	if(myDebug[1] == 0x01 && myDebug[2] == 0x02 )
		myStatus = PASS;

	return(myStatus);
}

















