/*----------------------------------------------------------------------------
 *      Name:    initBoard.c
 *      Purpose: Initialize Board
 *
 *	InitalizeBoard  initialize the hardware I/O pins, internal and external devices. All
 *	of the hardware devices are to be initialized prior to global interrupts being
 *	enabled.
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#include <LPC214X.H>                        /* LPC214x definitions */
#include "system.h"
#include "errLog.h"
#include "type.h"
#include "i2c.h"
#include "queue.h"
#include "sysCfg.h"
#include "usb.h"
#include "usbhw.h"
#include "parser.h"


extern void a2dInit(void);
extern void d2aInit(void);
extern void extPsaInit(void);
extern void parserInit(void);
extern void pulHandlerProcess(ushort myMode, ushort action);
extern void	lpHandlerProcess(ushort myMode);
extern void	lcHandlerProcess(ushort myMode);
extern void	cHandlerProcess(ushort myMode);
extern void	delayHandlerInit(void);
extern void	echoHandlerInit(void);
extern void leHandlerProcess(ushort myMode);
extern void	tecHandlerProcess(ushort myMode);
extern void vobtsHandlerProcess(ushort myMode);
extern void processEpcMode(void);
extern void diagsInit(void);
extern void setShadowRegsToDef(EE_AREA_TYPE);
extern void	monitorWarmUp(void);
extern uchar getBoardID(void);
extern void wDogInit(void);
extern void rfInjHandlerProcess(ushort myMode);
extern void calHtrCntHandlerProcess(ushort myMode);
extern void dfblcHandlerProcess(ushort myMode);
extern void calDfblcDacCntHandlerProcess(ushort myMode);

extern short warmUpInitTempIsFine;	// diode temperature during initial warmup period
extern char pulMethod;
extern char powerUp;

ulong PIN_TE_ON = 0x00010000;
ulong PIN_I_MOD_ON = 0x00080000;
ulong PIN_MOD_ON = 0x00080000;


void boardIdInit(void);

static uchar boardID = 0;

void initSw(void)
{
	vicDefInit();
#ifndef SIMULATOR_ACTIVE

	errLogInit();

	// Initialize internal Hardware
	d2aInit(); 	// internal D2A
	a2dInit();	// internal A2D
#endif

	qInit();
	serialInit();
	timerInit();
	timer1Start();	// Executive Timer

	lsrOnStopWatch(LSR_STOP_WATCH, RESET);
	lsrOnStopWatch(INIT_CURR_PWRUP_STOP_WATCH, RESET);


#ifndef SIMULATOR_ACTIVE
	i2CInit();

#endif



#ifndef SIMULATOR_ACTIVE
#if VORT_BOARD
	// Initialize external Hardware
	eepromInit();	  // dependent on i2CInit(), to display errors: EE CRC Error!

	// dependant on i2CInit()
	extPsaInit();			// 4 bit I/O for pulse mode PSA9536
	extA2dInit();

	extQuadD2aInit();
	extDualPotsLsrDfbInit();
	extSinglePotLsrInit();
	
	extDualPotsModInit();
	coldBlockTempInit();
#endif
#endif

#if VORT_BOARD
	sysCfgInit();
#else
		setShadowRegsToDef(EE_AREA_SYS_CFG);
		setShadowRegsToDef(EE_AREA_CAL);
#endif

	parserInit();

		/*
			1	Do EEPROM check using check sums.
			2	Set all of the shadow registers to the values stored in the EEPROM for both Calibration Data and User data.
			3	Check status of the Autostart line if Low then do not set LE=1 even if this was the last setting.
			4	Check EEPROM for first turn on and if true then load default values and DO NOT set LE=1.
			5	Set LP, LC, C, Delay, Echo, LE, TEC, Prompt to the same values and execute the alogoritms just as if the commands were typed in.
				Note: this means that the approprate D/A's and Port's are set to the same values.
		*/
#if VORT_BOARD
#ifndef SIMULATOR_ACTIVE



		// External Computer Control - AutoStart (active low)
		if((IO0PIN & PIN_AUTO_START) == 0) //Added parenthesis to condition DMS 11/6/09
		{
			isAutoStartActive = FALSE;
			currentLe = 0;	// 	Check status of the Autostart line if Low then do not set LE=1 even if this was the last setting.
		}
		else
 			isAutoStartActive = TRUE;


		warmUpInitTempIsFine=TRUE;	//Added to allow non-volatile pulse mode which calls LPHandler to change the DACs (LPHandler requires warmUpInitTempIsFine=TRUE)

		calHtrCntHandlerProcess(NO_PARSER);
#if 0
		dfblcHandlerProcess(NO_PARSER);
#endif
		calDfblcDacCntHandlerProcess(NO_PARSER);

		cHandlerProcess(NO_PARSER);

		//pulHandlerProcess requires isLaserEnableActive = TRUE which is set in parserInit() above
		if(calShadowRegs.nonVolPUL == TRUE) //Selectable by Mfr Cmd
		{
			if(sysCfgShadowRegs.pulseMode == TRUE) //Restore PUL to state as when powered down
			{
				pulMethod = PUL_AT_POWERUP;
				pulHandlerProcess(NO_PARSER, 1); //Uses drive current determined by calibration performed when user last entered PUL=1
			}
			else
			{
				pulHandlerProcess(NO_PARSER, 0);
			}
		}
		else
		{
			pulHandlerProcess(NO_PARSER, 0);
		}		

		warmUpInitTempIsFine=FALSE;

		monitorWarmUp();

		tecHandlerProcess(NO_PARSER);
		
		vobtsHandlerProcess(NO_PARSER);
		delayHandlerInit();
		//tecHandlerInit();

		if(warmUpInitTempIsFine == TRUE)
			leHandlerProcess(NO_PARSER);

		echoHandlerInit();

		// Enables External Power Control Input Port (1= External Control)
		processEpcMode();

		//Apply RF Injection setting from EEPROM
		rfInjHandlerProcess(NO_PARSER);

		// USB init must happen after eeprom and sysCfg inits
		USB_Init();                               // USB Initialization
    	USB_Connect(TRUE);                        // USB Connect
#endif
#endif

		wDogInit();
}

void initBoard(void)
{
	// PIN Assignment: setting IODIR to a 1 (output), 0 (Input)
	IODIR0 &= ~PIN_BOARD_ID_0;				// Board ID LSBit	(Input)
	IODIR0 &= ~PIN_BOARD_ID_1;				// Board ID 			(Input)
	IODIR0 &= ~PIN_BOARD_ID_2;				// Board ID MSBit	(Input)
	IODIR0 |= PIN_BOARD_ID_CTRL;			// Board ID output control (possibly wired to inputs P0.4 - P0.6)

	IODIR0 |= PIN_SCL0;		 // output

	boardIdInit();		 // Must be called after the Direction bit have been set for the board ID status and control

	if(getBoardID() == 1)
	{
		PIN_TE_ON 		= 0x00080000;		// P0.19
		PIN_I_MOD_ON 	= 0x00010000;		// P0.16
		PIN_MOD_ON 		= 0x00010000;		// P0.16
	}
	else
	{
		PIN_TE_ON 		= 0x00010000;		// P0.16
		PIN_I_MOD_ON 	= 0x00080000;		// P0.19
		PIN_MOD_ON		= 0x00080000;	 	// P0.19
	}

	IODIR0 &= ~PIN_EXT_INTERLOCK;			// INTERLOCK	P0.10	IN	External Interlock input

	IODIR0 &= ~PIN_LSR_EN;						// LASER ENABLE	P0.11	IN	External Laser ON-OFF

	IODIR0 &= ~PIN_AUTO_START;				// AUTOSTART	P0.12	IN	External Autostart on-off

	FIO0DIR |= PIN_DEBUG_STROBE;				// DEBUG STROBE	P0.13	OUT
	IODIR0 |= PIN_DEBUG_STROBE;				// DEBUG STROBE	P0.13	OUT

	IODIR0 |= PIN_LED_STATUS;					// LED STATUS STROBE P0.15	OUT

	IODIR0 |= PIN_TE_ON;							// TE ON	P0.16 or P0.19	OUT	Controls TE Cooler ON-OFF

	IODIR0 |= PIN_FAULT_INDC;					// FAULT INDICATOR	P0.17	OUT	External Fault indicator

	IODIR0 |= PIN_LSR_RDY;						// LASER READY	P0.18	OUT	External Laser Ready line

	IODIR0 |= PIN_I_MOD_ON;						// I MODULATION ON	P0.19	or P0.16 OUT	OUT	Turns External Modulation of the laser Current  ON-OFF

	IODIR0 |= PIN_LIGHT_MOD_ON;				// LIGHT MODULATION ON	P0.20 OUT	Turns External Modulation of the laser light ON-OFF

	IODIR0 |= PIN_LSR_ON;							// LASER OK	P0.21	OUT	Microprocessor control of laser light
	IO0CLR = PIN_LSR_ON;
	lsrOnStopWatch(LSR_STOP_WATCH, STOP);

	IODIR0 |= PIN_LSR_EMISSION_LED;				// LASER EMISSION	P0.22	OUT	Indicates Laser Emission likely


	IODIR0 &= ~PIN_COLD_BLOCK_TEMP;		// COLD BLOCK TEMP, (ADC0.1)	P0.28	ANALOG IN	Reads Cold Block Temperature
	IODIR0 &= ~PIN_BASE_PLATE_TEMP;		// BASE PLATE TEMP, (ADC0.2)	P0.29	ANALOG IN	Read Base Plate Temperature

	IODIR0 &= ~PIN_USB_RENUM;					// USB renumeration control, keep High-Z (input) until ready to enumerate


	initSw();
	powerUp = FALSE;
}

uchar getBoardID(void)
{
	return boardID;
}

// determine if this is the X1 board, or a newer board
// Note: X1 board has P0.4-P0.6 floating
// P0.7     P0.6     P0.5     P0.4
// h/l      same    same    same    rev 1
// h/l      same    same    0       rev 2
// h/l      same    0       same    rev3
// h/l      0    		same    same    rev4
// h/l      same    same    1       rev5
// h/l      same    1       same    rev6
// h/l      1    		same    same    rev7

void boardIdInit(void)
{
	uchar tmpBoardIdHi = 0xA5;
	uchar tmpBoardIdLow = 0xC3;

	IO0SET = PIN_BOARD_ID_CTRL;
	tmpBoardIdHi = (IO0PIN & (PIN_BOARD_ID_0 | PIN_BOARD_ID_1 | PIN_BOARD_ID_2)) >> 4;

	IO0CLR = PIN_BOARD_ID_CTRL;
	tmpBoardIdLow = (IO0PIN & (PIN_BOARD_ID_0 | PIN_BOARD_ID_1 | PIN_BOARD_ID_2)) >> 4;		 // FRG This line did not work correctly!

	if(tmpBoardIdHi == 0x7 && tmpBoardIdLow == 0x0)
		boardID = 1; 	// Revision 1
	else if(((tmpBoardIdHi & 0x6) == 0x06) && ((tmpBoardIdLow & 0x6) == 0) && ((tmpBoardIdHi & 0x01) == 0))
		boardID = 2;	// Revision 2
	else if(((tmpBoardIdHi & 0x5) == 0x05) && ((tmpBoardIdLow & 0x5) == 0) && ((tmpBoardIdHi & 0x02) == 0))
		boardID = 3;	// Revision 3
	else if(((tmpBoardIdHi & 0x3) == 0x03) && ((tmpBoardIdLow & 0x3) == 0) && ((tmpBoardIdHi & 0x04) == 0))
		boardID = 4;	// Revision 4
	else if(((tmpBoardIdHi & 0x6) == 0x06) && ((tmpBoardIdLow & 0x6) == 0) && ((tmpBoardIdHi & 0x01) == 1))
		boardID = 5;	// Revision 5
	else if(((tmpBoardIdHi & 0x5) == 0x05) && ((tmpBoardIdLow & 0x5) == 0) && ((tmpBoardIdHi & 0x02) == 1))
		boardID = 6;	// Revision 6
	else if(((tmpBoardIdHi & 0x3) == 0x03) && ((tmpBoardIdLow & 0x3) == 0) && ((tmpBoardIdHi & 0x04) == 1))
		boardID = 7;	// Revision 7
	else
		boardID = BOARD_ID_X1;
}

