/*----------------------------------------------------------------------------
 *      Name:    main.C
 *      Purpose: QD Laser Controller
 *
 *		Primary Author: David Sweeney, Vortran Laser Technology, 7/2012
 *
 *	This is the start of C code for the QD Laser Controller
 *  Vortran errors > 16 still apply for BrandX, when turning off the laser
 *
 *		Interrupts: Vector/Slot 0 is higher priority then vector 15.
 *			I2C 0 : 		I2C 0 - for EEPROM, A/D, D/A, Digital Pot			(Slot 1)
 *			Timer 0: 	Delay/Timeouts - software usage									(Slot 2)
 *			Timer 1: 	execTick 																				(Slot 3)
 *			UART 0: 	software update port and Vortran Protocol Port 	(Slot ?)
 *			USB: 			communication with Client GUI 									(Slot 4)
 *			UART 1: 	Vortran Protocol Port with Client GUI						(Slot 5)
 *			WDog : 		Watch Dog timer
 *			SWI : 		EEPROM, A/D, D/A, Digital Pot
 *
 *			vec1: I2C
 *			vec2: Timer0
 *			vec3: Timer1
 *			vec4: USB
  *			vec6: Uart0
 *			vec7: A/D 0
 *			vec8: Brown Out Detection
 *			vec5: Uart1

 *
 *
 *		Work Arounds:
 *			(HW) JTAG interface does not work when the USB cable is connected works fine with Keil Development board. Work around, disconnect
 *					USB, download using JTAG, connect USB cable.
 *					power down during saveSysCfg Defaults causes the eeprom to get destroyed, and initSysCfg!! is triggered
 *			i2c initial read of pots, to determine if I2C device is working, appears to fail on some boards and locks up I2C Bus
 *					extDualPotsModInit() hw detection error issue
 *			unable to set extDualPotsModInit using debug commands
 *			strol - hex value conversions
 *
 *		Known Bugs:
 *			*if I2c device fails on a read, an incorrect value is still returned and used by app code. Return a 0 value.
 *			Change DelayMs() to timer0.
 *			System Monitoring: Determine timing calls for super loop when device is idle. Document and adjust as needed
 *			*how should swerr errors determine the laser diode operation?
 *			DelayUs() has a call to DelayMs() which sould be removed and tested!!
 *
 *		Features not implemented
 *			*swerr should be able to use any transport, right now just comm0
 *			*how does the USB and serial cmd and response mechanism work for the client: how many "stacked" cmds, what time is required, ...
 *			*Dozen commands locks up system.
 *			*System Cfg modifications to eeprom "image version" are not automatically handled by software on boot.	Use "burn" eeprom command
 *			* instead of super loop using counters for timing, use timer with 10ms interval, and trigger features based on a more accurate mechanism
 *			*USB VID and PID
 *			Add Help
 *			Add sw errors(Partially done)
 *			Literals for array sizes need to be analyzed
 *			loop for ever need error conditions
 *			Search all "FRG" and add errors, and also timeouts for infinite loops
 *
 *		Enhancements:
 *			*swerr should be stored in eeprom for breadcrumbs, when pwr goes off, lose bread crumbs
 *			*Parsing should not occur with serial until a CR key is detected, or overflow
 *			Code Protect
 *			Watchdog timer
 *			Queues integrated into serial TX
 *			Add timeouts for serial wait forever code (checking reg status)
 *			Read/Write Delay period can be minimized with EEPROM from Vortran Board (Partially complete for writes)
 *			Comm serial FIFO in hw
 *			Add help to system config
 *			*USB response buffer slows down super loop for long responses (?h), use queues.
 *
 *		Testing:
 *			Brown out detection
 *			Wdog
 *			hours laser on
 *			Protocol: Vortran, BrandX
 *			Temperature errors, and diode on/off control
 *			Double buffer system cfg area (for laser hours)
 *			read of port0.4 and port0.5 port0.6, port0.7 output for board ID.
 *			*when diode temp fault is occurring, sending serial return/enter key causes diode temp fault to clear "laser on" and then generate fault to disable "laser"
 *
 *
 *		Notes:
 *			NXP 2144 - 128K of code space, 16K of RAM
 *			Keil Development board only has 256 Bytes of eeprom, which means the system cfg image is smaller then in production
 *			Queues are mutually exclusive, since they are used in the super loop, and the serial ISRs
 *			if comm1 is active, then the interlock on/off switch causes comm1 interrupt errors (RX_ERROR)
 *			HOW TO ANALYZE A DATA ABORT EXCEPTION: The instruction which caused the exception is at address R14, subtract 8	from R14
 *			HOW TO ANALYZE A PREFETCH ABORT EXCEPTION: The instruction which caused the exception is at address R8, subtract 8	from R8
 *			Change the CPU device type (2144 or 2148) depending on which board is being used.
 *			When using the internal A2D and the uVision3 with the	peripherals A/D dialog open, single stepping may clear register status prematurely for
 *				the A/D, since the uVision3 is "reading" and displaying A/D registers (which clears some status). Workaround: Close A/D dialog window when debugging
 *			Why would queue.c: VICIntEnClr = 1 << 6;       // Disable UART0 Interrupt
 *				Cause resets, when serial data is received (streamed) into comm0, comm0 rx ISR just tosses the bytes received.
 *
 * V2.0.1Y    1/22/14 Fixes bug in rilHandler() that returns the reply "?IL=1/0" twice before the prompt
 * V2.0.0Y    Production release for Stradus QD firmware. Has "Y" designation for QD
 * V1.0.15d   Power-up in C=1 mode with LC=ILC, then switch to C=0 mode after 15 seconds
 * V1.0.14d   Added condition to monitorDiodeOverCurrent() to require 5 occurences of passing threshold to set over-current fault
 *			  Changed default max DFB current to 200mA	 
 * V1.0.13d   Fixed MOC and ?MOC are not valid commands. Added them to command dispatch table.
 * V1.0.12d   Added MOC Max Over Current to cal image with commands to separate full scale current (MLC) from maximum over-current limit
 *				to avoid undesired over current faults when laser is calibrated with current DAC at 4095 so MLC=ILC. Small upword
 *				current drifts over MLC would cause over-current faults.
 * V1.0.11d   Modified lpHandlerProcess() for BrandX mode so lp may be changed with laser emission ON or OFF
 * V1.0.10d   Several changes to BrandX cmds- removed '?' from query responses, matched number printf formats correctly
 *			  Added calDfblcDacCntHandler, calDfblcDacCntHandlerProcess to set DFB DAC counts upon startup.
 * V1.0.9d    Fixed MHC cal cmd changes MLC value. 
 * V1.0.8d    Placed saveSysCfgDelta before serial reply for all cmds so cal software keeps synchronization
 * V1.0.7d    Added ?DFBLCS, CDACDFB,?CDACDFB for recalibration, Added help file entries, Changed default for ILC, MLC, DFBILC, DFBMLC
 * V1.0.6d    Increased Max Laser Current Default to 500mA (SOA)
 * V1.0.5d    Added Calibration commands for Heater Counts, DFB Current, and SOA Current
 * V1.0.4d 	  For Revision 2 boards only, Driver for Current Sense Switch to read DFB,SOA,Heater current sequentially
 * V1.0.3d    Disabled Diode Current Fault
 * V1.0.2d	  Added DFB Current DAC Counts "DFB" and "?DFB" cmds and calDfbHandlerProcess. 
 * V1.0.1d	  Fixed SinglePotLsr for SOA Light Calibration does not retain setting when cycling power. 
 * V1.0.0d    Added drivers/files for DFB, SOA, and Heater control devices: extDualPotsLsrDfb.c, extQuadD2a.c, extSinglePotLsr.c
 *			  Added Diagnostic R/W cmds and queries
 *			  
 *				
 *			   
 *			  
 * Stradus Debug builds since V3.0.0 firmware release:
 * V3.0.0d3   Adds spurious interrupt handler in vic.c
 * V3.0.0d2   TBD Adds I2C functionality to LCSP, LPSP, ?LCSP, ?LPSP cmds, queries to read write extDualPotsLsr part
 * V3.0.0d1   TBD RF Injection Mods
 * V3.0.0     1/3/12 Vortran-DMS  Adds HID USB interface, Adds ?OP4 query, Kicks WDOG in putString() for large blocking transfers
 *                                Adds #define USB_PORT_INIT to combine response and prompt instead of doing at least 2 separate transfers 
 *                                for each cmd/query, Adds MODE mfr cmd/qry, Adds support for MODE=1 presumably for 532nm product.
 * 			  Vortran-QD Laser Module Based on Stradus Code Version V3.0.0d3
 *----------------------------------------------------------------------------
 *
 *      Copyright (c) 2008 SoftwareFlair inc.
 *
 *
 *---------------------------------------------------------------------------*/

#include <LPC214X.H>                        /* LPC214x definitions */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "type.h"
#include "system.h"
#include "usb.h"
#include "usbcfg.h"
#include "usbhw.h"
#include "usbcore.h"
#include "i2c.h"
#include "string.h"
#include "queue.h"
#include "serial.h"
#include "sysCfg.h"
#include "errLog.h"
#include "parser.h"


// externs
extern void parser(char myPort, char *myBuf, Q_STREAM stream, uchar myMsgSize);
extern void logStopWatch12Mins(void);
extern void isCmdInActiveFor20Secs(void);
extern void isInitialCurrPowerUpLapsed(void);
extern void reset(void);
extern void leHandlerProcess(ushort myMode);
extern void processEpcMode(void);
extern uchar getBoardID(void);
extern char pulMethod;
extern char powerUp;
extern void pulHandlerProcess(ushort myMode, ushort action);
extern void rfInjHandlerProcess(ushort myMode);

// prototypes
void processEvents(uchar myPort);
void monitorCdrh(void);
void monitorStandBy(void);
void monitorWarmUp(void);
void monitorInterLockOpen(void);
void monitorDiodeOverCurrent(void);
void monitorDiodeTempFault(void);
void monitorBasePlateTempfault(void);
void lsrOffDueToFault(void);
void statusLedToggle(void);		// Status LED is toggled on and off as required

short isInterLockOpen(void);

// Globals
char tmpBuf[MAX_CMD_SIZE];

short warmUpInitTempIsFine = FALSE;	// diode temperature during initial warmup period
short wasHere = 0;
char statLedFunction = STATUS_LED_RUNNING;

/* Main Program */
// SuperLoop  the super loop, often also referred to as the background executive,
// shall loop forever and perform various operations based on system timing or
// events. A power down condition shall cleanly inhibit any super loop typical
// operations. The typical operations shall consist of parsing commands,
// performing power down, and monitoring system state/sensors.
int main (void)
{
	ushort loopCnt = 0;
	ulong myOneSecTicks = 0;
	ulong myOneSecTicksStatLed = 0;
	ulong myDebugTicksStatLed = 0;
	static uchar myTurnToRun = 0;
	//char myStr1[] = "h\r";
	//char myStr2[] = "?c\r";
	//char myStr3[] = "?ls\r";
	//char myStr4[] = "?obt\r";

	initBoard();

		putString(COMM_0, "\n\n*******************\n");
		putString(COMM_0, "Stradus\n");
		putString(COMM_0, "(C) 2008 Vortran\n");
		sprintf(tmpBuf, "Sw: V%s, Hw: V%d\n", SW_VERSION, getBoardID());
		putString(COMM_0, tmpBuf);
		putString(COMM_0, "*******************\n");
		displayPrompt(COMM_0);

#if ACTIVATE_COMM1
		putString(COMM_1, "\n\n*******************\n");
		putString(COMM_1, "Stradus\n");
		putString(COMM_1, "(C) 2008 Vortran\n");
		sprintf(tmpBuf, "Sw: V%s, Hw: V%d\n", SW_VERSION, getBoardID());
		putString(COMM_1, tmpBuf);
		putString(COMM_1, "*******************\n");
		displayPrompt(COMM_1);
#endif

		if(prevWDogOccurred == TRUE)	// previous watch dog reset occurred?
		{
			putString(COMM_0, "\nCPU reset by Watch Dog!\n");
			displayPrompt(COMM_0);
		}

		wDogKick();

		delayMs(1000); // let system stabalize

		brownOutInit(); //Wait at least 1 full second after power-up to avoid premature trigger of BOD interrupt

  	while (1)      // Loop forever - super loop
  	{
			if(statLedFunction==STATUS_LED_RUNNING)
			{
				if(oneSecTicks != myOneSecTicksStatLed)
				{
					statusLedToggle();		// Status LED is toggled on and off as required
					myOneSecTicksStatLed = oneSecTicks;
				}
			}
			else
				if(statLedFunction==STATUS_LED_DEBUG)
				{
					if(debugTicks != myDebugTicksStatLed)
					{
						statusLedToggle();
						myDebugTicksStatLed = debugTicks;
					}
				}

			loopCnt++;

			isInitialCurrPowerUpLapsed();


//#if TEMP_FRG
			if((loopCnt % 0xF000) == 0)					// Monitor Inputs
			{
				logStopWatch12Mins();	// update "laser on" time?

				// the vortran protocol requires that the full command string be entered in a specific period, otherwise the command is tossed.
				// this is not a requirement for BrandX protocol
				if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
					isCmdInActiveFor20Secs();

#ifndef SIMULATOR_ACTIVE
				coldBlockTempGet();						// Get Cold Block Temperature
#endif
				
			}
//#endif // TEMP_FRG


			//writeQ(SER0_Q, IN_Q, myStr1, 2);	// FRG DEBUG!!!
			//writeQ(SER0_Q, IN_Q, myStr2, 3);	// FRG DEBUG!!!
			//writeQ(SER0_Q, IN_Q, myStr3, 4);	// FRG DEBUG!!!
			//writeQ(SER0_Q, IN_Q, myStr4, 5);	// FRG DEBUG!!!

			// Parse "Vortran " messages
			processEvents(USB_PORT);	// process events from USB out commands, Serial, ...
			processEvents(COMM_0);	// process events from USB out commands, Serial, ...
#if ACTIVATE_COMM1
			processEvents(COMM_1);	// process events from USB out commands, Serial, ...
#endif

			wDogKick();				// Kick Watch Dog


//#if TEMP_FRG
#ifndef SIMULATOR_ACTIVE
			if((IO0PIN & PIN_LSR_ON) == 0 && (currentLe == 1))	 // if the laser diode is off due to temp fault, see if the temp fault has cleared so we can turn laser on
				if(isErrLogStatGrtr16() == FALSE && (warmUpInitTempIsFine == TRUE))
					leHandlerProcess(NO_PARSER);
#endif

			if(monCdrhHandlerMode == TRUE)
				monitorCdrh();

#ifndef SIMULATOR_ACTIVE

			tecHandlerProcess(NO_PARSER);
#endif

			if(monStdbHandlerMode == TRUE)
				monitorStandBy();

			if(monIntlHandlerMode == TRUE && (warmUpInitTempIsFine == TRUE))
				monitorInterLockOpen();

			// every second monitor one of the temperatures
			if(oneSecTicks != myOneSecTicks)
			{
				myOneSecTicks = (oneSecTicks % 4);

				if(monDiCuHandlerMode == TRUE && myTurnToRun == 0)
					monitorDiodeOverCurrent();

				if(myTurnToRun == 1)
					monitorWarmUp();

				if(monDTmpHandlerMode == TRUE && myTurnToRun == 2)
					if(warmUpInitTempIsFine == TRUE)
						monitorDiodeTempFault();

				if(monBPltHandlerMode == TRUE && myTurnToRun == 3)
					monitorBasePlateTempfault();

				if(++myTurnToRun > 3)
					myTurnToRun = 0;
			}

			// did the I2C ISR report an error?
			if(isrI2cErr == TRUE)
			{
				errLogFlagSet(ERR_I2C_ERROR);	// yes, store the error
				isrI2cErr = FALSE;
			}

			isErrLogStatGrtr16();

		 	// did a brown out occur?
			if(didBrownOutOccur() == TRUE)
			{
				putString(COMM_0, "*Rst!");					// FRG TODO: Debug Text!!!
				delayMs(3);

				reset();
			}


//#endif //TEMP_FRG

  }
}

// events that are from USB Out commands
// that cannot be handled in the USB interrupts due to I2C
// interrupts being called
void processEvents(uchar myPort)
{
	//uchar mySer0BytesIn;
	static uchar mySer0Data[MAX_Q_BUF_SIZE];
	uchar mySer1BytesIn;
	static uchar mySer1Data[MAX_Q_BUF_SIZE];
	uchar myUsbBytesIn;
	static uchar myUsbData[MAX_Q_BUF_SIZE];
	uchar myCnt = 0;
	uchar myPos;
	char parse=0x00;
	static uchar Ser0Index;

	if(myPort == COMM_0)
	{
#if 0
		if(charsToProcessRx0 != 0)
		{
			// see if we have a CR Key
			for(myPos = 0; myPos < 	MAX_CMD_SIZE; myPos++)
			{
				mySer0Data[myPos] = rx0Buf[myPos];

				if(rx0Buf[myPos] == CR_KEY)
				{
					comm0RxInPos = 0;
					rx0Buf[0] = 0;

					mySer0BytesIn = myPos + 1;

					VICIntEnable = 1 << 6;       /* Enable UART0 Interrupt */
					parser(COMM_0, (char *)mySer0Data, SER0_Q, mySer0BytesIn);
				}
			}
		}
#endif

#if 0 //Firmware Versions 2.1.21 and earlier

		if(mySerial0Pos)
		{
			allInterruptsOff();
			writeQ(SER0_Q, IN_Q, mySerial0In, mySerial0Pos);
		    mySerial0Pos = 0;
			allInterruptsOn();

			wasHere = 1;
			mySer0BytesIn = qFullIndicator(SER0_Q, IN_Q);

			if(mySer0BytesIn)
			{
				wasHere = 2;
				readQ(SER0_Q, IN_Q, mySer0Data, mySer0BytesIn);

				//flushQ(SER0_Q, IN_Q, mySer0BytesIn);	 // Toss the character

			 	wasHere = 3;
				parser(COMM_0, (char *)mySer0Data, SER0_Q, mySer0BytesIn);

				wasHere = 4;
				//displayPrompt(COMM_0);
				//putString(COMM_0, "\n\r");

				while(myCnt < mySer0BytesIn)
					mySer0Data[myCnt++] = 0;
			}
		}
#endif
		//&&&&&&&&& Version 2.1.22 and later Vortran DMS 3/5/10 &&&&&&&&&&
		//This version does not use input queue to accept other commands while the
		//current command is being processed

		if(mySerial0Pos>0) //New bytes in on UART0
		{
			VICIntEnClr = 1 << 6;       /* Disable UART0 Interrupt Make access to mySer0Data Mutually Exclusive*/
			for(myPos = 0; myPos < 	mySerial0Pos; myPos++)
			{
				mySer0Data[Ser0Index++] = mySerial0In[myPos];
				if(mySerial0In[myPos]==0x0D)
				{
					parse=0x01;
					break;
				}
			}
			mySerial0Pos=0;
			VICIntEnable = 1 << 6;       /* Enable UART0 Interrupt */
		}

		if(parse==0x01)
		{
			parse=0x00;
			parser(COMM_0, (char *)mySer0Data, SER0_Q, Ser0Index);
			for(myPos=0;myPos < Ser0Index;myPos++)
			{
				mySer0Data[myPos]=0x00;
			}
			Ser0Index=0;
		}

		//&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&


	}
	else if(myPort == COMM_1)
	{
		mySer1BytesIn = qFullIndicator(SER1_Q, IN_Q);

		if(mySer1BytesIn)
		{
			readQ(SER1_Q, IN_Q, mySer1Data, mySer1BytesIn);

			parser(COMM_1, (char *)mySer1Data, SER1_Q, mySer1BytesIn);

			while(myCnt < mySer1BytesIn)
				mySer1Data[myCnt++] = 0;
		}
	}
	else if(myPort == USB_PORT)
	{
		myUsbBytesIn = qFullIndicator(USB_Q, IN_Q);

		if(myUsbBytesIn)
		{		    
			readQ(USB_Q, IN_Q, myUsbData, myUsbBytesIn);

			parser(USB_PORT, (char *)myUsbData, USB_Q, myUsbBytesIn);

			while(myCnt < myUsbBytesIn)
				myUsbData[myCnt++] = 0;

			//myUsbBytesIn = strlen(myUsbOutBuf);

			//parser(USB_PORT, (char *)myUsbOutBuf, USB_Q, myUsbBytesIn);

			//while(myCnt < myUsbBytesIn)
				//myUsbOutBuf[myCnt++] = 0;
		}
	}
}

void monitorCdrh(void)
{
	short myTecState = FALSE;

	if(cdrhTriggersLsr == TRUE)
	{
	 	// has 5 seconds elapsed?
		if(cdrhStartTime > 500)	// timer 1 increments cdrhStartTime every 10 ms
		{
			if(isColdBlockTempOk() == TRUE)
			{
#if VORT_BOARD
				if(getBoardID() == BOARD_ID_X1)
				{
		 			if(IO0PIN & PIN_TE_ON)
		 				myTecState = TRUE;
				}
				else if(getBoardID() == 1)
				{
		 			if((IO0PIN & PIN_TE_ON) == 0) 	// Bit is flipped for Rev 1
		 				myTecState = TRUE;
				}

				if(myTecState == TRUE)
				{
					//if(turn off faults ==0)
	 				if(isAutoStartActive == TRUE)
					{
#endif
						if(isLsrEnableActive == TRUE)
						{
							if(isCalImageGood == TRUE)
							{
								IO0SET = PIN_LSR_EMISSION_LED; // LED	ON
								IO0SET = PIN_LSR_ON; // Activate the Laser
								lsrOnStopWatch(LSR_STOP_WATCH, START);
							}
						}
#if VORT_BOARD
					}
				}
#endif
			}

			cdrhTriggersLsr = FALSE;
			cdrhStartTime = 0;
		}
	}
}

// Set this state with any of the following conditions:  LE=0, Interlock Open
void monitorStandBy(void)
{
	if(((isLsrEnableActive == FALSE) || (isInterLockOpen() == TRUE)) && warmUpInitTempIsFine == TRUE)		 // laser off or interlock open and we are not in warm up?
	{
		errLogFlagSet(ERR_STANDBY);	// yes, we are in standby
		if(isErrLogStatGrtr16()==FALSE) //Check for other simultaneous faults before asserting LaserReady
		{
			IO0SET = PIN_LSR_RDY; //Assert Laser Ready Line DMS 11/6/09
		}
	}
	else
		errLogFlagClear(ERR_STANDBY);
}

// Warmup State:  ?OBT has a delta greater than x from ?OBTS.
void monitorWarmUp(void)
{
 	volatile short myTicks = 0;
	ushort myVal = 0;
	float myFloat = 0;

	if(warmUpInitTempIsFine == FALSE)
	{
		if(isErrLogStatGrtr32() == FALSE)
		{
			//?OBT has a delta greater than 2 from ?OBTS.
			// get the Cold Block Value
			a2dChnlStart(A2D_CHNL_COLD_BLOCK);

		 	timer0Init(MAX_TIMEOUT_A2D);
			timer0Start();

			while(isA2d0ConvDone == FALSE)
			{
				if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
				{
					swError(__FILE__, __LINE__);
				 	break;
				}
			}

			timer0Stop();


			myVal = a2dChnlRead();	// get ?OBT
			myFloat = (myVal * 0.02933) + 10;

			if((myFloat > (sysCfgShadowRegs.vobts + calShadowRegs.obtDeviation)) || (myFloat < (sysCfgShadowRegs.vobts - calShadowRegs.obtDeviation)))		// ?OBT has a delta greater than x from ?OBTS.
			{
				errLogFlagSet(ERR_WARMUP);

				lsrOffDueToFault();		// the laser must be turned off
			}
			else
			{
				warmUpInitTempIsFine = TRUE;		// diode is in valid range
				errLogFlagClear(ERR_WARMUP); //Clear Error Flag upon leaving warmup Vortran-DMS 11/20/09
				leHandlerProcess(NO_PARSER);
			}
		}
	}
}

// monitor the external interlock switch. If the interlock is open then disable the laser,
// if the interlock is closed then enable the laser
void monitorInterLockOpen(void)
{
	static short ALLOW_ON = TRUE; //Modifications using ALLOW_ON/ALLOW_OFF Vortran-DMS 11/19/09
	static short ALLOW_OFF = TRUE;

	if(isInterLockOpen() == TRUE)
	{

		if(ALLOW_OFF==TRUE)
		{
			lsrOffDueToFault();		// the laser must be turned off

			errLogFlagSet(ERR_INTERLOCK_OPEN);
			ALLOW_OFF=FALSE;
			ALLOW_ON=TRUE;
		}
	}
	else
	{
		if(ALLOW_ON==TRUE)
		{
			leHandlerProcess(NO_PARSER);
			ALLOW_ON=FALSE;
			ALLOW_OFF=TRUE;
		}
	}

	/* Prior to V2.1.15, does not detect interlock open until interlock is first detected closed Vortran-DMS 11/19/09
	static short myDoOncePerToggle = TRUE;

	if(isInterLockOpen() == TRUE)
	{

		if(myDoOncePerToggle == FALSE)
		{
			lsrOffDueToFault();		// the laser must be turned off

			errLogFlagSet(ERR_INTERLOCK_OPEN);

		 	myDoOncePerToggle = TRUE;
		}
	}
	else
	{
		if(myDoOncePerToggle == TRUE)
		{
			leHandlerProcess(NO_PARSER);
			myDoOncePerToggle = FALSE;
		}
	}
	*/
}

//If the 5 second cdrh delay is active, it should be applied when the interlock is opened and closed again.
short isInterLockOpen(void)
{
	if((IO0PIN & PIN_EXT_INTERLOCK) == 0)		 // interlock open?
	{
		// turn off laser
		return TRUE;	// yes
	}
	else
	{
		// if laser is not on, turn on laser, and apply cdrh
		errLogFlagClear(ERR_INTERLOCK_OPEN);
		//errLogFlagClear(ERR_STANDBY);
		return FALSE;
	}
}

// Set fault when ?LC is greater than the maximum laser current stored in the EEPROM.
void monitorDiodeOverCurrent(void)
{
#if VORT_BOARD
	float dfbLsrCur;
	float soaLsrCur;
	float htrLsrCur;
	static short ocCount=0;

	getAllExtA2dRegs();
	soaLsrCur = extA2dCurrentValues[EXT_A2D_SOA_CURR] * 0.1224;
	dfbLsrCur = extA2dCurrentValues[EXT_A2D_DFB_CURR] * 0.1224;
	htrLsrCur = extA2dCurrentValues[EXT_A2D_HEATER_CURR] * 0.1224;

	if(ocCount>4)
	{
		  ocCount=0;
		  errLogFlagSet(ERR_DIODE_OVER_CURRENT);
	}
	else
	{
		if(soaLsrCur > calShadowRegs.maxOverCurr || dfbLsrCur > calShadowRegs.maxCurrDfb || htrLsrCur > calShadowRegs.maxCurrHtr)	
			ocCount++;
		else
			ocCount=0;	
	}
	
#if 0
	if(calShadowRegs.endOfLifeEnable == TRUE) //Monitor End of Life
	{
		if(curLsrCur > 1.2 * calShadowRegs.initLsrCur)
		{
			errLogFlagSet(ERR_END_OF_LIFE);
		}
	}
#endif

#endif
}

// Set fault when ?OBT delta is 2 greater than ?OBTS.  No laser emission when fault is active
void monitorDiodeTempFault(void)
{
#if VORT_BOARD
	ushort myVal;
	float myFloat = 0;
	volatile short myTicks = 0;

	// get the Cold Block Value
	a2dChnlStart(A2D_CHNL_COLD_BLOCK);

 	timer0Init(MAX_TIMEOUT_A2D);
	timer0Start();

	while(isA2d0ConvDone == FALSE)
	{
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			swError(__FILE__, __LINE__);
		 	break;
		}
	}

	timer0Stop();

	myVal = a2dChnlRead();
	myFloat = (myVal * 0.02933) + 10;


	if(myFloat < (sysCfgShadowRegs.vobts - calShadowRegs.obtDeviation) || myFloat > (sysCfgShadowRegs.vobts + calShadowRegs.obtDeviation))
	{
		if(warmUpInitTempIsFine == TRUE) //Vortran DMS 2/4/10 V2.1.17 Allow ERR_WARMUP handler to catch this instead until unit comes to proper temperature after powerup
		{
			errLogFlagSet(ERR_DIODE_TEMP_FAULT);
		}
	}
	else
	{
		errLogFlagClear(ERR_DIODE_TEMP_FAULT);
	}

#endif
}

// Set fault when ?BPT is greater than 50.
void monitorBasePlateTempfault(void)
{

#if VORT_BOARD
	float myBaseplatetemp;

	myBaseplatetemp = basePlateTempGet();

	if(myBaseplatetemp > 50)
		errLogFlagSet(ERR_BASE_PLATE_TEMP_FAULT);
	else
		errLogFlagClear(ERR_BASE_PLATE_TEMP_FAULT);


#endif
}

void sendStatusToGUI(char * myText)
{

}

// warmup or interlock can cause the laser to be forced off
void lsrOffDueToFault(void)
{
			// make sure the laser is off, or tuned off
			if(IO0PIN & PIN_LSR_ON)
				IO0CLR = PIN_LSR_ON; 	// turn laser off

			if(IO0PIN & PIN_LSR_RDY) ///DMS 11/6/09
				IO0CLR = PIN_LSR_RDY; //Deassert Laser Ready Line

			IO0CLR = PIN_LSR_EMISSION_LED; // LED	for Laser Emission is turned off
			isLsrEnableActive = FALSE;

			lsrOnStopWatch(LSR_STOP_WATCH, STOP);
}

void statusLedToggle(void)		// Status LED is toggled on and off as required
{
	if(IO0PIN & PIN_LED_STATUS)
		IO0CLR = PIN_LED_STATUS;
	else
		IO0SET = PIN_LED_STATUS;
}

// software reset
void reset (void)
{
	((void (*) (void)) 0x000000) ();
}





