/*----------------------------------------------------------------------------
 *      Name:    parser.c
 *      Purpose: interpret commands
 *
 *	Parser  shall parse commands from serial and or USB. The command shall
 *	perform an operation and or return status.
 *
 *	High Priority Commands: Laser Current, Laser Power, and Laser On/Off
 *	Synchronization numbers
 *----------------------------------------------------------------------------
 *
 *
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#include <LPC214X.H>                        /* LPC214x definitions */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "type.h"
#include "system.h"
#include "sysCfg.h"
#include "queue.h"
#include "serial.h"
#include "i2c.h"
#include "errLog.h"
#include "extDualPots.h"
#include "parser.h"
#include "extEeprom.h"

// Commands
// text for commands, Maximum Arguments, and function handler
const PROTOCOL_CMDS vortCmds[] = {
	{"C", 			1,	cHandler},
	{"DELAY", 	    1,	delayHandler},
//	{"DFBLC",       1,  dfblcHandler},
	{"ECHO", 		1,	echoHandler},
	{"EPC", 		1,	epcHandler},
	{"LC", 			1,	lcHandler},
	{"LE", 			1,	leHandler},
	{"LP", 			1,	lpHandler},
	{"PP", 			1,	ppHandler},
	{"PROMPT", 	    1,	promptHandler},
	{"PROTO", 	    1,	protoHandler},
	{"PUL", 		1,	pulHandler},
	{"TEC", 		1,	tecHandler},
	{"VOBTS", 	    1,	vobtsHandler},
	{"VORT", 		0,	vortHandler},

	// queries
	{"?BPT", 		0,	rBptHandler},
	{"?C", 			0,	rCHandler},
	{"?CC", 		0,	rCcHandler},
	{"?DELAY", 	    0,	rDelayHandler},
	{"?DFBLC",      0,  rDfblcHandler},
//	{"?DFBLCS",     0,  rDfblcsHandler},
	{"?ECHO", 	    0,	rEchoHandler},
	{"?EPC", 		0,	rEpcHandler},
	{"?FC", 		0,	rFcHandler},
	{"?FD", 		0,	rFdHandler},
	{"?FP", 		0,	rFpHandler},
	{"?FV", 		0,	rFvHandler},
	{"?H", 			0,	rHelpHandler},
	{"?HLC",        0,  rHlcHandler},
	{"?IL", 		0,	rIlHandler},
	{"?LC", 		0,	rLcHandler},
	{"?LCS", 		0,	rLcsHandler},
	{"?LE", 		0,	rLeHandler},
	{"?LH", 		0,	rLhHandler},
	{"?LI", 		0,	rLiHandler},
	{"?LP", 		0,	rLpHandler},
	{"?LPS", 		0,	rLpsHandler},
	{"?LS", 		0,	rLsHandler},
	{"?LW", 		0,	rLwHandler},
	{"?MAXP", 	    0,	rMaxpHandler},
	{"?OBT", 		0,	rObtHandler},
	{"?OBTS", 	    0,	rObtsHandler},
	{"?OP1",        0,  rOpStat1Handler},
	{"?OP2",        0,  rOpStat2Handler},
	{"?OP3",        0,  rOpStat3Handler},
	{"?OP4",        0,  rOpStat4Handler},
	{"?OP5",        0,  rOpStat5Handler},
	{"?PP", 		0,	rPpHandler},
	{"?PUL", 		0,	rPulHandler},
	{"?RP", 		0,	rRpHandler},
	{"?TEC", 		0,	rTecHandler},

	// Calibration commands
	{"CDAC", 		1,	calDacHandler},
	{"CDACDFB",     1,  calDacDfbHandler},
	{"CV", 			1,	calCalVerHandler},
	{"DFBDAC",      1,  calDfblcDacCntHandler},
	{"DFBLCSD",     1,  calDfblcsdHandler},
	{"DFBLCSP",		1,  calDfblcspHandler},
	{"MPL", 		1,	calMaxPwrLsrHandler},
	{"HEAT",        1,  calHtrCntHandler},
	{"HSD",         1,  calHtrSetDacHandler},
	{"ILC", 		1,	calInitLsrCurHandler},
	{"ILCDFB",		1,  calInitDfbCurHandler},
	{"NLP", 		1,	calNomLsrPwrHandler},
	{"MLC", 		1,	calMaxLsrCurHandler},
	{"MLCDFB",      1,  calMaxLsrCurDfbHandler},
	{"MHC",         1,  calMaxHtrCurHandler},
	{"MOC",         1,  calMaxOverCurHandler},
	{"NWL", 		1,	calNomWaveLenHandler},
	{"MWL", 		1,	calMeasWaveLenHandler},
	{"LPSD",        1,  calLsrPwrSetDacHandler},
	{"LPSP", 		1,	calLsrPwrSetPotHandler},
	{"LCSD",        1,  calLsrCurSetDacHandler},
	{"LCSP", 		1,	calLsrCurSetPotHandler},
	{"LPN", 		1,	calLsrPartNumHandler},
	{"LSN", 		1,	calLsrSerialNumHandler},
	{"LBS", 		1,	calLsrBeamShapeHandler},
	{"OBTD", 		1,	calObtDeviationHandler},
	{"LCSF",        1,  calLcScaleFactorHandler},
	{"EPCSF",       1,  calEpcScaleFactorHandler},
	{"EPCSF2",       1,  calEpcScaleFactor2Handler},
	{"LPOA",        1,  calLpoaHandler},
	{"EOL",         1,  calEOLHandler},
	{"EOLFL",       1,  calEOL_FaultHandler},
	{"NVPUL",       1,  calNVPULHandler},
	{"EPCSF2",      1,  calEpcScaleFactor2Handler},
	{"MODE",        1,  calModeHandler},
	{"RFINJ",        1,  calRfInjHandler},


	// Calibration command responses
	{"?CDAC", 	    0,	rcalDacHandler},
	{"?CDACDFB",    0,  rcalDacDfbHandler},
	{"?CV", 		0,	rcalCalVersionHandler},
	{"?DFBDAC",      0, rcalDfblcDacCntHandler},
	{"?DFBLCSP",     0,  rcalDfblcspHandler},
	{"?HC", 		0,	rcalHelpHandler},
	{"?MPL", 		0,	rcalMaxPwrLsrHandler},
	{"?HEAT",       0,  rcalHtrCntHandler},
	{"?ILC", 		0,	rcalInitLsrCurHandler},
	{"?ILCDFB",     0,  rcalInitDfbCurHandler},
	{"?NLP", 		0,	rcalNomLsrPwrHandler},
	{"?MLC", 		0,	rcalMaxLsrCurHandler},
	{"?MLCDFB", 	0,	rcalMaxLsrCurDfbHandler},
	{"?MHC",        0,  rcalMaxHtrCurHandler},
	{"?MOC",        0,  rcalMaxOverCurHandler},
	{"?NWL", 		0,	rcalNomWaveLenHandler},
	{"?MWL", 		0,	rcalMeasWaveLenHandler},
	{"?LPSP", 	    0,	rcalLsrPwrSetPotHandler},
	{"?LCSP", 	    0,	rcalLsrCurSetPotHandler},
	{"?LPN", 		0,	rcalLsrPartNumHandler},
	{"?LSN", 		0,	rcalLsrSerialNumHandler},
	{"?LBS", 		0,	rcalLsrBeamShapeHandler},
	{"?OBTD", 	    0,	rcalObtDeviationHandler},
	{"?LCSF",       0,  rcalLcScaleFactorHandler},
	{"?EPCSF",      0,  rcalEpcScaleFactorHandler},
	{"?EPCSF2",      0,  rcalEpcScaleFactor2Handler},
	{"?LPOA",       0,  rcalLpoaHandler},
	{"?EOL",        0,  rcalEOLHandler},
	{"?EOLFL",      0,  rcalEOL_FaultHandler},
	{"?NVPUL",       0,  rcalNVPULHandler},
	{"?EPCSF2",       0,  rcalEpcScaleFactor2Handler},
	{"?MODE",       0,  rcalModeHandler},
	{"?RFINJ",       0, rcalRfInjHandler},

	// Diagnostic commands
	{"R", 			1,	rHandler},	// read hardware device registers
	{"W", 			2,	wHandler},	// write hardware devices registers
	{"SP", 			1,	spHandler},	// set hardware Port 0 Pin
	{"LH", 			1,	lhHandler},	// set Laser Hours
	{"CP", 			1,	cpHandler},	// clear hardware Port 0 Pin
	{"GP", 			0,	gpHandler},	// get hardware Port Pins
	{"DSYSCFG",     0,	dSysCfgHandler},	// display system config
	{"FORCEVER",    1,  forceVerHandler}, // force version of syscfg and cal EEPROM contents, Use carefully!!!
	{"CPSWERR",     0,	cpSwErrHandler},	// clear previous software errors	in eeprom
	{"DPSWERR",     0,	dpSwErrHandler},	// display previous software errors	in eeprom
	//{"MAXP", 		1,	maxpHandler},	// set Max power
	{"MCDRH", 	    1,	monCdrhHandler},	// enable/disable monitor laser conditions: CDRH
	{"MSTDB", 	    1,	monStdbHandler},	// enable/disable monitor laser conditions: Stand By
	{"MINTL", 	    1,	monIntlHandler},	// enable/disable monitor laser conditions: Interlock open
	{"MDICU", 	    1,	monDiCuHandler},	// enable/disable monitor laser conditions: Diode Current
	{"MDITM", 	    1,	monDTmpHandler},	// enable/disable monitor laser conditions: Diode Temp
	{"MBPLT", 	    1,	monBPltHandler},	// enable/disable monitor laser conditions: Base Plate Temp
	{"BURN", 		0,	burnHandler},			// burn defaults into eeprom: system config, and calibration
	{"TECMODE",     1,	tecModeHandler},	// is there TECooler hardware? 1: yes, 0: no
	{"PSA", 		1,	psaHandler},			// Set/Clear the Output pins: pulse mode  D0, as well as D1, D2, D3
	{"SWERR", 	    0,	swErrHandler},		// display software errors
	{"WDOG", 		0,	wDogOffHandler},	// write hardware devices registers disable kicking the dog, watchdog shall trigger
	{"?HD", 		0,	hdHandler},	// set Laser Hours
//	{"?PSA", 		0,	rPsaHandler},			// get the Output pins: pulse mode  D0, as well as D1, D2, D3
	{"?WDOG", 		0,	rWDogHandler},	// read status of watch dog reset previously occured

	{"",				0,	0}					// DO NOT MOVE/REMOVE!!	END OF COMMANDS!!

};

const char helpTableVort[][MAX_CMD_RSP_SIZE] = {
	"C=[0/1] - Set laser Drive Control Mode: Power/Current", //53 + null
	"CFC - Clear Stored Fault Codes",  //30 + null
	"DELAY=[0/1] - Set 5 Second CDRH Delay: Off/on",
	"ECHO=[0/1] - Toggle Terminal Echo: Off/On",
	"EPC=[0/1] - Set External Power Control Off/On",
	"LC=###.# - Set Laser Current: mA",
	"LE=[0/1] - Toggle Laser Emission Off/On",
	"LP=###.# - Set Laser Power: mW",
	"PROMPT=[0/1] - Toggle Terminal Prompt: Off/On",
	//"PROTO=[0/1] - Set Protocol: Vortran/BrandX",
	"RFC - Recall Stored Fault Codes",
	"PP=[0-1000] - Set Peak Pulse Power in Mw",
	"PUL=[0/1] - Set Pulse Mode",
	"TEC=[0/1] - Toggle TE Cooler: Off/On",
	//"VOBTS=[15-35] - Optical Block Temperature in Celsius",			 // Backdoor command!!!	Not for end users
	"\n",

  	"?BPT - Request Base Plate Temperature",
	"?C -  Request Laser Drive Control Status",
	"?CC - Request Computer Control Status",
	"?DELAY - Request 5 Second CDRH Delay Status",
	"?ECHO - Request Echo Status",
	"?EPC - Request External Power Status",
	"?FC - Request Fault Code, and clear faults",
	"?FD - Request Fault Description",
	"?FP - Request Protocol Version",
	"?FV - Request Firmware Version",
	"?H - Request Help File",
	"?IL - Request Interlock Status",
	"?LC - Request Laser Current",
	"?LCS - Request Laser Current Settings",
	"?LE - Request Laser Emission Status",
	"?LH - Request Laser Operating Hours",
	"?LI - Request Laser Identification",
	"?LP - Request Laser Power mW",
	"?LPS - Request Laser Power Setting",
	"?LS - Request Laser Status",
	"?LW - Request Laser Wavelength",
	"?MAXP - Request Maximum Laser Power",
	"?OBT - Request Optical Block Temperature",
	"?OBTS - Request Optical Block Temperature Setting",
	"?RP - Request Rated Laser Power",
	"?PP - Request Peak Pulse Power",
	"?PUL - Request Pulse Mode",
	"?TEC - Request TE Cooler/Heater Status",

	""		// END OF TABLE MARKER!! DO NOT REMOVE!!
};

// Commands
// text for commands, Maximum Arguments, and function handler
const PROTOCOL_CMDS brandxCmds[] = {
	{">", 			1,	promptHandler},
	{"ANA", 		1,	cHandler},
//#if 0
	//{"CFC", 		0,	cfcHandler},
	{"CDRH", 		1,	delayHandler},
	{"DST", 		1,	vobtsHandler},
	{"E", 			1,	echoHandler},
	{"EXT", 		1,	epcHandler},
	//{"LC", 			1,	lcHandler},
	{"L", 			1,	leHandler},
	{"OP",          1,  opHandler},
	{"P", 			1,	lpHandler},
	{"PROTO", 	1,	protoHandler},
	//{"RFC", 		0,	rfcHandler},
	{"T", 			1,	tecHandler},
	//{"SS", 			1,	ssHandler},
	{"VORT", 		0,	vortHandler},

	// queries
	{"?ANA", 		0,	rCHandler},
	{"?BT", 		0,	rBptHandler},
	{"?C", 			0,	rLcHandler},
	{"?CDRH", 	    0,	rDelayHandler},
	{"?DT", 		0,	rObtHandler},
	{"?DST", 		0,	rObtsHandler},
	{"?E",          0,  rEchoHandler},
	{"?EXT", 		0,	rEpcHandler},
	{"?F", 			0,	rFcHandler},
	{"?FF",			0,	rFfHandler},
	{"?H", 			0,	rHelpHandler},
	{"?HH", 		0,	rLhHandler},
	{"?HID", 		0,	rLiHandler},
	{"?INT", 		0,	rIlHandler},
	//{"?K", 			0,	rIlHandler},
	{"?L", 			0,	rLeHandler},
	//{"?LCK", 		0,	rIlHandler},
	{"?M", 			0,	rCcHandler},
	{"?MAXLP", 	0,	rMaxpHandler},
	{"?NOMP", 	0,	rRpHandler},
	{"?OP",     0,  rOpHandler},
	{"?P", 			0,	rLpHandler},
	{"?PVPS", 	0,	rFpHandler},
	//{"?ECHO", 	0,	rEchoHandler},
	{"?SP", 		0,	rLpsHandler},
	{"?SS", 		0,	rSsHandler},
	{"?STA", 		0,	rStaHandler},
	{"?SV", 		0,	rFvHandler},
	{"?SVH", 		0,	rFvHandler},
	{"?SVPS", 	0,	rFvHandler},
	{"?T", 			0,	rTecHandler},
	{"?WAVE", 	0,	rLwHandler},
	//{"?FD", 		0,	rFdHandler},

	//{"?LCS", 		0,	rLcsHandler},

	//{"?LS", 		0,	rLsHandler},

	{"",				0,	0}					// DO NOT MOVE/REMOVE!!	END OF COMMANDS!!
};

const char helpTableBrandX[][MAX_CMD_RSP_SIZE] = {
	">=[0/1] - Toggle Terminal Prompt: Off/On",
	"ANA=[0/1] - Set laser Drive Control Mode: Power/Current",
	//"CFC - Clear Stored Fault Codes",
	"CDRH=[0/1] - Set 5 Second CDRH Delay: Off/Nn",
	"DST=[15-35] - Optical Block Temperature in Celsius",			 // VOBTS
	"E=[0/1] - Toggle Terminal Echo: Off/On",
	"EXT=[0/1] - Set Laser Current: Off/On",
	//"LC=###.# - Set Laser Current: mA",
	"L=[0/1] - Set Laser Enable: Off/On",
	"P=###.# - Set Laser Power: mW",
	"PROTO=[0/1] - Set Protocol: Vortran/BrandX",
	//"RFC - Recall Stored Fault Codes",
	"T=[0/1] - Toggle TE Cooler: Off/On",
	"SS=[0/1] - Toggle TE Cooler: Off/On",
	"\n",

	"?ANA -  Request Laser Drive Control",
    "?BT - Request Base Plate Temperature",
	"?C - Request Laser Current Control",
	"?CDRH - Request 5 Second CDRH Delay Status",
	"?DT - Request Optical Block Temperature",
	"?DST - Request Optical Block Temperature Setting",
	//"?E - Request Echo",
	"?EXT - Request External Power",
	"?F - Request Fault Code",
	"?H - Request Help File",
	"?HH - Request Laser Operating Hours",
	"?HID - Request Laser Identification",
	"?INT - Request Interlock Status",
	"?K - Request Interlock Status",
	"?L - Request Laser Emission",
	"?LCK - Request Interlock Status",
	"?M - Request Computer Control Status",
	"?MAXLP - Request Maximum Laser Power",
	"?NOMP - Request Rated Laser Power",
	"?SS - Request TE Cooler/Heater",
	"?STA - Request Fault Code",
	"?T - Request TE Cooler/Heater",
	//"?FD - Request Fault Description",
	"?P - Request Laser Power mW",
	"?PVPS - Request Protocol Version",
	"?SP - Request Laser Power Setting",
	"?SV - Request Firmware Version",
	"?SVH - Request Firmware Version",
	"?SVPS - Request Firmware Version",
	//"?LCS - Request Laser Current",

	//"?LS - Request Laser Status",
	"?WAVE - Request Laser Wavelength",
	""		// END OF TABLE MARKER!! DO NOT REMOVE!!
};

// NOTE: make sure you add these to parserInit()
uchar isLsrEnableActive = TRUE;
char isAutoStartActive = 0;
char isManufModeActive = FALSE;
uchar isTeCoolerOn = 1;
short cdrhTriggersLsr = FALSE;
ulong cdrhStartTime = 0;
char pulMethod = PUL_USER_INIT;
char powerUp = TRUE;
char powerUpInitCurr = FALSE;

//uchar indxDebug = 0;	// debug

char myResponseBuf[MAX_CMD_SIZE];
char myBuf[MAX_CMD_RSP_SIZE];
static char myTmpBuf[MAX_CMD_SIZE];
uchar currentLe = 1;

void strupr(char *myInputBuf, char *myUprCaseBuf);

char myCmdToken[MAX_CMD_SIZE];
char* arg1;
char* arg2;


static uchar myOutputCommPort;

// Laser Monitor  modes
#ifndef SIMULATOR_ACTIVE
short monCdrhHandlerMode = TRUE;
short monStdbHandlerMode = TRUE;
short monIntlHandlerMode = TRUE;
short monDiCuHandlerMode = TRUE;
short monDTmpHandlerMode = TRUE;
short monBPltHandlerMode = TRUE;
#else
short monCdrhHandlerMode = FALSE;
short monStdbHandlerMode = FALSE;
short monIntlHandlerMode = FALSE;
short monDiCuHandlerMode = FALSE;
short monDTmpHandlerMode = FALSE;
short monBPltHandlerMode = FALSE;
#endif

const PROTOCOL_CMDS *myCmds;

// prototypes
void processEpcMode(void);
void leHandlerProcess(ushort myMode);
void lcHandlerProcess(ushort myMode);
void cHandlerProcess(ushort myMode);
void lpHandlerProcess(ushort myMode);
void pulHandlerProcess(ushort myMode, ushort action);
void tecHandlerProcess(ushort myMode);
void vobtsHandlerProcess(ushort myMode);
void rfInjHandlerProcess(ushort myMode);
void calHtrCntHandlerProcess(ushort myMode);
void dfblcHandlerProcess(ushort myMode);
void calDfblcDacCntHandlerProcess(ushort myMode);
void displayPrompt(uchar myPort);


// externs
extern void readAllA2dChnls(uchar myPort);
extern void readD2aChnl(uchar myPort);
extern void coldBlockTempSet(ushort myVal);
extern void modifyModulationPots(uchar myVal);
extern void setShadowRegsToDef(EE_AREA_TYPE myArea);
extern void saveCfgDefaults(EE_AREA_TYPE myArea);
extern void displaySwError(void);
extern short isTeCoolerHwOnBoard;
extern uchar wDogActive;
extern void extPsaControl(uchar myVal);
extern void extPsaSet(uchar myPort, uchar myVal);
extern uchar getBoardID(void);

void parserInit(void)
{
	isLsrEnableActive = TRUE;
	isAutoStartActive = FALSE;
	isManufModeActive = FALSE;
	isTeCoolerOn = 1;
	pulMethod = PUL_USER_INIT;
	powerUp = TRUE;
	powerUpInitCurr = FALSE;
}


// This paser accepts an stream of characters and determines if a valid command is present
// If valid then the command function/feature is performed and a specific output response is
// generated/composed for the specified stream. The "stream" argument is either Serial or USB.
void parser(char myPort, char *myBuf, Q_STREAM stream, uchar myMsgSize)
{
	uchar indx = 0;
	Q_STREAM myQ;
	char *myTmpPtr;
	char *myFinalPtr;
	uchar myLength;
	uchar myTmpLength;
	uchar hitCR = 0;
	uchar myPos = 0;
	uchar myTmpLen = 0;
	uchar myCnt = 0;
	uchar cmdMatched = 0;
	uchar numOfSpaces = 0;
	uchar charsRemoved = 0;
	uchar charsRemovedLeadingBS = 0;
	char	didWeFindCrKey = FALSE;
	char 	didWeFindEndOfCmdToken = FALSE;
	STOP_WATCH_SEL myStopWatch;
	short ignoreCmds = FALSE;

	//swTrace(__FILE__, __LINE__);		// log Trace: we have been here!

	myOutputCommPort = myPort; // save the current port	which generated the command

	if(myPort == COMM_0)	// determine which stop watch to use , and which Q shall be used
	{
		myStopWatch = CMD_COMM0_STOP_WATCH;
		myQ = SER0_Q;
	}
	else if(myPort == COMM_1)
	{
		myStopWatch = CMD_COMM1_STOP_WATCH;
		myQ = SER1_Q;
	}
	else if(myPort == USB_PORT)
	{
		myStopWatch = CMD_USB_STOP_WATCH;
		myQ = USB_Q;
	}
	else
	{
		swError(__FILE__, __LINE__);
		return;
	}

  if(*myBuf == CR_KEY || *myBuf == NL_KEY)	// Toss the enter key, or Toss the New Line key, and re-display the prompt
	{
		if(*myBuf == CR_KEY)
			displayPrompt(myPort);

		flushQ(myQ, IN_Q, 1);	 // Toss the character
		lsrOnStopWatch(myStopWatch, RESET);
		return;
	}
	else
		lsrOnStopWatch(myStopWatch, START);	 // we have a valid character, start the stop watch for this command

	// Initialize
	myCnt = 0;
	myTmpPtr = myBuf;

//#if 0
	while(*myTmpPtr == BS_KEY && myCnt++ < myMsgSize)	// remove all leading backspaces
	{
		myTmpPtr++;	// toss the Backspace
		charsRemovedLeadingBS++;
	}
//#endif

	myFinalPtr = myTmpBuf;

	// strip off backspace embedded in the input string
	for(myCnt = 0; myCnt < myMsgSize - charsRemovedLeadingBS; myCnt++)
	{
//#if 0
		if(*myTmpPtr == BS_KEY)
		{
			myFinalPtr--;	 // remove previous character
			*myFinalPtr = 0;
			myTmpPtr++;
			charsRemoved += 2;
		}
		else	// regular none backspace character
		{
			// FRG Debug
			//if(*myTmpPtr == CR_KEY)
				//myCnt += 0;
//#endif
			*myFinalPtr++ = *myTmpPtr++;	// save the character
//#if 0
		}
//#endif
	}

	*myFinalPtr = 0;
	//charsRemoved += charsRemovedLeadingBS;

	strcpy(myBuf, myTmpBuf);

	for(myCnt = 0; myCnt < MAX_CMD_SIZE; )	 // clear arrays
	{
		myTmpBuf[myCnt] = 0;
 		myCmdToken[myCnt++] = 0;
	}

	//strncpy(myTmpBuf, myBuf, myMsgSize);

	strupr(myBuf, myTmpBuf);		// convert command to upper case
	strupr(myBuf, myCmdToken);		// convert command to upper case

	myTmpPtr = myCmdToken;
	myCnt = 0;

	// truncate the input to just the "command" no spaces, New Line character, equal signs, or arguments
	while(*myTmpPtr	!= CR_KEY && *myTmpPtr != 0)
	{
		if(*myTmpPtr	== ' ')
		{
			didWeFindEndOfCmdToken = TRUE;
		 	*myTmpPtr = NULL;		// end of command token.
			break;
		}
		else if(*myTmpPtr	== '=')
		{
			didWeFindEndOfCmdToken = TRUE;
		 	*myTmpPtr = NULL;		// end of command token.
			break;
		}
		else
			myTmpPtr++;

		if(myCnt++ > MAX_CMD_SIZE)		// did we exceed the size?
		{
			*myCmdToken = NULL;					// yes, toss input
			swError(__FILE__,__LINE__);
		 	break;	// error shall be handled in following statements
		}
	}

	if(*myTmpPtr == CR_KEY)
	{
		didWeFindCrKey = TRUE;
		*myTmpPtr = NULL;					// yes, remove the Carriage return character
		lsrOnStopWatch(myStopWatch, RESET);
	}

#if 0
	// looking for the command delimeter "CR", so we can process the command
	for(myCnt = 0; myCnt < MAX_CMD_SIZE; myCnt++)
	{
		if(myTmpBuf[myCnt] == CR_KEY)
		{
			lsrOnStopWatch(myStopWatch, RESET);
			break;
		}
	}
#endif

	if(myCnt > MAX_CMD_SIZE)
	{
		flushQ(myQ, IN_Q, MAX_CMD_SIZE);
		//lsrOnStopWatch(myStopWatch, RESET);

		return;	// did not get the "CR"
	}
	else if(didWeFindEndOfCmdToken == TRUE)	// found "=" or " ", equal character or space
	{
		// is there a CR Key entered?
		myTmpPtr++;

		// search for the CR Key
		while(*myTmpPtr	!= CR_KEY && *myTmpPtr	!= 0)
		{
			myTmpPtr++;

			if(myCnt++ > MAX_CMD_SIZE)		// did we exceed the size?
			{
				*myCmdToken = NULL;					// yes, toss input
			 	break;	// error shall be handled in following statements
			}
		}

		if(*myTmpPtr == CR_KEY)
		{
			didWeFindCrKey = TRUE;
			*myTmpPtr = NULL;					// yes, remove the Carriage return character
			lsrOnStopWatch(myStopWatch, RESET);
		}
		else			// did not find CR key!! No reason to parse command
			return;
	}
	else if(didWeFindCrKey == FALSE)
		return;	// did not get the "CR"



	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		myCmds = vortCmds;
	else
		myCmds = brandxCmds;

	while(myCmds[indx].cmd[0] != 0)	 	// loop through all available valid commands
	{
		myLength = strlen(myCmds[indx].cmd);

		if(strcmp(myCmdToken, myCmds[indx].cmd) == 0)	// is this a valid command?	 Note: Could use hashing in the future.
		{
			if(myCmds[indx].maxArgs == 0)		 // Pass Arguments?
			{
				// verify that the input token length entered by the user, is the same length as the Vortran command
				//if(myLength == (strlen(myTmpBuf) - 1))	// do not count the CR as part of the command length entered by the user
				//{
					if(myTmpBuf[myLength] == CR_KEY)		// yes, same length
					{
						myTmpBuf[myLength] = NULL;		// change the carriage return into a NULL character

//						indxDebug = indx;	// debug

						if(ignoreCmds == TRUE)
							displayPrompt(myPort);		// FRG DEBUG!!!!
						else
						{
							if(sysCfgShadowRegs.protoMode != EE_DEF_PROTO_VORT)
								if(sysCfgShadowRegs.prompt==1)
									putString(myOutputCommPort, "\n");
								else
									if(sysCfgShadowRegs.echo==1)
										putString(myOutputCommPort, "\n");

							myCmds[indx].cmdHandler();		// no arguments
						}
						flushQ(myQ, IN_Q, myLength + 1 + charsRemoved);
						lsrOnStopWatch(myStopWatch, RESET);

						cmdMatched = 1;
					}
				//}
			}
			else if(myCmds[indx].maxArgs == 1)		// 1 argument expected
			{
				if(myTmpBuf[myLength + 3] == CR_KEY)
				{
					myTmpBuf[myLength + 3] = NULL;		// change the carriage return into a NULL character

					arg1 = (char*) myTmpBuf;	// yes, 1 argument

					myTmpLength = myLength;

					while(myTmpLength--)	 // move past the command token
						arg1++;

					while(*arg1 == ' ')
					{
						arg1++;	// move passed the space(s)
						myLength++;
					}

					if(*arg1 == '=')
					{
						arg1++;	// move passed the equal character '='
						myLength++;
					}

//				 	indxDebug = indx;	// FRG: debug

					
					if(sysCfgShadowRegs.protoMode != EE_DEF_PROTO_VORT)
						if(sysCfgShadowRegs.prompt==1)
							putString(myOutputCommPort, "\n");
						else
							if(sysCfgShadowRegs.echo==1)
								putString(myOutputCommPort, "\n");

					myCmds[indx].cmdHandler();	// PASS TOKENS!!!

					flushQ(myQ, IN_Q, myLength + 3 + charsRemoved);
					lsrOnStopWatch(myStopWatch, RESET);

					cmdMatched = 1;
				}
				else
				{
					myTmpPtr = &myTmpBuf[myLength];

					myTmpLength = myLength;

					// remove spaces
					while(*myTmpPtr == ' ')
					{
						myTmpPtr++;
						myLength++;
						// FRG TODO check for boundary conditions!!!
					}

					// is there an "=" character"
					if(*myTmpPtr == '=')
					{
						myTmpPtr++;		// move to next character
						myLength++;

						// remove spaces
						while(*myTmpPtr == ' ')
						{
							myTmpPtr++;
							myLength++;
							// FRG TODO check for boundary conditions!!!
						}

						// get argument/token
						arg1 = myTmpPtr;	// yes, 1 argument

						// find the "CR"
						while(*myTmpPtr != CR_KEY)
						{
							myTmpPtr++;
							myLength++;
						}

						*myTmpPtr = 0;	// replace the "CR" with a NULL
						myLength++;

//						indxDebug = indx;	// debug

						if(sysCfgShadowRegs.protoMode != EE_DEF_PROTO_VORT)
							if(sysCfgShadowRegs.prompt==1)
								putString(myOutputCommPort, "\n");
							else
								if(sysCfgShadowRegs.echo==1)
									putString(myOutputCommPort, "\n");

						myCmds[indx].cmdHandler();	// PASS TOKENS!!!

						flushQ(myQ, IN_Q, myLength + charsRemoved);
						lsrOnStopWatch(myStopWatch, RESET);

						cmdMatched = 1;
					}
				}
			}
			else if(myCmds[indx].maxArgs == 2)
			{
				numOfSpaces = 0;

				// find the Carriage Return, must have a CR, must also have 2 spaces
				while(myPos < (myMsgSize - charsRemoved))
				{
					if(myTmpBuf[myLength + myPos]	== ' ')
						numOfSpaces++;

					if(myTmpBuf[myLength + myPos]	== CR_KEY)
					{
						hitCR = 1;	// found it!
						myTmpBuf[myLength + myPos] = NULL;		// change the carriage return into a NULL character
						break;
					}

					myPos++;
				}

				if(hitCR && (numOfSpaces >= 2))
				{
					arg1 = (char*) myTmpBuf;	// yes, 1 argument

					myTmpLen = myLength;

					while(myTmpLen--)					// move past the "command" characters
						arg1++;

					while(*arg1 == ' ')
					{
						*arg1 = NULL;		// Fill spaces with NULL characters
						arg1++;	// move passed the space(s)
					}

					if(*arg1 == NULL)		// is this the end of the message?
					{
						flushQ(myQ, IN_Q, arg1 - myBuf + 1 + charsRemoved);
						lsrOnStopWatch(myStopWatch, RESET);

						return;
					}

					arg2 = arg1;	// get 2nd argument

					// read the token and search for space character
					while(*arg2 != ' ')
					{
					  if(*arg2 == NULL)
						{
							flushQ(myQ, IN_Q, arg2 - myBuf + 1 + charsRemoved);
							lsrOnStopWatch(myStopWatch, RESET);

							return;
						}

						arg2++;	// move passed the character(s)
					}

					*arg2 = NULL;		// Fill spaces with NULL characters
					arg2++;	// move passed this space, that was converted to a NULL

					// are there more spaces
					while(*arg2 == ' ')
					{
					  if(*arg2 == NULL)
						{
							flushQ(myQ, IN_Q, arg2 - myBuf + 1 + charsRemoved);
							lsrOnStopWatch(myStopWatch, RESET);

							return;
						}

						*arg2 = NULL;		// Fill spaces with NULL characters
						arg2++;	// move passed the space(s)
					}

//					indxDebug = indx;	// debug
					if(sysCfgShadowRegs.protoMode != EE_DEF_PROTO_VORT)
						if(sysCfgShadowRegs.prompt==1)
							putString(myOutputCommPort, "\n");
						else
							if(sysCfgShadowRegs.echo==1)
								putString(myOutputCommPort, "\n");

					myCmds[indx].cmdHandler();	// PASS TOKENS!!!

					flushQ(myQ, IN_Q, myLength + myPos + 1 + charsRemoved);
					lsrOnStopWatch(myStopWatch, RESET);

					cmdMatched = 1;
				}
			}

			if(cmdMatched)
				break;
		}

		indx++;
	}

	// Was a command executed?
	if(cmdMatched == 0)
	{
		// no, ...

		// did we get a Carriage return?
		for(myCnt = 0; myCnt < (myMsgSize - charsRemoved); myCnt++)
		{
			if(myBuf[myCnt] == CR_KEY)
			{
				flushQ(myQ, IN_Q, myCnt + 1 + charsRemoved);
				lsrOnStopWatch(myStopWatch, RESET);	// FRG: TODO both comm ports should not be using the same stop watch for the 10 seconds of no activity. Should be mutually exclusive
			}
		}

		// log  the error, and display the prompt
		errLogFlagSet(ERR_INVALID_COMMAND);
		displayPrompt(myPort);
	}
}



///////////////////////////////////////////////////
// Laser Drive Control Mode
/*
C

Function	Laser Drive Control Mode

Description
	As a command it is used to toggle the laser diode drive control mode between the light loop and a
	user entered current setting. (0 = Power Control, 1 = Current Control)
	As a query, it returns the present laser diode drive control mode.

Command/Query	Both
Syntax	Command	C=0, C=1
				Query		?C
Range	 1 = Power Control, 0 = Current Control
Resolution	N/A
Stored on Set	Yes
Dependencies
Not Allowed Conditions

Sequence
C=1	Set the Control Laser Current (I2C D/A VoA) to the last "LC" value or if not set to full power as limited by the light Regulator.
	Set the Control Laser Power Output (I2C D/A VoB) to 4095.
	Return the Laser Current (I2C A/D Vin0).

C=0	Set the Control Laser Power Output (I2C D/A VoB) to the last stored "LP" value.
	Set the Control Laser Current (I2C D/A VoA) to 4095
	Return the Laser Power Output (I2C A/D Vin1)		12bit A/DVin1(I2C) * (4095 / Max_Pwr_MW)

?C	Returns status of C either 1 or 0.
*/
ushort cHandler(void)
{
	cHandlerProcess(VORT_PARSER);

	return 0;
}

// called once from initSw(), use the eeprom value to set Current
void cHandlerProcess(ushort myMode)
{
	ushort myVal = 0;
	ushort myArg;
	uchar myEPCVal = 0;

	if(sysCfgShadowRegs.pulseMode == FALSE || powerUp == TRUE || powerUpInitCurr == TRUE)
	{
		if(myMode == VORT_PARSER)
			myArg = atoi(arg1);
		else
			myArg = sysCfgShadowRegs.c;


		if(myArg == LSR_CURRENT_CONTROL) 		// Current Control?
		{
			if(!isErrLogStatGrtr16() || powerUp == TRUE) //Set DACs always at power-up
			{
				if(sysCfgShadowRegs.epc == TRUE)
				{
				 	  //Adjust modulation pot for current control setting
					  myEPCVal = (2.0 * 255.0 * sysCfgShadowRegs.lc * 2.5)/(5.0 * calShadowRegs.maxLsrCur) + 0.5;
					  modifyModulationPots(myEPCVal);
				}

				// Set the Control Laser Current (I2C D/A VoA) to the last "LC" value or if not set to full power as limited by the light Regulator.
				myVal = sysCfgShadowRegs.lc * (4095.0 / calShadowRegs.maxLsrCur);
				extQuadD2aSoaSetCur(myVal);

				// Set the Control Laser Power Output (I2C D/A VoB) to 4095.
				extQuadD2aSoaSetLight(4095);

	 			if(myMode == VORT_PARSER)
				{
					// EEPROM_C
					sysCfgShadowRegs.c = LSR_CURRENT_CONTROL;
					saveSysCfgDelta(EE_AREA_SYS_CFG);

					if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
						sprintf(myBuf, "\n%s=1", myCmdToken);
					else
						sprintf(myBuf, "ANA=1");
					putString(myOutputCommPort, myBuf);
				}
			}
			else
			{
				errLogFlagSet(ERR_INVALID_COMMAND);
				if(myMode == VORT_PARSER)
				{
					if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
						sprintf(myBuf, "\n%s=0", myCmdToken);
					else
						sprintf(myBuf, "ANA=0");
					putString(myOutputCommPort, myBuf);
				}
			}
		}
		else if(myArg == LSR_POWER_CONTROL)			// Power Control
		{
			if(!isErrLogStatGrtr16() || powerUp == TRUE || powerUpInitCurr == TRUE) //Set DACs always at power-up
			{
				if(powerUp == TRUE)
				{
					//Temporarily and silently enter current control mode with DACs set for Initial Laser Current
					if(sysCfgShadowRegs.epc == TRUE)
					{
					 	  //Adjust modulation pot for current control setting
						  myEPCVal = (2.0 * 255.0 * calShadowRegs.initLsrCur * 2.5)/(5.0 * calShadowRegs.maxLsrCur) + 0.5;
						  modifyModulationPots(myEPCVal);
					}
	
					// Set the Control Laser Current (I2C D/A VoA) to the last "LC" value or if not set to full power as limited by the light Regulator.
					myVal = calShadowRegs.initLsrCur * (4095.0 / calShadowRegs.maxLsrCur);
					extQuadD2aSoaSetCur(myVal);
	
					// Set the Control Laser Power Output (I2C D/A VoB) to 4095.
					extQuadD2aSoaSetLight(4095);

					lsrOnStopWatch(INIT_CURR_PWRUP_STOP_WATCH, START);
				}
				else
				{
					powerUpInitCurr = FALSE;
					//Adjust modulation pot for power control setting
					if(sysCfgShadowRegs.epc == TRUE)
					{
					 	  //Adjust modulation pot for current control setting
						  modifyModulationPots(231);
					}
	
					// Set the Control Laser Power Output (I2C D/A VoB) to the last stored "LP" value.
					myVal = (ushort)(sysCfgShadowRegs.lp * ((4095 - (4095*((float)calShadowRegs.lpOffsetAdjust/100))) / calShadowRegs.maxPwrLsr) + (((float)calShadowRegs.lpOffsetAdjust/100)*4095));
	
					extQuadD2aSoaSetLight(myVal);
	
					// Set the Control Laser Current (I2C D/A VoA) to 4095
					extQuadD2aSoaSetCur(4095);
				}

				if(myMode == VORT_PARSER)
				{
					// EEPROM_C
					sysCfgShadowRegs.c = LSR_POWER_CONTROL;
					saveSysCfgDelta(EE_AREA_SYS_CFG);

					if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
						sprintf(myBuf, "\n%s=0", myCmdToken);
					else
						sprintf(myBuf, "ANA=0");
					putString(myOutputCommPort, myBuf);					
				}
			}
			else
			{
				errLogFlagSet(ERR_INVALID_COMMAND);
				if(myMode == VORT_PARSER)
				{
					if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
						sprintf(myBuf, "\n%s=0", myCmdToken);
					else
						sprintf(myBuf, "ANA=0");
					putString(myOutputCommPort, myBuf);
				}
			}
		}
		else if(myMode == VORT_PARSER)
		{
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

			if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
				sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.c);
			else
				sprintf(myBuf, "ANA=%d", sysCfgShadowRegs.c);
			putString(myOutputCommPort, myBuf);
		}
	}
	else
	{
		if(myMode == VORT_PARSER)
		{
			errLogFlagSet(ERR_INVALID_COMMAND);	// error

			if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
				sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.c);
			else
				sprintf(myBuf, "ANA=%d", sysCfgShadowRegs.c);
			putString(myOutputCommPort, myBuf);
		}
	}
	if(myMode == VORT_PARSER && powerUp == FALSE)
		displayPrompt(myOutputCommPort);
}



///////////////////////////////////////////////////
// Clear Fault Code Handler
ushort cfcHandler(void)
{
	storedFaultsClearAll();

	putString(myOutputCommPort, "\n");
	sprintf(myBuf, myCmdToken);			 // yes
	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// 5 Second Delay On/Off
ushort delayHandler(void)
{
	if(*arg1 == '1')
	{
		sysCfgShadowRegs.delay = TRUE;
		saveSysCfgDelta(EE_AREA_SYS_CFG);

		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
			sprintf(myBuf, "\n%s=1", myCmdToken);			 // yes
		else
			sprintf(myBuf, "CDRH=1");
		putString(myOutputCommPort, myBuf);
	}
	else if(*arg1 == '0')
	{
		sysCfgShadowRegs.delay = FALSE;
		saveSysCfgDelta(EE_AREA_SYS_CFG);

		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
			sprintf(myBuf, "\n%s=0", myCmdToken);			 // yes
		else
			sprintf(myBuf, "CDRH=0");
		putString(myOutputCommPort, myBuf);
	}
	else
	{
		errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
			sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.delay);
		else
			sprintf(myBuf, "CDRH=%d", sysCfgShadowRegs.delay);
		putString(myOutputCommPort, myBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

// called once from initSw(), use the eeprom value to set the delay
void delayHandlerInit(void)
{

	if(sysCfgShadowRegs.delay == TRUE)
		;
	else if(sysCfgShadowRegs.delay == FALSE)
		;
	else
	{
		sysCfgShadowRegs.delay = TRUE;
		swError(__FILE__, __LINE__);
	}
}



ushort dfblcHandler()
{
	dfblcHandlerProcess(VORT_PARSER);
	return 0;
}

void dfblcHandlerProcess(ushort myMode)
{
	float myVal;
	ushort myCalc;

	if(myMode == VORT_PARSER)
		myVal = atof(arg1);
	else
		myVal = sysCfgShadowRegs.dfblc;


	if(myVal > 0 && myVal <= calShadowRegs.maxCurrDfb)
	{
		if(!isErrLogStatGrtr16() || powerUp == TRUE) //Set DACs always at power-up
		{
			// Control Laser Current VoA = LC * (Maximum Laser Current / 4095)
			myCalc = (ushort) (myVal * (4095.0 / calShadowRegs.maxCurrDfb));
			extQuadD2aDfbSetCur(myCalc);

			if(myMode == VORT_PARSER)
			{
				sysCfgShadowRegs.dfblc = myVal;
				saveSysCfgDelta(EE_AREA_SYS_CFG);
				sprintf(myBuf, "\n%s=%s", myCmdToken, arg1);
				putString(myOutputCommPort, myBuf);
			}
		}
		else 	// In a fault condition, the query will return the measured laser current.
		{
			if(myMode == VORT_PARSER)
			{
				//getAllExtA2dRegs();
				errLogFlagSet(ERR_INVALID_COMMAND);	// error

				sprintf(myBuf, "\n%s=%3.1f", myCmdToken, extA2dCurrentValues[EXT_A2D_DFB_CURR] * 0.1224);			 // yes
				putString(myOutputCommPort, myBuf);
			}
		}
	}
	else if(myMode == VORT_PARSER)
	{
		errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		sprintf(myBuf, "\n%s=%3.1f", myCmdToken, sysCfgShadowRegs.dfblc);
		putString(myOutputCommPort, myBuf);
	}

	if(myMode == VORT_PARSER)
		displayPrompt(myOutputCommPort);
}

///////////////////////////////////////////////////
// Echo On/Off
ushort echoHandler(void)
{
	if(*arg1 == '1')
	{
		sysCfgShadowRegs.echo = TRUE;
		saveSysCfgDelta(EE_AREA_SYS_CFG);

		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
			sprintf(myBuf, "\n%s=1", myCmdToken);
		else
			sprintf(myBuf, "E=1");
		putString(myOutputCommPort, myBuf);

		//EEPROM_ECHO
	}
	else if(*arg1 == '0')
	{
		sysCfgShadowRegs.echo = FALSE;
		saveSysCfgDelta(EE_AREA_SYS_CFG);

		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
			sprintf(myBuf, "\n%s=0", myCmdToken);
		else
			sprintf(myBuf, "E=0");
		putString(myOutputCommPort, myBuf);

		//EEPROM_ECHO
	}
	else
	{
		errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
			sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.echo);
		else
			sprintf(myBuf, "E=%d", sysCfgShadowRegs.echo);
		putString(myOutputCommPort, myBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

// called once from initSw(), use the eeprom value to set the echo
void echoHandlerInit(void)
{
	if(sysCfgShadowRegs.echo == TRUE)
		;
	else if(sysCfgShadowRegs.echo == FALSE)
		;
	else
	{
		sysCfgShadowRegs.echo = TRUE;
		swError(__FILE__, __LINE__);
	}
}

///////////////////////////////////////////////////
// External power Control
ushort epcHandler(void)
{
	if(calShadowRegs.mode != 1)
	{
		if(*arg1 == '1')
		{
			// EEPROM Write
			sysCfgShadowRegs.epc = TRUE;
			saveSysCfgDelta(EE_AREA_SYS_CFG);
	
			processEpcMode();
	
			if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
				sprintf(myBuf, "\n%s=1", myCmdToken);
			else
				sprintf(myBuf, "EXT=1");
			putString(myOutputCommPort, myBuf);
		}
		else if(*arg1 == '0')
		{
			sysCfgShadowRegs.epc = FALSE;
			saveSysCfgDelta(EE_AREA_SYS_CFG);
	
			if(getBoardID() == BOARD_ID_X1)
			{
				IO0CLR = PIN_LIGHT_MOD_ON;
				IO0CLR = PIN_I_MOD_ON;
			}
			else if(getBoardID() == 1)
			{
				IO0CLR = PIN_LIGHT_CUR_MOD_ON;
				IO0CLR = PIN_MOD_ON;
			}
	
			if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
				sprintf(myBuf, "\n%s=0", myCmdToken);
			else
				sprintf(myBuf, "EXT=0");
			putString(myOutputCommPort, myBuf);
		}
		else
		{
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error
	
			if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
				sprintf(myBuf, "\n%s=%d", myCmdToken,sysCfgShadowRegs.epc);
			else
				sprintf(myBuf, "EXT=%d",sysCfgShadowRegs.epc);
			putString(myOutputCommPort, myBuf);
		}
	}
	else
	{
		errLogFlagSet(ERR_INVALID_COMMAND); //error
	
		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
			sprintf(myBuf, "\n%s=%d", myCmdToken,sysCfgShadowRegs.epc);
		else
			sprintf(myBuf, "EXT=%d",sysCfgShadowRegs.epc);	 
		putString(myOutputCommPort, myBuf);	
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

// Enables External Power Control Input Port (1= External Control)
void processEpcMode(void)
{
	uchar myVal = 0;

	if(sysCfgShadowRegs.epc == TRUE)
	{
		if(sysCfgShadowRegs.c == LSR_CURRENT_CONTROL)
		{

			if(getBoardID() == BOARD_ID_X1)
			{
				IO0CLR = PIN_LIGHT_MOD_ON;
				IO0SET = PIN_I_MOD_ON;
			}
			else if(getBoardID() == 1)
			{
				IO0SET = PIN_LIGHT_CUR_MOD_ON;
				IO0SET = PIN_MOD_ON;
			}

			// cnts (80 & 81) = (2*255cnts*LCS*2.5V)/(5.0V*Maximum Laser Current)
			// Round up to the nearest Integer.
			myVal = (2.0 * 255.0 * sysCfgShadowRegs.lc * 2.5)/(5.0 * calShadowRegs.maxLsrCur) + 0.5;

			modifyModulationPots(myVal);
		}
		else
		{
			if(getBoardID() == BOARD_ID_X1)
			{
				IO0SET = PIN_LIGHT_MOD_ON;
				IO0CLR = PIN_I_MOD_ON;
			}
			else if(getBoardID() == 1)
			{
				IO0CLR = PIN_LIGHT_CUR_MOD_ON;
				IO0SET = PIN_MOD_ON;
			}

			// When C=0 set both MOD Pots to the same count which is 231cnts (80 = 231, 81 = 231)
			// as this will give full power at 5.0V in.
			// The control voltage at the output of the Mod amp. = (Pot cnts * VEPC)/ (2*255cnts)
			// for power Vout is fixed at 2.27V.
			modifyModulationPots(231);
		}
	}
}

ushort calHtrCntHandler()
{
	calHtrCntHandlerProcess(VORT_PARSER);
	return 0;
}

void calHtrCntHandlerProcess(ushort myMode)
{
	ushort myArg;

	if(myMode == VORT_PARSER)
		myArg = atoi(arg1);
	else
		myArg = calShadowRegs.htrCnt;

 	if(myMode==VORT_PARSER)
 	{
		if(isManufModeActive == TRUE)
		{
			if(myArg<=4095)
			{
				calShadowRegs.htrCnt=myArg;
				extQuadD2aSetHeater(calShadowRegs.htrCnt);
				saveSysCfgDelta(EE_AREA_CAL);
				sprintf(myResponseBuf, "\nHEAT=%d", calShadowRegs.htrCnt);
				putString(myOutputCommPort, myResponseBuf);
			}
			else
				errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
		}
		else
		{
		 	 errLogFlagSet(ERR_INVALID_COMMAND);
		}

		displayPrompt(myOutputCommPort);
 	}
	else
	{
		if(myArg<=4095)
		{
			calShadowRegs.htrCnt=myArg;
			saveSysCfgDelta(EE_AREA_CAL);
			extQuadD2aSetHeater(calShadowRegs.htrCnt);
		}
	}

}

///////////////////////////////////////////////////
// Laser SOA Current Control 0 -Max Value (1000ma), 0 - 4095
ushort lcHandler(void)
{
	if(sysCfgShadowRegs.pulseMode == FALSE)
		lcHandlerProcess(VORT_PARSER);
	else
	{
		errLogFlagSet(ERR_INVALID_COMMAND);	// error

		sprintf(myBuf, "\n%s=%3.1f", myCmdToken, sysCfgShadowRegs.lc);
		putString(myOutputCommPort, myBuf);
	}

	return 0;
}

// called once from initSw(), use the eeprom value to set Laser Current
void lcHandlerProcess(ushort myMode)
{
	float myVal;
	ushort myCalc;

	if(myMode == VORT_PARSER)
		myVal = atof(arg1);
	else
		myVal = sysCfgShadowRegs.lc;


	if(myVal > 0 && myVal <= calShadowRegs.maxLsrCur)
	{
		// Allowed when C=1, Diode is ON, and no faults present
		if(isErrLogStatGrtr16() == FALSE && sysCfgShadowRegs.c == LSR_CURRENT_CONTROL && isLsrEnableActive == TRUE)
		{
			myCalc = (ushort) (myVal * (4095.0 / calShadowRegs.maxLsrCur ));		// Control Laser Current VoA = LC * (Maximum Laser Current / 4095)
			extQuadD2aSoaSetCur( myCalc);

			if(myMode == VORT_PARSER)
			{
				// EEPROM_LC
				sysCfgShadowRegs.lc = myVal;
				saveSysCfgDelta(EE_AREA_SYS_CFG);

				sprintf(myBuf, "\n%s=%s", myCmdToken, arg1);
				putString(myOutputCommPort, myBuf);
			}

			processEpcMode();
		}
		else 	// In a fault condition, the query will return the measured laser current.
		{
			if(myMode == VORT_PARSER)
			{
				//getAllExtA2dRegs();
				errLogFlagSet(ERR_INVALID_COMMAND);	// error

				sprintf(myBuf, "\n%s=%3.1f", myCmdToken, extA2dCurrentValues[EXT_A2D_SOA_CURR] * 0.1224);			 // yes
				putString(myOutputCommPort, myBuf);
			}
		}
	}
	else if(myMode == VORT_PARSER)
	{
		errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		sprintf(myBuf, "\n%s=%3.1f", myCmdToken, sysCfgShadowRegs.lc);
		putString(myOutputCommPort, myBuf);
	}

	if(myMode == VORT_PARSER)
		displayPrompt(myOutputCommPort);
}

///////////////////////////////////////////////////
// Laser Enable On/Off
ushort leHandler(void)
{
	leHandlerProcess(VORT_PARSER);

	return 0;
}

// Laser enable initialization
// called once from initSw(), and also in monitorInterlock().
void leHandlerProcess(ushort myMode)
{
	ushort myArg;
	short myTecState = FALSE;

	if(myMode == VORT_PARSER)
		myArg = atoi(arg1);
	else
		myArg = currentLe; //Allows turn on/off of laser diode without receiving LE command DMS 11/6/09

	if(myArg == 1)
	{
		isAutoStartActive=TRUE; //Reset flag to TRUE upon first user initiated "LE=1" DMS 11/6/09
		if(isErrLogStatGrtr16() == FALSE)
		{
	 		if(isColdBlockTempOk() == TRUE)		 	// ADD LATER!!!
			//if(TRUE)		// DEBUG!!!!
#if VORT_BOARD
				if(getBoardID() == BOARD_ID_X1)
				{
		 			if(IO0PIN & PIN_TE_ON)
		 				myTecState = TRUE;
				}
				else if(getBoardID() == 1)
				{
		 			if((IO0PIN & PIN_TE_ON) == 0)
		 				myTecState = TRUE;
				}

	 			if(myTecState)
	 					if(isAutoStartActive == TRUE)
						{
							if(IO0PIN & PIN_EXT_INTERLOCK)
							{
#endif
								isLsrEnableActive = TRUE;
								//errLogFlagClear(ERR_STANDBY);
								IO0SET = PIN_LSR_EMISSION_LED; // LED	ON
								IO0SET = PIN_LSR_RDY; //Assert Laser Ready Line DMS 11/6/09

								currentLe = 1;	// Is not saved to EEPROM

								if(myMode == VORT_PARSER)
								{
									if(sysCfgShadowRegs.protoMode==EE_DEF_PROTO_VORT)
									{
										sprintf(myBuf, "\n%s=1", myCmdToken);										
									}
									else
									{
										sprintf(myBuf, "L=1");
									}
									putString(myOutputCommPort, myBuf);
								}

								if(sysCfgShadowRegs.delay == 0)
								{
									if(isCalImageGood == TRUE)
									{
										IO0SET = PIN_LSR_ON; // Activate the Laser
										lsrOnStopWatch(LSR_STOP_WATCH, START);
									}

									cdrhTriggersLsr = FALSE;
								}
								else
								{
									if(cdrhTriggersLsr == FALSE)
									{
										cdrhTriggersLsr = TRUE;
										cdrhStartTime = 0;
									}
								}
#if VORT_BOARD
							}
						}
#endif
		}
	}
	else if(myArg == 0 && *arg1 == '0')
	{
		isLsrEnableActive = FALSE;
		IO0CLR = PIN_LSR_ON;				// In-Activate the Laser
		IO0CLR = PIN_LSR_EMISSION_LED;	// LED OFF
		//errLogFlagSet(ERR_STANDBY);

		lsrOnStopWatch(LSR_STOP_WATCH, STOP);

 		if(myMode == VORT_PARSER)
		{
			if(sysCfgShadowRegs.protoMode==EE_DEF_PROTO_VORT)
			{
				sprintf(myBuf, "\n%s=0", myCmdToken);										
			}
			else
			{
				sprintf(myBuf, "L=0");
			}
			putString(myOutputCommPort, myBuf);
		}

		currentLe = 0;	// Is not saved to eeprom
	}
	else if(myMode == VORT_PARSER)
	{
		errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error
		if(sysCfgShadowRegs.protoMode==EE_DEF_PROTO_VORT)
			sprintf(myBuf, "\n%s=%d", myCmdToken, currentLe);
		else
			sprintf(myBuf, "L=%d", currentLe);
		putString(myOutputCommPort, myBuf);
	}

	if(myMode == VORT_PARSER)
		displayPrompt(myOutputCommPort);
}

///////////////////////////////////////////////////
// Set Laser Power	0 -Max Value (1000mW), 0 - 4095
/*
LP
Function	Laser Power
Description		As a command, it sets laser power through USB and RS-232 interface.
							As a query, it will return the laser power measured by the light loop.
Command/Query	Both
Syntax	Command	LP=###.#	(mW)	Store as Requested Laser Power
				Query		?LP							returns in mW
Range	Zero to the maximum power the laser system is calibrated to.
Resolution	1:4095
Stored on Set	Yes
Dependencies	Not Allowed in Current Control Mode

Conditional Information:
		The laser power can be set at any time.  The return value will be based on the present laser state.
	 	For anything other than a ready condition, the fault code will be retruned.
	 	In a ready conditin, the internally measured laser pewer will be returned.


	Function													I2C Device Address
	Control Laser Light Pwr (LP)			00011110

																		I2C Device Address
	Laser Power output (?LP)					010100001


LP	12bit D/AVoutB(I2C)=	LP * (4095 / Max_Pwr_mW)

?LP	?LP=	12bit A/DVin1(I2C) * (4095 / Max_Pwr_MW)

	Note: Both the D/A value and the A/D value are normalized to 2.5V which is the Vref for both devices
*/
ushort lpHandler(void)
{
	if(sysCfgShadowRegs.pulseMode == FALSE)
		lpHandlerProcess(VORT_PARSER);
	else
	{
		errLogFlagSet(ERR_INVALID_COMMAND);	// error

	 	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
			sprintf(myBuf, "\n%s=%3.1f", myCmdToken, sysCfgShadowRegs.lp);
		else
			sprintf(myBuf, "P=%.2f", sysCfgShadowRegs.lp);

		putString(myOutputCommPort, myBuf);
	}

	return 0;
}

// called from initSw(), and also the parser, use the eeprom value to set Laser Power
void lpHandlerProcess(ushort myMode)
{
	//float myFlt;
	float myVal;
	float myArg;

	if(myMode == VORT_PARSER)
		myArg = atof(arg1);
	else
		myArg = sysCfgShadowRegs.lp;


	if(myArg > 0 && myArg < 1000 && myArg <= calShadowRegs.maxPwrLsr)		 	// range check in mW, Values greater than MAXP, will not change present set value.
	{
		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)	
		{
			 //Allowed in power control mode only, and when diode on with no faults present
			if(isErrLogStatGrtr16() == FALSE && sysCfgShadowRegs.c == LSR_POWER_CONTROL && isLsrEnableActive == TRUE) 
			{			
				sysCfgShadowRegs.lp = myArg; 		// no, perform the operation
	
				// LP	12bit D/AVoutB(I2C)=	LP * (4095 / Max_Pwr_mW)
				myVal = sysCfgShadowRegs.lp * ((4095 - (4095*((float)calShadowRegs.lpOffsetAdjust/100))) / calShadowRegs.maxPwrLsr) + (((float)calShadowRegs.lpOffsetAdjust/100)*4095);
				extQuadD2aSoaSetLight(myVal);
	
				if(myMode == VORT_PARSER)
				{
					saveSysCfgDelta(EE_AREA_SYS_CFG);
	
					getAllExtA2dRegs();
	
					//sprintf(myResponseBuf, "%s=%3.1f", myCmdToken, myArg);
					//sprintf(myBuf, "%s=%3.1f", myCmdToken, extA2dValues[EXT_A2D_LSR_PWR] * (4095 / calShadowRegs.maxPwrLsr));

					sprintf(myBuf, "\n%s=%3.1f", myCmdToken, extA2dValues[EXT_A2D_LSR_PWR] * (calShadowRegs.maxPwrLsr / 4095.0));			 // yes					
					putString(myOutputCommPort, myBuf);
				}							
			}
			else 	// fault condition
			{
				// In a fault condition, the query will return the measured laser power.
				// ?LP = 12bit A/DVin1(I2C) * (4095 / maxPwrMw)
	
				getAllExtA2dRegs();
				errLogFlagSet(ERR_INVALID_COMMAND);	// error
	
				if(myMode == VORT_PARSER)
				{
					sprintf(myBuf, "\n%s=%3.1f", myCmdToken, extA2dValues[EXT_A2D_LSR_PWR] * (calShadowRegs.maxPwrLsr / 4095.0));
					putString(myOutputCommPort, myBuf);
				}
			}
		}
		else
		{
			 //Allowed in power control mode only, no faults present, brandX can change power with diode off
			if(isErrLogStatGrtr16() == FALSE && sysCfgShadowRegs.c == LSR_POWER_CONTROL) 
			{			
				sysCfgShadowRegs.lp = myArg; 		// no, perform the operation
	
				// LP	12bit D/AVoutB(I2C)=	LP * (4095 / Max_Pwr_mW)
				myVal = sysCfgShadowRegs.lp * ((4095 - (4095*((float)calShadowRegs.lpOffsetAdjust/100))) / calShadowRegs.maxPwrLsr) + (((float)calShadowRegs.lpOffsetAdjust/100)*4095);
				extQuadD2aSoaSetLight(myVal);
	
				if(myMode == VORT_PARSER)
				{
					saveSysCfgDelta(EE_AREA_SYS_CFG);
	
					getAllExtA2dRegs();
	
					//sprintf(myResponseBuf, "%s=%3.1f", myCmdToken, myArg);
					//sprintf(myBuf, "%s=%3.1f", myCmdToken, extA2dValues[EXT_A2D_LSR_PWR] * (4095 / calShadowRegs.maxPwrLsr));
					sprintf(myBuf, "P=%.2f", extA2dValues[EXT_A2D_LSR_PWR] * (calShadowRegs.maxPwrLsr / 4095.0));			 // yes	
					putString(myOutputCommPort, myBuf);
				}							
			}
			else 	// fault condition
			{
				// In a fault condition, the query will return the measured laser power.
				// ?LP = 12bit A/DVin1(I2C) * (4095 / maxPwrMw)
	
				getAllExtA2dRegs();
				errLogFlagSet(ERR_INVALID_COMMAND);	// error
	
				if(myMode == VORT_PARSER)
				{
					sprintf(myBuf, "P=%.2f", extA2dValues[EXT_A2D_LSR_PWR] * (calShadowRegs.maxPwrLsr / 4095.0));
					putString(myOutputCommPort, myBuf);
				}
			}
		}
		
	}
	else if(myMode == VORT_PARSER)
	{
		errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
			sprintf(myBuf, "\n%s=%3.1f", myCmdToken, sysCfgShadowRegs.lp);
		else
			sprintf(myBuf, "P=%.2f", sysCfgShadowRegs.lp);
		putString(myOutputCommPort, myBuf);
	}

 	if(myMode == VORT_PARSER)
		displayPrompt(myOutputCommPort);
}

/////////////////////////////////////
// MCDRH -  enable/disable monitor laser status for CDRH
ushort monCdrhHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		if(*arg1 == '1')
		{
			monCdrhHandlerMode = TRUE;

			sprintf(myBuf, "\n%s=%d", myCmdToken, monCdrhHandlerMode);
			putString(myOutputCommPort, myBuf);

		}
		else if(*arg1 == '0')
		{
			monCdrhHandlerMode = FALSE;

			sprintf(myBuf, "\n%s=%d", myCmdToken, monCdrhHandlerMode);
			putString(myOutputCommPort, myBuf);

		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		displayPrompt(myOutputCommPort);
	}
	else
	{
		// log  the error, and display the prompt
		errLogFlagSet(ERR_INVALID_COMMAND);
		displayPrompt(myOutputCommPort);
	}

	return 0;
}

/////////////////////////////////////
// MSTDB -  enable/disable monitor laser status for Stand By
ushort monStdbHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		if(*arg1 == '1')
		{
			monStdbHandlerMode = TRUE;

			sprintf(myBuf, "\n%s=%d", myCmdToken, monStdbHandlerMode);
			putString(myOutputCommPort, myBuf);

		}
		else if(*arg1 == '0')
		{
			monStdbHandlerMode = FALSE;

			sprintf(myBuf, "\n%s=%d", myCmdToken, monStdbHandlerMode);
			putString(myOutputCommPort, myBuf);

		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		displayPrompt(myOutputCommPort);
 	}
	else
	{
		// log  the error, and display the prompt
		errLogFlagSet(ERR_INVALID_COMMAND);
		displayPrompt(myOutputCommPort);
	}

	return 0;
}

/////////////////////////////////////
// MINTL -  enable/disable monitor laser status for Interlock open
ushort monIntlHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		if(*arg1 == '1')
		{
			monIntlHandlerMode = TRUE;

			sprintf(myBuf, "\n%s=%d", myCmdToken, monIntlHandlerMode);
			putString(myOutputCommPort, myBuf);

		}
		else if(*arg1 == '0')
		{
			monIntlHandlerMode = FALSE;

			sprintf(myBuf, "\n%s=%d", myCmdToken, monIntlHandlerMode);
			putString(myOutputCommPort, myBuf);

		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		displayPrompt(myOutputCommPort);
	}
	else
	{
		// log  the error, and display the prompt
		errLogFlagSet(ERR_INVALID_COMMAND);
		displayPrompt(myOutputCommPort);
	}

	return 0;
}

/////////////////////////////////////
// MDICU -  enable/disable monitor laser status for Diode Current
ushort monDiCuHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		if(*arg1 == '1')
		{
			monDiCuHandlerMode = TRUE;

			sprintf(myBuf, "\n%s=%d", myCmdToken, monDiCuHandlerMode);
			putString(myOutputCommPort, myBuf);

		}
		else if(*arg1 == '0')
		{
			monDiCuHandlerMode = FALSE;

			sprintf(myBuf, "\n%s=%d", myCmdToken, monDiCuHandlerMode);
			putString(myOutputCommPort, myBuf);

		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		displayPrompt(myOutputCommPort);
	}
	else
	{
		// log  the error, and display the prompt
		errLogFlagSet(ERR_INVALID_COMMAND);
		displayPrompt(myOutputCommPort);
	}

	return 0;
}

/////////////////////////////////////
// MDITM - enable/disable monitor laser status for Diode Temperature
ushort monDTmpHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		if(*arg1 == '1')
		{
			monDTmpHandlerMode = TRUE;

			sprintf(myBuf, "\n%s=%d", myCmdToken, monDTmpHandlerMode);
			putString(myOutputCommPort, myBuf);

		}
		else if(*arg1 == '0')
		{
			monDTmpHandlerMode = FALSE;

			sprintf(myBuf, "\n%s=%d", myCmdToken, monDTmpHandlerMode);
			putString(myOutputCommPort, myBuf);

		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		displayPrompt(myOutputCommPort);
	}
	else
	{
		// log  the error, and display the prompt
		errLogFlagSet(ERR_INVALID_COMMAND);
		displayPrompt(myOutputCommPort);
	}

	return 0;
}

/////////////////////////////////////
// MBPLT - enable/disable monitor laser status for Base Plate Temperature
ushort monBPltHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		if(*arg1 == '1')
		{
			monBPltHandlerMode = TRUE;

			sprintf(myBuf, "\n%s=%d", myCmdToken, monBPltHandlerMode);
			putString(myOutputCommPort, myBuf);

		}
		else if(*arg1 == '0')
		{
			monBPltHandlerMode = FALSE;

			sprintf(myBuf, "\n%s=%d", myCmdToken, monBPltHandlerMode);
			putString(myOutputCommPort, myBuf);

		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		displayPrompt(myOutputCommPort);
	}
	else
	{
		// log  the error, and display the prompt
		errLogFlagSet(ERR_INVALID_COMMAND);
		displayPrompt(myOutputCommPort);
	}

	return 0;
}

ushort burnHandler(void)			// eeprom set to defaults: system Config, Calibration
{
	if(isManufModeActive == TRUE)
	{
		saveCfgDefaults(EE_AREA_SYS_CFG);	// save the system config
		saveCfgDefaults(EE_AREA_CAL);			// save the calibration information

		sprintf(myBuf, "\nSet eeprom images to defaults\n");
		putString(myOutputCommPort, myBuf);

		displayPrompt(myOutputCommPort);
	}
	else
	{
		// log  the error, and display the prompt
		errLogFlagSet(ERR_INVALID_COMMAND);
		displayPrompt(myOutputCommPort);
	}

	return 0;
}

// some of the development boards do not have the TECooler hardware working
ushort tecModeHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		if(*arg1 == '1')
		{
			isTeCoolerHwOnBoard = TRUE;
			sprintf(myBuf, "\n%s=1", myCmdToken);
			putString(myOutputCommPort, myBuf);
		}
		else if(*arg1 == '0')
		{
			isTeCoolerHwOnBoard = FALSE;
			sprintf(myBuf, "\n%s=0", myCmdToken);
			putString(myOutputCommPort, myBuf);
		}

		displayPrompt(myOutputCommPort);
	}
	else
	{
		// log  the error, and display the prompt
		errLogFlagSet(ERR_INVALID_COMMAND);

		displayPrompt(myOutputCommPort);
	}

	return 0;
}

// force the pulse mode output pin high or low
ushort psaHandler(void)
{
	short myTmp;
	short validCmd = TRUE;

	if(isManufModeActive == TRUE)
	{
 		// did the user enter a hex value to set the 4 bit port?	arg1 has already been converted to upper case,	as needed
		if((*arg1 >= '0' && *arg1 <= '9') || (*arg1 >= 'A' && *arg1 <= 'F'))
		{
			if(*arg1 >= '0' && *arg1 <= '9')
			{
				myTmp = atoi(arg1);
				extPsaControl(myTmp);
			}
			else if(*arg1 >= 'A' && *arg1 <= 'F')
			{
				myTmp = *arg1 - 55;		// convert to number 10 - 15
				extPsaControl(myTmp);
			}
			else
			{
				validCmd = FALSE;
				errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
			}

		 	if(validCmd == TRUE)
			{
				sprintf(myBuf, "\n%s=1", myCmdToken);
				putString(myOutputCommPort, myBuf);
			}

			displayPrompt(myOutputCommPort);
		}
		else
		{
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
			displayPrompt(myOutputCommPort);
		}
	}
	else
	{
		// log  the error, and display the prompt
		errLogFlagSet(ERR_INVALID_COMMAND);
		displayPrompt(myOutputCommPort);
	}

	return 0;
}

ushort swErrHandler(void)			// display software errors
{
	if(isManufModeActive == TRUE)
	{
		displaySwError();

		displayPrompt(myOutputCommPort);
	}
	else
	{
		// log  the error, and display the prompt
		errLogFlagSet(ERR_INVALID_COMMAND);
		displayPrompt(myOutputCommPort);
	}

	return 0;
}

ushort wDogOffHandler(void)			// disable the watchdog kick!
{
	if(isManufModeActive == TRUE)
	{
		wDogKick();
		wDogActive = 0;

		displayPrompt(myOutputCommPort);
	}
	else
	{
		// log  the error, and display the prompt
		errLogFlagSet(ERR_INVALID_COMMAND);
		displayPrompt(myOutputCommPort);
	}

	return 0;
}

///////////////////////////////////////////////////
//OP Operating Mode Brand X Only
ushort opHandler(void)
{
	if(sysCfgShadowRegs.protoMode != EE_DEF_PROTO_VORT)
	{
		sprintf(myBuf, "OP=1");
		putString(myOutputCommPort, myBuf);
	}
		
	displayPrompt(myOutputCommPort);
 	return 0;
}

///////////////////////////////////////////////////
// PP	Peak Pulse Power
ushort ppHandler(void)
{
	short myArg;
	ushort myMode = VORT_PARSER;

	myArg = atoi(arg1);

	//sprintf(myBuf, "\nDbg: PP=%d", myArg);
	//putString(myOutputCommPort, myBuf);


	if(myArg >= 0 && myArg <= 1000 && myArg <= calShadowRegs.maxPwrLsr)		 	// range check in mW
	{
		if(isErrLogStatGrtr16() == FALSE)
		{
			sysCfgShadowRegs.peakPower = myArg;

			if(sysCfgShadowRegs.pulseMode == 1 && (IO0PIN & PIN_LSR_EN) && (IO0PIN & PIN_LSR_ON)) //Only enter pulse mode when in pulse mode, laser enable is not held low externally, and processor asserts diode to be ON
				pulseModeOn();

			saveSysCfgDelta(EE_AREA_SYS_CFG);

			if(myMode == VORT_PARSER)
			{
				sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.peakPower);
				putString(myOutputCommPort, myBuf);
			}
		}
		else
		{
			errLogFlagSet(ERR_INVALID_COMMAND);	// error

		 	if(myMode == VORT_PARSER)
			{
				sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.peakPower);
				putString(myOutputCommPort, myBuf);
			}
		}
	}
	else
	{
		//sprintf(myBuf, "\nDbg: PP out of Range 0 - 1000");
		//putString(myOutputCommPort, myBuf);

		errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.peakPower);
		putString(myOutputCommPort, myBuf);
	}


	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Prompt On/Off
ushort promptHandler(void)
{
	if(*arg1 == '1')
	{
		sysCfgShadowRegs.prompt = TRUE;
		saveSysCfgDelta(EE_AREA_SYS_CFG);

		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		{
			sprintf(myBuf, "\n%s=1", myCmdToken);
		}
		else
		{
			sprintf(myBuf, ">=1");
		}
		putString(myOutputCommPort, myBuf);		
	}
	else if(*arg1 == '0')
	{
		if(sysCfgShadowRegs.protoMode != EE_DEF_PROTO_VORT)
			if(sysCfgShadowRegs.prompt==1)
				sprintf(myBuf, "\n");
		sysCfgShadowRegs.prompt = FALSE;
		saveSysCfgDelta(EE_AREA_SYS_CFG);

		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		{
			sprintf(myBuf, "\n%s=0", myCmdToken);
		}
		else
		{
			sprintf(myBuf, ">=0");
		}
		putString(myOutputCommPort, myBuf);
	}
	else
	{
		errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error
		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
			sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.prompt);
		else
			sprintf(myBuf, ">=%d", sysCfgShadowRegs.prompt);
		putString(myOutputCommPort, myBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Protocol Vortran, brand x
ushort protoHandler(void)
{
	if(*arg1 == '1')
	{
		sysCfgShadowRegs.protoMode = TRUE;

		saveSysCfgDelta(EE_AREA_SYS_CFG);

		sprintf(myBuf, "\n%s=BrandX", myCmdToken);
		putString(myOutputCommPort, myBuf);


	}
	else if(*arg1 == '0')
	{
		sysCfgShadowRegs.protoMode = FALSE;

		saveSysCfgDelta(EE_AREA_SYS_CFG);

		sprintf(myBuf, "\n%s=Vortran", myCmdToken);
		putString(myOutputCommPort, myBuf);


	}
	else
	{
		errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		putString(myOutputCommPort, "\n");
		sprintf(myBuf, "%s=%d", myCmdToken, sysCfgShadowRegs.protoMode);
		putString(myOutputCommPort, myBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

// Changes operating mode to pulse mode
void pulseModeOn(void)
{
	char myEpc;
	float myTmpLp;

	//Set PSA=0 therefore setting laser to CW 11/19/09 Vortran-DMS
	extPsaSet(0,0);

	// Then the light control DAC (I2C D/A VoB) is set to zero (0)
	extQuadD2aSoaSetLight(0);

	delayMs(50);

	// Then set the Current Drive (I2C D/A VoA) to 4095
	extQuadD2aSoaSetCur(4095);

	// The PP value is set as an LP= value so the light output equals that value
	myTmpLp = sysCfgShadowRegs.lp; //Temporarily stored so that lpHandlerProcess can be used to set peakPower
	sysCfgShadowRegs.lp = sysCfgShadowRegs.peakPower;
	lpHandlerProcess(NO_PARSER);


	myEpc = sysCfgShadowRegs.epc;
	sysCfgShadowRegs.epc = 0;

	// epc = 0
	if(getBoardID() == BOARD_ID_X1)
	{
		IO0CLR = PIN_LIGHT_MOD_ON;
		IO0CLR = PIN_I_MOD_ON;
	}
	else if(getBoardID() == 1)
	{
		IO0CLR = PIN_LIGHT_CUR_MOD_ON;
		IO0CLR = PIN_MOD_ON;
	}

	delayMs(50);

	if(pulMethod == PUL_USER_INIT)
	{
		// The drive voltage is measured (I2C A/D Vin2) in counts then this same count is set on the current DAC (I2C D/A VoA)
		getAllExtA2dRegs();
		sysCfgShadowRegs.currDrvSetting = extA2dValues[EXT_A2D_LSR_DRV_CUR];
		extQuadD2aSoaSetCur(extA2dValues[EXT_A2D_LSR_DRV_CUR]);
	}
	else
		if(pulMethod == PUL_AT_POWERUP)
		{
			extQuadD2aSoaSetCur(sysCfgShadowRegs.currDrvSetting);
			pulMethod = PUL_USER_INIT; //Next time PUL=1 is done, full calibration will be done upon setting
		}

	//Store Drive Current counts into EEPROM for setting PUL after power is cycled without performing the complete calibration process
	//extA2dValues[EXT_A2D_LSR_DRV_CUR] //type is ushort



	// Then the light control DAC (I2C D/A VoB) is set to maximum (4095)
	extQuadD2aSoaSetLight(4095);

	// Then a PSA=1 is sent changing the laser mode to pulse.
	extPsaSet(0,1);

	if(myEpc == 1)
	{
		sysCfgShadowRegs.epc = 1;
		processEpcMode();
	}

	sysCfgShadowRegs.lp = myTmpLp;
}



///////////////////////////////////////////////////
// pulHandlerProcess() Changes operating mode for pulse mode
//Called from parser.c, pulHandler() and quietly from initBoard.c, initSw()
//
void pulHandlerProcess(ushort myMode, ushort action)
{
		//Added NO_PARSER functionality for setting PUL=0 from initBoard() DMS V2.1.29
		if(myMode==NO_PARSER)
		{
			delayMs(50);
			if(sysCfgShadowRegs.c == LSR_POWER_CONTROL)		 // This mode may only be entered when the C=0
			{
				// This procedure assume that a value for PP exists in the EEPROM as user entered data.
				// IF PP=0 then PP will equal LP.
				if(sysCfgShadowRegs.peakPower == 0)
					sysCfgShadowRegs.peakPower = sysCfgShadowRegs.lp;
				if(action==1) // PUL=1
				{
					sysCfgShadowRegs.pulseMode = TRUE;

					pulseModeOn();

					saveSysCfgDelta(EE_AREA_SYS_CFG);
				}
				else //PUL=0
				{
					sysCfgShadowRegs.pulseMode = FALSE;

					// A zero count is set on the light control DAC(I2C D/A VoB)
					extQuadD2aSoaSetLight(0);

					// After a delay of 100ms the current control DAC (I2C D/A VoA) is set to maximum
					delayMs(100);
					extQuadD2aSoaSetCur(4095);

					// Then the stored LP value is used for LP= setting the light control DAC (I2C D/A VoB)
					lpHandlerProcess(NO_PARSER);

					// Then PSA=0 is set turning the laser mode to CW
					extPsaSet(0,0);

					saveSysCfgDelta(EE_AREA_SYS_CFG);
				}
			}

		}
		else
		if(myMode==VORT_PARSER)
		{
			if((isErrLogStatGrtr16() == FALSE) && sysCfgShadowRegs.c == LSR_POWER_CONTROL && (IO0PIN & PIN_LSR_EN) && (IO0PIN & PIN_LSR_ON) && calShadowRegs.mode != 1) // This mode may only be entered by user command when the C=0, when laser enable line is NOT held LOW externally, and processor asserts diode to be ON
			{
				// This procedure assume that a value for PP exists in the EEPROM as user entered data.
				// IF PP=0 then PP will equal LP.
				if(sysCfgShadowRegs.peakPower == 0)
					sysCfgShadowRegs.peakPower = sysCfgShadowRegs.lp;

				if(*arg1 == '1')
				{
					sysCfgShadowRegs.pulseMode = TRUE;

					pulseModeOn();

					saveSysCfgDelta(EE_AREA_SYS_CFG);

					sprintf(myBuf, "\n%s=1", myCmdToken);
					putString(myOutputCommPort, myBuf);
				}
				else if(*arg1 == '0')
				{
					sysCfgShadowRegs.pulseMode = FALSE;

					// A zero count is set on the light control DAC(I2C D/A VoB)
					extQuadD2aSoaSetLight(0);

					// After a delay of 100ms the current control DAC (I2C D/A VoA) is set to maximum
					delayMs(100);
					extQuadD2aSoaSetCur(4095);

					// Then the stored LP value is used for LP= setting the light control DAC (I2C D/A VoB)
					lpHandlerProcess(NO_PARSER);

					// Then PSA=0 is set turning the laser mode to CW
					extPsaSet(0,0);

					saveSysCfgDelta(EE_AREA_SYS_CFG);

					sprintf(myBuf, "\n%s=0", myCmdToken);
					putString(myOutputCommPort, myBuf);
				}
				else
				{
					errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

					sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.pulseMode);
					putString(myOutputCommPort, myBuf);
				}
			}
			else
			{
				errLogFlagSet(ERR_INVALID_COMMAND);	// error

				sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.pulseMode);
				putString(myOutputCommPort, myBuf);
			}


			displayPrompt(myOutputCommPort);
	}

}

///////////////////////////////////////////////////
// PUL Changes operating mode for pulse mode
//
// PUL=1
// This procedure assume that a value for PP exists in the EEPROM as user entered data
// The PP value is set as an LP= value so the light output equals that value
// EPC = 0
// Delay of 50ms
// The drive voltage is measured (I2C A/D Vin2) in counts then this same count is set on the current DAC (I2C D/A VoA)
// Then the light control DAC (I2C D/A VoB) is set to maximum (4095)
// Then a PSA=1 is sent changing the laser mode to pulse.
// If was EPC = 1 at start then EPC = 1
//
// PUL=0
// This procedure is used to safely turn off pulse mode.
// A zero count is set on the light control DAC(I2C D/A VoB)
// After a delay of 100ms the current control DAC (I2C D/A VoA) is set to maximum
// Then the stored LP value is used for LP= setting the light control DAC (I2C D/A VoB)
// Then PSA=0 is set turning the laser mode to CW

ushort pulHandler(void)
{
	pulHandlerProcess(VORT_PARSER,0); //"action" is ignored in VORT_PARSER mode
	return 0;
}


///////////////////////////////////////////////////
// Recall Fault Code Handler
ushort rfcHandler(void)
{
	sprintf(myBuf, "\n%s=%d", myCmdToken, storedFaultsGet());

	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// TE Cooler/Heater On/Off
ushort tecHandler(void)
{
	tecHandlerProcess(VORT_PARSER);

	return 0;
}

// called from initSw() and the parser, use the eeprom value to set the TE Cooler
void tecHandlerProcess(ushort myMode)
{
	ushort myArg;

	if(myMode == VORT_PARSER)
		myArg = atoi(arg1);
	else
		myArg = sysCfgShadowRegs.isTecOn;

	if(myArg == 1)
	{
		if(getBoardID() == BOARD_ID_X1)
			IO0SET = PIN_TE_ON;
		else if(getBoardID() == 1)
			IO0CLR = PIN_TE_ON;



		errLogFlagClear(ERR_TEC_OFF);
		isTeCoolerOn = TRUE;

		if(myMode == VORT_PARSER)
		{
			sysCfgShadowRegs.isTecOn = TRUE;
			saveSysCfgDelta(EE_AREA_SYS_CFG);

			if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
				sprintf(myBuf, "\n%s=1", myCmdToken);
			else
				sprintf(myBuf, "T=1");
			putString(myOutputCommPort, myBuf);
		}
	}
	else if(myArg == 0 && *arg1 == '0')
	{
		IO0CLR = PIN_LSR_ON;						// STOP LASER!!
		IO0CLR = PIN_LSR_EMISSION_LED;	// Stop Laser Emission!

		if(getBoardID() == BOARD_ID_X1)
			IO0CLR = PIN_TE_ON;
		else if(getBoardID() == 1)
			IO0SET = PIN_TE_ON;

		isLsrEnableActive = FALSE;
		//errLogFlagSet(ERR_STANDBY);

		lsrOnStopWatch(LSR_STOP_WATCH, STOP);

		errLogFlagSet(ERR_TEC_OFF);	// error
		isTeCoolerOn = FALSE;

		if(myMode == VORT_PARSER)
		{
			sysCfgShadowRegs.isTecOn = FALSE;
			saveSysCfgDelta(EE_AREA_SYS_CFG);

			if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
				sprintf(myBuf, "\n%s=0", myCmdToken);
			else
				sprintf(myBuf, "T=0");
			putString(myOutputCommPort, myBuf);
		}

	}
	else 	if(myMode == VORT_PARSER)
	{
		errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
			sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.isTecOn);
		else
			sprintf(myBuf, "T=%d", sysCfgShadowRegs.isTecOn);
		putString(myOutputCommPort, myBuf);
	}

	if(myMode == VORT_PARSER)
		displayPrompt(myOutputCommPort);
}


///////////////////////////////////////////////////
// Optical Block temperature 15 - 35 degrees C
ushort vobtsHandler(void)
{
	vobtsHandlerProcess(VORT_PARSER);

	return 0;
}



void vobtsHandlerProcess(ushort myMode)
{
	ushort myVal = 0;
	//ushort myCalc = 0;
	float myFloat = 0;
	volatile short myTicks = 0;
	ushort myArg;

	if(myMode == VORT_PARSER)
		myArg = atoi(arg1);

	else
		myArg = sysCfgShadowRegs.vobts;


	if(myArg >= 15 && myArg <= 35)
	{
		if(isErrLogStatGrtr16() == FALSE)
		{
			// DAC 0 = 34.1 * (VOBTS - 10) cnts
			//myCalc = 34.1 * (myVal - 10);
			coldBlockTempSet(myArg);

			if(myMode == VORT_PARSER)
			{
				sysCfgShadowRegs.vobts = myArg;
				saveSysCfgDelta(EE_AREA_SYS_CFG);

				if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
					sprintf(myResponseBuf, "\n%s=%d", myCmdToken, myArg);
				else
					sprintf(myResponseBuf, "DST=%d", myArg);
				putString(myOutputCommPort, myResponseBuf);
			}
		}
		else	 // a fault condition exists
		{
			if(myMode == NO_PARSER)
				coldBlockTempSet(22);
			else
			{
				// (COLD BLOCK TEMP, (Internal ADC0.1) * .02933) +10
				// get the Cold Block Value
				a2dChnlStart(A2D_CHNL_COLD_BLOCK);

			 	timer0Init(MAX_TIMEOUT_A2D);
				timer0Start();

				while(isA2d0ConvDone == FALSE)
				{
					if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
					{
						swError(__FILE__, __LINE__);
					 	break;
					}
				}

				timer0Stop();

				//while(isA2dDone() == TRUE)
					//;

				myVal = a2dChnlRead();
				myFloat = (myVal * 0.02933) + 10;

				if(myMode == VORT_PARSER)
				{
					if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
						sprintf(myBuf, "\n%s=%3.1f", myCmdToken, myFloat);
					else
						sprintf(myBuf, "DST=%3.1f", myFloat);
					putString(myOutputCommPort, myBuf);
				}
			}
		}
	}
	else if(myMode == VORT_PARSER)
	{
		errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error
			
		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
			sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.vobts);
		else
			sprintf(myBuf, "DST=%d", sysCfgShadowRegs.vobts);
		putString(myOutputCommPort, myBuf);
	}

	if(myMode == VORT_PARSER)
 		displayPrompt(myOutputCommPort);
}


///////////////////////////////////////////////////
// Backdoor to activate manufacturing mode commands
ushort vortHandler(void)
{
		isManufModeActive = TRUE;

		sprintf(myResponseBuf, "\nManuf Mode Active");
		putString(myOutputCommPort, myResponseBuf);

		displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?BPT - Request the base plate temperature
ushort rBptHandler(void)
{
	float myFloat;

	myFloat = basePlateTempGet();

	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%3.f", myCmdToken, myFloat);
	else
		sprintf(myBuf, "BT=%d", (ushort)myFloat);

	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?C - request Laser Drive Control Mode
ushort rCHandler(void)
{
	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
	{
		sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.c);
	}
	else
	{
		sprintf(myBuf, "ANA=%d", sysCfgShadowRegs.c);		
	}
	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}


/////////////////////////////////////
// ?CC - request Computer Control
ushort rCcHandler(void)
{
	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%d", myCmdToken, isAutoStartActive);
	else
		sprintf(myBuf, "M=%d", isAutoStartActive);
	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}


/////////////////////////////////////
// ?DELAY - Request 5 Second Emission Delay Status
ushort rDelayHandler(void)
{
	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
	{
		sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.delay);
	}
	else
	{
		sprintf(myBuf, "CDRH=%d", sysCfgShadowRegs.delay);
	}
	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
//?DFBLC -  Request DFB Current
ushort rDfblcHandler(void)
{
	sprintf(myBuf, "\n%s=%3.1f", myCmdToken, extA2dCurrentValues[EXT_A2D_DFB_CURR] * 0.1224);
	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
//?DFBLCS - Request DFB Current Setting
ushort rDfblcsHandler(void)
{
	sprintf(myBuf, "\n%s=%3.1f", myCmdToken,sysCfgShadowRegs.lc);
	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?ECHO - Request Echo Status
ushort rEchoHandler(void)
{
	if(sysCfgShadowRegs.protoMode==EE_DEF_PROTO_VORT)
	{
		sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.echo);
	}
	else
	{
		 sprintf(myBuf, "E=%d", sysCfgShadowRegs.echo);
	}
	putString(myOutputCommPort, myBuf);

	 displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?EPC - Request External Power Control Status
ushort rEpcHandler(void)
{
	if(sysCfgShadowRegs.protoMode==EE_DEF_PROTO_VORT)
	{
		sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.epc);
	}
	else
	{
		sprintf(myBuf, "EXT=%d", sysCfgShadowRegs.epc);
	}

	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);

	return 0;
}



/////////////////////////////////////
// ?FC - Request Fault Code status
ushort rFcHandler(void)
{
	ulong myErrFlags = 0;

	//myErrFlags = prevErrLogFlagGet();//Vortran-DMS 11/19/09
	myErrFlags = errLogFlagGet();//Use current error flags Vortran-DMS 11/19/09

	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%d", myCmdToken, myErrFlags);
	else
		sprintf(myBuf, "F=%d", myErrFlags);

	putString(myOutputCommPort, myBuf);

	//errLogFlagClearAll();	// clear the errors Commented out Vortran-DMS 11/19/09

 	displayPrompt(myOutputCommPort);

	return 0;
}


/////////////////////////////////////
// ?FD - Request fault Description
ushort rFdHandler(void)
{
	errLogDiscriptionGet(myOutputCommPort);

	//errLogFlagClearAll();	// clear the errors

 	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?FF - Request Fault Code (Binary) and status
ushort rFfHandler(void)
{
	ulong myErrFlags = 0;
	ulong myBit = 0x00008000;
	ulong brandXFlags=0;

	if(sysCfgShadowRegs.protoMode != EE_DEF_PROTO_VORT)
	{
		myErrFlags = errLogFlagGet();

		if(myErrFlags == 0) //OK, On
			brandXFlags = 0;
		if(myErrFlags & 0x0001) //OK, Standby
			brandXFlags = 0;
		if(myErrFlags & 0x0004) //Out of Range
			brandXFlags |= 0x0002;
		if(myErrFlags & 0x0008)	//Bad Cmd
			brandXFlags |=0x0001;
		if(myErrFlags & 0x0010) //Interlock
			brandXFlags |= 0x0020; //32
		if(myErrFlags & 0x0040)//Diode Curr
		    brandXFlags |= 0x0800; //2048
		if(myErrFlags & 0x0080)//Diode Temp
			brandXFlags |=0x0010; //16
		if(myErrFlags & 0x0100)//Baseplate Temp
			brandXFlags |=0x0008; //8
		if(myErrFlags & 0x0200) //EEPROM
			brandXFlags |= 0x0004;//4
		if(myErrFlags & 0x0400) //I2C
			brandXFlags |= 0x0040;//64
		// output error flags in binary: MSBit - LSBit
		while(myBit)
		{
		 	if(brandXFlags & myBit)
				putString(myOutputCommPort, "1");
			else
				putString(myOutputCommPort, "0");

			// place a space every 8 bits for read-ability
			//if(myBit & 0x01000000 || myBit & 0x00010000	|| myBit & 0x00000100)
				//putString(myOutputCommPort, " ");

			myBit = myBit >> 1;
		}
	}

 	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?FP - Request Firmware Protocol
ushort rFpHandler(void)
{
	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%s", myCmdToken, SW_PROTOCOL_VERSION);
	else
		sprintf(myBuf, "%s", SW_PROTOCOL_VERSION);
	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?FV - Request Firmware Version
ushort rFvHandler(void)
{
	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%s", myCmdToken, SW_VERSION);
	else
		sprintf(myBuf, "%s", SW_VERSION);
	putString(myOutputCommPort, myBuf);
	//SW_VERSION

 	displayPrompt(myOutputCommPort);

	return 0;
}

////////////////////////////////////////////////////
// Help
ushort rHelpHandler(void)
{
	uchar myIndx = 0;

	if(myOutputCommPort==USB_PORT)
	{
	 	myOutputCommPort = USB_PORT_INIT;
	}

	putString(myOutputCommPort, "\n\nHelp Commands\n");	 //16
	sprintf(myBuf, "Sw: V%s, Hw: V%d", SW_VERSION, getBoardID()); //22
	putString(myOutputCommPort, myBuf);

	putString(myOutputCommPort, "\n");

	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
	{
		while(helpTableVort[myIndx][0] != 0)
		{
			sprintf(myBuf, "%s\n", &helpTableVort[myIndx++][0]);
			putString(myOutputCommPort, myBuf);
		}
	}
	else
	{
//#if 0 // FRG ADD BACK!!
		while(helpTableBrandX[myIndx][0] != 0)
		{
			sprintf(myBuf, "%s\n", &helpTableBrandX[myIndx++][0]);
			putString(myOutputCommPort, myBuf);
		}
//#endif
	}

	if(isManufModeActive == TRUE)	// the following commands are only active in manufacturing mode
	{
		putString(myOutputCommPort, 	"\n");
		putString(myOutputCommPort, 	"?HC - Request Help File: Calibration\n");
		putString(myOutputCommPort, 	"?HD - Request Help File: Diagnostics\n");
	}

	displayPrompt(myOutputCommPort);
	return 0;
	}

//////////////////////////////////////
// ?HLC - Request Heater Current
ushort rHlcHandler(void)
{
	sprintf(myBuf, "\n%s=%3.1f", myCmdToken, extA2dCurrentValues[EXT_A2D_HEATER_CURR] * 0.1224);
	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?IL - Request Interlock Status
ushort rIlHandler(void)
{
	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
	{
		if(IO0PIN & PIN_EXT_INTERLOCK)
			sprintf(myBuf, "\n%s=1", myCmdToken);
		else
			sprintf(myBuf, "\n%s=0", myCmdToken);
	}
	else
	{
		if(IO0PIN & PIN_EXT_INTERLOCK)
			sprintf(myBuf, "INT=1");
		else
			sprintf(myBuf, "INT=0");
	}

	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?LC - Request Laser Current Control
ushort rLcHandler(void)
{
	//getAllExtA2dRegs();
	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%3.1f", myCmdToken, extA2dCurrentValues[EXT_A2D_SOA_CURR] * 0.1224);			 // yes
	else
		sprintf(myBuf, "C=%.2f", extA2dCurrentValues[EXT_A2D_SOA_CURR] * 0.1224);			 // yes

	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);

	return 0;
}


/////////////////////////////////////
// ?LCS - Request the Laser Current
ushort rLcsHandler(void)
{
	sprintf(myBuf, "\n%s=%3.1f", myCmdToken, sysCfgShadowRegs.lc);			 // yes
	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?LE - Request Laser Enable
ushort rLeHandler(void)
{
	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%d", myCmdToken, isLsrEnableActive);
	else
	    sprintf(myBuf, "L=%d", isLsrEnableActive);

	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?LH - Request Laser Operating Hours
ushort rLhHandler(void)
{
	//sprintf(myBuf, "?LH=%f", sysCfgShadowRegs.laserHours);

	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
	{
		if(((ushort)(sysCfgShadowRegs.laserHours * 10) % 2) == 0)	// convert to 12 min resolution "0.2", 0.4, 0.6, ...
			sprintf(myBuf, "\n%s=%.1f", myCmdToken, sysCfgShadowRegs.laserHours);
		else
			sprintf(myBuf, "\n%s=%.f", myCmdToken, sysCfgShadowRegs.laserHours + 0.1);	// convert "odd" 10th digit to an even value by roll up. 0.1 => 0.2
    }
	else
		sprintf(myBuf, "HH=%.2f", sysCfgShadowRegs.laserHours);

	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// request Laser Identification
//
// VLmmyy####, #####, ###nm, ###mW, C
ushort rLiHandler(void)
{
	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%s, %s, %dnm, %dmW, %s", myCmdToken, calShadowRegs.lsrSerialNum, calShadowRegs.lsrPartNum, calShadowRegs.nominalWaveLen, calShadowRegs.nominalLsrPwr, calShadowRegs.lsrBeamShape);
	else
		sprintf(myBuf, "%s  %s %d", calShadowRegs.lsrSerialNum, calShadowRegs.lsrPartNum, calShadowRegs.nominalWaveLen);


	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?LP -
ushort rLpHandler(void)
{
	// ?LP = 12bit A/DVin1(I2C) * (maxPwrMw / 4095)

	//getAllExtA2dRegs();

	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%3.1f", myCmdToken, extA2dValues[EXT_A2D_LSR_PWR] * (calShadowRegs.maxPwrLsr / 4095.0));			 // yes
	else
		sprintf(myBuf, "P=%.2f", extA2dValues[EXT_A2D_LSR_PWR] * (calShadowRegs.maxPwrLsr / 4095.0));			 // yes

	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);
	return 0;
}

/////////////////////////////////////
// ?LPS -
ushort rLpsHandler(void)
{
 	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%3.1f", myCmdToken, sysCfgShadowRegs.lp);
	else
		sprintf(myBuf, "SP=%.2f", sysCfgShadowRegs.lp);

	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?LS - Laser Status
ushort rLsHandler(void)
{

	// ?C
	sprintf(myBuf, "\n?C=%d\n", sysCfgShadowRegs.c);
	putString(myOutputCommPort, myBuf);

	// ?LPS
	//getAllExtA2dRegs();

	sprintf(myBuf, "?LPS=%f\n", sysCfgShadowRegs.lp);			 // yes
	putString(myOutputCommPort, myBuf);

	// ?LCS
	sprintf(myBuf, "?LCS=%3.1f\n", sysCfgShadowRegs.lc);			 // yes
	putString(myOutputCommPort, myBuf);

  // ?EPC
	sprintf(myBuf, "?EPC=%d\n", sysCfgShadowRegs.epc);
	putString(myOutputCommPort, myBuf);

  // ?Delay
	sprintf(myBuf, "?DELAY=%d", sysCfgShadowRegs.delay);
	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}


/////////////////////////////////////
// ?LW - Request Laser Wavelength
ushort rLwHandler(void)
{
	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%d", myCmdToken, calShadowRegs.measWaveLen);
	else
		sprintf(myBuf, "WAVE=%d", calShadowRegs.measWaveLen);
	putString(myOutputCommPort, myBuf);

 	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?MAXP - Request Maximum Laser Power
ushort rMaxpHandler(void)
{
	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myResponseBuf, "\n%s=%3.1f", myCmdToken, calShadowRegs.maxPwrLsr);
	else
 		sprintf(myResponseBuf, "MAXLP=%.2f", calShadowRegs.maxPwrLsr);

	putString(myOutputCommPort, myResponseBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?MCDRH - Request monitor laser status for CDRH
ushort rMonCdrhHandler(void)
{
	sprintf(myResponseBuf, "\n%s=%d", myCmdToken, monCdrhHandlerMode);
	putString(myOutputCommPort, myResponseBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?MSTDB - Request monitor laser status for Stand By
ushort rMonStdbHandler(void)
{
	sprintf(myResponseBuf, "\n?MSTDB=%d", monStdbHandlerMode);
	putString(myOutputCommPort, myResponseBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?MINTL - Request monitor laser status for Interlock open
ushort rMonIntlHandler(void)
{
	sprintf(myResponseBuf, "\n%s=%d", myCmdToken, monIntlHandlerMode);
	putString(myOutputCommPort, myResponseBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?MDICU - Request monitor laser status for Diode Current
ushort rMonDiCuHandler(void)
{
	sprintf(myResponseBuf, "\n%s=%d", myCmdToken, monDiCuHandlerMode);
	putString(myOutputCommPort, myResponseBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?MDITM - Request monitor laser status for Diode Temperature
ushort rMonDTmpHandler(void)
{
	sprintf(myResponseBuf, "\n%s=%d", myCmdToken, monDTmpHandlerMode);
	putString(myOutputCommPort, myResponseBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?MBPLT - Request monitor laser status for Base Plate Temperature
ushort rMonBPltHandler(void)
{
	sprintf(myResponseBuf, "\n%s=%d", myCmdToken, monBPltHandlerMode);
	putString(myOutputCommPort, myResponseBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?OBT - Request optical Block temperature setting
ushort rObtHandler(void)
{
	ushort myVal;
	float myFloat = 0;
	volatile short myTicks = 0;

#ifndef SIMULATOR_ACTIVE
	// get the Cold Block Value
	a2dChnlStart(A2D_CHNL_COLD_BLOCK);

 	timer0Init(MAX_TIMEOUT_A2D);
	timer0Start();

	while(isA2d0ConvDone == FALSE)
	{
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			swError(__FILE__, __LINE__);
		 	break;
		}
	}

	timer0Stop();

	myVal = a2dChnlRead();
#endif
	myFloat = (myVal * 0.02933) + 10;		// (COLD BLOCK TEMP, (Internal ADC0.1) * .02933) +10

	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%3.1f", myCmdToken, myFloat);
	else
		sprintf(myBuf, "DT=%d", (ushort)myFloat);

	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ? OBTS - Request optical Block temperature setting	saved in eeprom
ushort rObtsHandler(void)
{

	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.vobts);
	else
		sprintf(myBuf, "DST=%.2f",(float) sysCfgShadowRegs.vobts);

	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////
// ?OP1- Operating Status 1 - Condensed System Status Information to help GUI performance, Part 1
ushort rOpStat1Handler(void)
{
	ushort myBit = 0x8000;
	ushort binStat=0x0000;
	uchar* myChar;
	uchar tempBuffer[20];

	memset(tempBuffer, 0, sizeof(tempBuffer));

	binStat |= isLsrEnableActive;

	binStat |= sysCfgShadowRegs.c << 1;

	binStat |= sysCfgShadowRegs.epc << 2;

	binStat |= sysCfgShadowRegs.pulseMode << 3;

	binStat |= sysCfgShadowRegs.isTecOn << 4;

	binStat |= sysCfgShadowRegs.delay << 5;

	binStat |= calShadowRegs.mode << 10; //6 most significant bits

	myChar = tempBuffer;

	while(myBit)
	{
		if(binStat & myBit)
			*myChar='1';
		else
			*myChar='0';
		myChar++;

		myBit = myBit >> 1;
	}

	*myChar=0x00; //Terminate String

	sprintf(myBuf, "\n%s=%s", myCmdToken,tempBuffer);
	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;

}

////////////////////////////////////
//?OP2-Operating Status 2- Condensed System Status Information to help GUI performance, Part 2
ushort rOpStat2Handler(void)
{
	float myBptFloat;
	float myObtFloat = 0;
	volatile short myTicks = 0;
	ushort myVal;

	// get the Cold Block Value
	a2dChnlStart(A2D_CHNL_COLD_BLOCK);

	timer0Init(MAX_TIMEOUT_A2D);
	timer0Start();

	while(isA2d0ConvDone == FALSE)
	{
		if((myTicks = timer0ReadTicks()) > MAX_I2C_TIMEOUT_IN_TICKS)
		{
			swError(__FILE__, __LINE__);
			 break;
		}
	}

	timer0Stop();

	myVal = a2dChnlRead();

	myObtFloat = (myVal * 0.02933) + 10; // (COLD BLOCK TEMP, (Internal ADC0.1) * .02933) +10

	myBptFloat = basePlateTempGet();

	//?LPS*?LP*?LCS*?LC*?PP*?OBT*?BPT
	sprintf(myBuf, "\n%s=%3.1f*%3.1f*%3.1f*%3.1f*%d*%3.1f*%3.f", myCmdToken,sysCfgShadowRegs.lp,extA2dValues[EXT_A2D_LSR_PWR] * (calShadowRegs.maxPwrLsr / 4095.0),sysCfgShadowRegs.lc,extA2dCurrentValues[EXT_A2D_SOA_CURR] * 0.1224,sysCfgShadowRegs.peakPower,myObtFloat,myBptFloat);

	putString(myOutputCommPort, myBuf);
	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
//?OP3-Operating Status 3- Condensed System Status Information to help GUI performance, Part 3
ushort rOpStat3Handler(void)
{
    //?FV, ?MAXP, ?RP, ?LW, ?FP
	sprintf(myBuf, "\n%s=%s*%3.1f*%d*%d*%s",myCmdToken,SW_VERSION,calShadowRegs.maxPwrLsr,calShadowRegs.nominalLsrPwr,calShadowRegs.measWaveLen,SW_PROTOCOL_VERSION);

	putString(myOutputCommPort, myBuf);
	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
//?OP4-Operating Status 4- Condensed System Status Information to help GUI performance, Part 4
ushort rOpStat4Handler(void)
{
	ushort myBit = 0x8000;
	ushort binStat=0x0000;
	uchar* myChar;
	uchar tempBuffer[20];

	ulong myErrFlags = 0;

	myErrFlags = errLogFlagGet();

	memset(tempBuffer, 0, sizeof(tempBuffer));

	binStat |= isLsrEnableActive;

	binStat |= sysCfgShadowRegs.c << 1;

	binStat |= sysCfgShadowRegs.epc << 2;

	binStat |= sysCfgShadowRegs.pulseMode << 3;

	binStat |= sysCfgShadowRegs.isTecOn << 4;

	binStat |= sysCfgShadowRegs.delay << 5;

	binStat |= calShadowRegs.mode << 10; //6 most significant bits

	myChar = tempBuffer;

	while(myBit)
	{
		if(binStat & myBit)
			*myChar='1';
		else
			*myChar='0';
		myChar++;

		myBit = myBit >> 1;
	}

	*myChar=0x00; //Terminate String

	sprintf(myBuf, "\n%s=%s*%d*%.1f",myCmdToken,tempBuffer,myErrFlags,sysCfgShadowRegs.laserHours);
	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
//?OP5-Operating Status 5- Condensed System Status Information for QD Laser
ushort rOpStat5Handler(void)
{
   //?DFBLC, SOA Current, Heater Current
	sprintf(myBuf, "\n%s=%3.1f*%3.1f*%3.1f",myCmdToken,extA2dCurrentValues[EXT_A2D_DFB_CURR]*0.1224,extA2dCurrentValues[EXT_A2D_SOA_CURR]*0.1224,extA2dCurrentValues[EXT_A2D_HEATER_CURR]*0.1224);

	putString(myOutputCommPort, myBuf);
	displayPrompt(myOutputCommPort);

	return 0;
}


/////////////////////////////////////
// ?PSA - Request the I/O Port Expander Output Pin settings
ushort rPsaHandler(void)
{
	short myVal;

	myVal = extPsaGet();

	sprintf(myBuf, "\n%s=%d", myCmdToken, myVal);
	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?RP - Request rated Laser Power
ushort rRpHandler(void)
{
	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%d", myCmdToken, calShadowRegs.nominalLsrPwr);
	else
		sprintf(myBuf, "NOMP=%d", calShadowRegs.nominalLsrPwr);
	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
//OP Operating Mode Brand X Only
ushort rOpHandler(void)
{
	if(sysCfgShadowRegs.protoMode != EE_DEF_PROTO_VORT)
	{
		sprintf(myBuf, "OP=1");
		putString(myOutputCommPort, myBuf);
	}
		
	displayPrompt(myOutputCommPort);
 	return 0;
}

/////////////////////////////////////
// ?PP - Peak Power Mode
ushort rPpHandler(void)
{
	sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.peakPower);
	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

/////////////////////////////////////
// ?PUL - Pulse Mode
ushort rPulHandler(void)
{
	sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.pulseMode);
	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

ushort rSsHandler(void)
{
	if(sysCfgShadowRegs.protoMode != EE_DEF_PROTO_VORT)
	{
 		sprintf(myBuf, "SS=%d", isTeCoolerOn);
	}
	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}

//////////////////////////////////////
// ?STA - Request Fault Code Status
ushort rStaHandler(void)
{
 	ulong myErrFlags = 0;
	uchar myStatus=0;

	myErrFlags = errLogFlagGet();

	if(sysCfgShadowRegs.protoMode != EE_DEF_PROTO_VORT)
	{
		//1=Warmup
		//2=Standby
		//3=Laser ON
		//4=Error
		//5=Fatal Error
		if(myErrFlags>=512 && myErrFlags< 32767) //Fatal Error
			myStatus=5;
		else
			if(myErrFlags & 0x00000002) //Warmup
				myStatus=1;
			else
				if(myErrFlags & 0x00000001) //Standby
					myStatus=2;
				else
					if(myErrFlags==0) //Laser On, No Faults
						myStatus=3;



		sprintf(myBuf, "STA=%d", myStatus);
		putString(myOutputCommPort, myBuf);
	}
 	displayPrompt(myOutputCommPort);

	return 0;
}


/////////////////////////////////////
// ?TEC - Request TEC Operating Status
ushort rTecHandler(void)
{
	if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		sprintf(myBuf, "\n%s=%d", myCmdToken, isTeCoolerOn);
	else
		sprintf(myBuf, "T=%d", isTeCoolerOn);
	putString(myOutputCommPort, myBuf);

	displayPrompt(myOutputCommPort);

	return 0;
}


ushort rWDogHandler(void)			// get status of previous watch dog reset
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myBuf, "\n%s=%d", myCmdToken, prevWDogOccurred);
		putString(myOutputCommPort, myBuf);

		displayPrompt(myOutputCommPort);
	}
	else
	{
		// log  the error, and display the prompt
		errLogFlagSet(ERR_INVALID_COMMAND);
		displayPrompt(myOutputCommPort);
	}

	return 0;
}

///////////////////////////////////////////////////
// Convert the inout String to upper case, and store in the Upper Case Buffer.
void strupr(char *myInputBuf, char *myUprCaseBuf)
{
	char myChar;
	volatile char *myTmp;

	myTmp = myUprCaseBuf;

	while(*myInputBuf)										// are there charcaters in the input buffer we need to process?
	{
		myChar = *myInputBuf;

		if(myChar >= 'a' && myChar <= 'z')	// is the character lower case?
		{
			myChar = *myInputBuf - 0x20;			 			// yes, convert to upper case!
			*myTmp = myChar;
		}
		else if(myChar >= 'A' && myChar <= 'Z')	// is the character upper case?
			*myTmp = myChar;
		else
			*myTmp = myChar;

		myTmp++;
		myInputBuf++;
	}
}

////////////////////////////
// R - Debug - Register Peek
ushort rHandler(void)
{
	ulong myULong;

	if(isManufModeActive == TRUE)	// the following commands are only active in manufacturing mode
	{
		putString(myOutputCommPort, "\n");

		myULong = atol(arg1);

		if(myULong == 30)
		{
	 		readAllExtA2dRegs(myOutputCommPort);
		}
		else if(myULong == 60)	// internal A2D
		{
	 		readAllA2dChnls(myOutputCommPort);		// cold Block and Base Plate
		}
		else if(myULong == 70)	// internal D2A
		{
	 		readD2aChnl(myOutputCommPort);		//
		}
		else if(myULong == 80)
		{
	 		readAllExtPotsModRegs(myOutputCommPort);
		}
		else if(myULong == 85) //extQuadD2a
		{
			readAllExtD2aRegs(myOutputCommPort);
		}
		else if(myULong == 90) //extDualPotsLsrDfb
		{
			readAllExtPotsLsrDfbRegs(myOutputCommPort);
		}
		else if(myULong == 95) //extSinglePotLsr
		{
		 	readAllExtSingPotLsrRegs(myOutputCommPort);
		}
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

////////////////////////////
// W - Debug - Register poke
ushort wHandler(void)
{
	ulong myAddr;
	ulong myData;

	if(isManufModeActive == TRUE)	// the following commands are only active in manufacturing mode
	{
		myAddr = atoll(arg1);		 // get the first argument

		displayPrompt(myOutputCommPort);

		myData = atoll(arg2);		 // get the second argument

		// write to the external I2C device
		if(myAddr >= 10 && myAddr < 30)
			;
		else if(myAddr >= 30 && myAddr < 50)
			;	// can't write to the A/D
		else if(myAddr >= 50 && myAddr < 60)
			;
		else if(myAddr >= 70 && myAddr < 80)
			coldBlockTempSet(myData);
		else if(myAddr >= 80 && myAddr < 85)
			potsModWriteReg(myAddr,myData);
		else if(myAddr >= 85 && myAddr < 90) //extQuadD2A
			writeExtD2aReg(myAddr, myData);
		else if(myAddr >= 90 && myAddr < 95) //extDualPotsLsrDfb
			potsLsrDfbWriteReg(myAddr, myData);
		else if(myAddr >= 95 && myAddr < 100) //extSinglePotLsr
			potsLsrSingleWriteReg(myAddr,myData);

		sprintf(myBuf, "W %lu %lu", myAddr, myData);
		putString(myOutputCommPort, myBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

////////////////////////////
// HD - Debug - Help commands for Diagnostics
ushort hdHandler(void)
{
	if(myOutputCommPort==USB_PORT)
	{
	 	myOutputCommPort = USB_PORT_INIT;
	}
	putString(myOutputCommPort, "\n\nHelp Commands: Diagnostics\n");

	if(isManufModeActive == TRUE)	// the following commands are only active in manufacturing mode
	{
		putString(myOutputCommPort, "\n");
		putString(myOutputCommPort, "burn - eeprom set to default images\n");
		putString(myOutputCommPort, "cpswerr - clear previous sw errors\n");
		putString(myOutputCommPort, "dsyscfg - display eeprom system and cal config\n");
		putString(myOutputCommPort, "forcever - force the eeprom version for debugging purposes\n");
		putString(myOutputCommPort, "dpswerr - display previous sw errors\n");
		putString(myOutputCommPort, "swerr - display sw errors\n");
		putString(myOutputCommPort, "cp [pin] - clear port 0 pin: low\n");
		putString(myOutputCommPort, "gp - get port 0 pins\n");
		putString(myOutputCommPort, "lh=[value] - set laser hours\n");
		putString(myOutputCommPort, "obtd=[0-255] - OBT Temperature Deviation\n");
		putString(myOutputCommPort, "psa=[0-F] - Output pins: pulse mode D0,D1,D2,D3\n");
		putString(myOutputCommPort, "sp [pin] - set port 0 pin: high\n");
		putString(myOutputCommPort, "tecmode=[0/1] - if there is no TEC HW set to 0\n");
		putString(myOutputCommPort, "wdog - stop kicking the watch dog. Reset shall occur\n");
		putString(myOutputCommPort, "?wdog - get status of previous watchdog reset\n");
		putString(myOutputCommPort, "r [Addr] - register read\n");
		putString(myOutputCommPort, "w [Addr] [Val] - register write\n");
		putString(myOutputCommPort, "  Addr 1x D2A\n");
		putString(myOutputCommPort, "  Addr 3x A2D\n");
		putString(myOutputCommPort, "  Addr 5x POTs Lsr\n");
		putString(myOutputCommPort, "  Addr 6x internal A/D\n");
		putString(myOutputCommPort, "  Addr 7x internal D/A\n");
		putString(myOutputCommPort, "  Addr 8x POTs Mod\n");

		putString(myOutputCommPort, "mcdrh=[0/1]  - monitor cdrh\n");
		putString(myOutputCommPort, "mstdb=[0/1]  - monitor Stand By\n");
		putString(myOutputCommPort, "mintl=[0/1]  - monitor Interlock Open\n");
		putString(myOutputCommPort, "mdicu=[0/1]  - monitor Diode Current\n");
		putString(myOutputCommPort, "mditm=[0/1]  - monitor Diode Temperature\n");
		putString(myOutputCommPort, "mbplt=[0/1]  - monitor Base Plate Temperature\n");
	}

	displayPrompt(myOutputCommPort);
	return 0;
}

////////////////////////////
// LH - Debug - Set Laser Hours
ushort lhHandler(void)
{
	ulong myHours;

	if(isManufModeActive == TRUE)	// the following commands are only active in manufacturing mode
	{

		myHours = atoll(arg1);		 // get the first argument

		// Save the laser on time to non volatile
		sysCfgShadowRegs.laserHours = myHours;
		saveSysCfgDelta(EE_AREA_SYS_CFG);

		sprintf(myBuf, "\nLH=%d", myHours);
		putString(myOutputCommPort, myBuf);
	}
	else
		errLogFlagSet(ERR_INVALID_COMMAND);

	displayPrompt(myOutputCommPort);

	return 0;
}

////////////////////////////
// SP - Debug - Port 0 set
ushort spHandler(void)
{
	ulong myPin;
	short myStatus = PASS;

	if(isManufModeActive == TRUE)	// the following commands are only active in manufacturing mode
	{
		putString(myOutputCommPort, "\n");

		myPin = atoll(arg1);		 // get the first argument

		displayPrompt(myOutputCommPort);

		if(myPin == 16)
			IO0SET = PIN_TE_ON;
		else if(myPin == 17)
			IO0SET = PIN_FAULT_INDC;
		else if(myPin == 18)
			IO0SET = PIN_LSR_RDY;
		else if(myPin == 19)
			IO0SET = PIN_LIGHT_MOD_ON;
		else if(myPin == 20)
			IO0SET = PIN_I_MOD_ON;
		else if(myPin == 21)
		{
			IO0SET = PIN_LSR_ON;
			lsrOnStopWatch(LSR_STOP_WATCH, START);
		}
		else if(myPin == 22)
 			IO0SET = PIN_LSR_EMISSION_LED;
		else
			myStatus = FAIL;

	  if(myStatus == PASS)
		{
			sprintf(myBuf, "set port %d pin complete\n", myPin);
			putString(myOutputCommPort, myBuf);
		}
		else
		{
			sprintf(myBuf, "invalid pin!!\n");
			putString(myOutputCommPort, myBuf);
		}
	}
	else
		errLogFlagSet(ERR_INVALID_COMMAND);

	displayPrompt(myOutputCommPort);

	return 0;
}



////////////////////////////
// CP - Debug - Port 0 clear
ushort cpHandler(void)
{
	ulong myPin;
	short myStatus = PASS;

	if(isManufModeActive == TRUE)	// the following commands are only active in manufacturing mode
	{
		putString(myOutputCommPort, "\n");

		myPin = atoll(arg1);		 // get the first argument

		displayPrompt(myOutputCommPort);

		if(myPin == 16)
			IO0CLR = PIN_TE_ON;
		else if(myPin == 17)
			IO0CLR = PIN_FAULT_INDC;
		else if(myPin == 18)
			IO0CLR = PIN_LSR_RDY;
		else if(myPin == 19)
			IO0CLR = PIN_LIGHT_MOD_ON;
		else if(myPin == 20)
		{
			IO0CLR = PIN_I_MOD_ON;
			lsrOnStopWatch(LSR_STOP_WATCH, STOP);
		}
		else if(myPin == 21)
		{
			IO0CLR = PIN_LSR_ON;
			lsrOnStopWatch(LSR_STOP_WATCH, STOP);
		}
		else if(myPin == 22)
			IO0CLR = PIN_LSR_EMISSION_LED;
		else
			myStatus = FAIL;

	  if(myStatus == PASS)
		{
			sprintf(myBuf, "clear port 0 pin complete\n");
			putString(myOutputCommPort, myBuf);
		}
		else
		{
			sprintf(myBuf, "invalid pin!!\n");
			putString(myOutputCommPort, myBuf);
		}
	}
	else
		errLogFlagSet(ERR_INVALID_COMMAND);

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Clear the previous software errors stored in eeprom
ushort cpSwErrHandler(void)
{
	if(isManufModeActive == TRUE)	// the following commands are only active in manufacturing mode
	{
		saveSwErrDefaults();
		putString(myOutputCommPort, "Cleared\n");
		displayPrompt(myOutputCommPort);
	}

	return 0;
}

////////////////////////////
// GP - Debug - Get Port 0 Pins
ushort gpHandler(void)
{
	if(myOutputCommPort==USB_PORT)
	{
	 	myOutputCommPort = USB_PORT_INIT;
	}

	if(isManufModeActive == TRUE)	// the following commands are only active in manufacturing mode
	{
		putString(myOutputCommPort, "\n");

		if(IO0PIN & PIN_EXT_INTERLOCK)
			sprintf(myBuf, "P0.10 = 1  IntrLock\n");
		else
			sprintf(myBuf, "P0.10 = 0  IntrLock\n");

		putString(myOutputCommPort, myBuf);


		if(IO0PIN & PIN_LSR_EN)
			sprintf(myBuf, "P0.11 = 1  LsrEn\n");
		else
			sprintf(myBuf, "P0.11 = 0  LsrEn\n");

		putString(myOutputCommPort, myBuf);


		if(IO0PIN & PIN_AUTO_START)
			sprintf(myBuf, "P0.12 = 1  AutoStart\n");
		else
			sprintf(myBuf, "P0.12 = 0  AutoStart\n");

		putString(myOutputCommPort, myBuf);

		if(IO0PIN & PIN_TE_ON)
			sprintf(myBuf, "P0.16 = 1  TE\n");
		else
			sprintf(myBuf, "P0.16 = 0  TE\n");

		putString(myOutputCommPort, myBuf);

		if(IO0PIN & PIN_FAULT_INDC)
			sprintf(myBuf, "P0.17 = 1  Fault Indc\n");
		else
			sprintf(myBuf, "P0.17 = 0  fault Indc\n");

		putString(myOutputCommPort, myBuf);

		if(IO0PIN & PIN_LSR_RDY)
			sprintf(myBuf, "P0.18 = 1  Laser Ready\n");
		else
			sprintf(myBuf, "P0.18 = 0  Laser Ready\n");

		putString(myOutputCommPort, myBuf);

		if(IO0PIN & PIN_LIGHT_MOD_ON)
			sprintf(myBuf, "P0.19 = 1 Light Mod\n");
		else
			sprintf(myBuf, "P0.19 = 0  Light Mod\n");

		putString(myOutputCommPort, myBuf);
		if(IO0PIN & PIN_I_MOD_ON)
			sprintf(myBuf, "P0.20 = 1  Current Mod\n");
		else
			sprintf(myBuf, "P0.20 = 0  Current Mod\n");

		putString(myOutputCommPort, myBuf);

		if(IO0PIN & PIN_LSR_ON)
			sprintf(myBuf, "P0.21 = 1  Lsr On\n");
		else
			sprintf(myBuf, "P0.21 = 0  Lsr On\n");

		putString(myOutputCommPort, myBuf);

		if(IO0PIN & PIN_LSR_EMISSION_LED)
			sprintf(myBuf, "P0.22 = 1  Lsr Emission\n");
		else
			sprintf(myBuf, "P0.22 = 0  Lsr Emission\n");

		putString(myOutputCommPort, myBuf);
	}
	else
		errLogFlagSet(ERR_INVALID_COMMAND);


	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Display the system config values
ushort dSysCfgHandler(void)
{

	if(myOutputCommPort==USB_PORT)
	{
	 	myOutputCommPort = USB_PORT_INIT;
	}

	if(isManufModeActive == TRUE)
	{
		// System Configuration values
		sprintf(myBuf, "\nSysCfg Version:	%d\n", sysCfgShadowRegs.version);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "lp:			%f\n", sysCfgShadowRegs.lp);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "c:			%d\n", sysCfgShadowRegs.c);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "lc:			%f\n", sysCfgShadowRegs.lc);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "epc:			%d\n", sysCfgShadowRegs.epc);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "delay:			%d\n", sysCfgShadowRegs.delay);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "vobts:			%d\n", sysCfgShadowRegs.vobts);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "isTecOn:		%d\n", sysCfgShadowRegs.isTecOn);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "echo:			%d\n", sysCfgShadowRegs.echo);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "prompt:			%d\n", sysCfgShadowRegs.prompt);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "Laser hours:		%f\n", sysCfgShadowRegs.laserHours);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "Peak Power:			%d\n", sysCfgShadowRegs.peakPower);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "Pulse Mode:			%d\n", sysCfgShadowRegs.pulseMode);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "DFB Current:	    %f\n", sysCfgShadowRegs.dfblc);
		putString(myOutputCommPort, myBuf);


		//sprintf(myBuf, "Laser Id:		%s\n", sysCfgShadowRegs.laserId);
		//putString(myOutputCommPort, myBuf);

		putString(myOutputCommPort, "\n");

		// Calibration values
		sprintf(myBuf, "Cal Version:		%d\n", calShadowRegs.version);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "maxPwrLsr:		%f\n", calShadowRegs.maxPwrLsr);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "initLsrCur:		%d\n", 		calShadowRegs.initLsrCur);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "nominalLsrPwr:		%d\n", 		calShadowRegs.nominalLsrPwr);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "maxLsrCur:		%d\n", 		calShadowRegs.maxLsrCur);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "nominalWaveLen:		%d\n", 		calShadowRegs.nominalWaveLen);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "measWaveLen:		%d\n", 		calShadowRegs.measWaveLen);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "lsrPwrSetPot:		%d\n", 		calShadowRegs.lsrPwrSetPot);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "lsrCurSetPot:		%d\n", 		calShadowRegs.lsrCurSetPot);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "lsrPartNum:		%s\n", 		calShadowRegs.lsrPartNum);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "lsrSerialNum:		%s\n", 		calShadowRegs.lsrSerialNum);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "lsrBeamShape:		%s\n", 		calShadowRegs.lsrBeamShape);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "Cal DAC Value:		%d\n", 		calShadowRegs.calDacValue);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "LC Scale Factor:		%d\n", 		calShadowRegs.lcScaleFactor);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "EPC Scale Factor (RDAC3, Coarse):	%d\n", 		calShadowRegs.epcScaleFactor);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "LP Offset Adjust:		%d\n", 		calShadowRegs.lpOffsetAdjust);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "EOL Fault Enable:		%d\n", 		calShadowRegs.endOfLifeEnable);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "EOL Fault Line:		%d\n", 		calShadowRegs.endOfLifeFaultLine);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "Non-Volatile PUL Mode:		%d\n", 		calShadowRegs.nonVolPUL);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "EPC Scale Factor 2 (RDAC1, Fine):	%d\n", 		calShadowRegs.epcScaleFactor2);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "Op Mode:	        %d\n", calShadowRegs.mode);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "RF Injection Enable:	        %d\n", calShadowRegs.rfInj);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "Max DFB Curr: %d\n",calShadowRegs.maxCurrDfb);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "Max Heater Curr: %d\n",calShadowRegs.maxCurrHtr);
		putString(myOutputCommPort, myBuf);
		sprintf(myBuf, "Heater DAC Counts:	    %d\n", calShadowRegs.htrCnt);
		putString(myOutputCommPort, myBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

ushort forceVerHandler(void)
{
	short myVersion;

	if(isManufModeActive == TRUE)	// the following commands are only active in manufacturing mode
	{
		putString(myOutputCommPort, "\n");

		myVersion = atoi(arg1);		 // get the first argument

		sysCfgShadowRegs.version = myVersion;
		saveSysCfgDelta(EE_AREA_SYS_CFG);

		sprintf(myBuf, "FORCEVER=%d", myVersion);
		putString(myOutputCommPort, myBuf);
	}
	else
		errLogFlagSet(ERR_INVALID_COMMAND);

	displayPrompt(myOutputCommPort);

	return 0;

}


///////////////////////////////////////////////////
// Display the previous software errors	in eeprom
ushort dpSwErrHandler(void)
{
	uchar myErrNum = 0;
	uchar myIndx = 0;
	ushort myEeAddr = 0;
	uchar myEeTmpBuf[MAX_SW_ERR_LEN];

#if VORT_BOARD
	if(isManufModeActive == TRUE)
	{
		for(myErrNum = 0; myErrNum < MAX_SW_ERRORS; myErrNum++)
		{
			myIndx = 0;
			myEeAddr = EE_SW_ERR_START + (MAX_SW_ERR_LEN * myErrNum);	// determine the eeprom offset in the sw errors image area

			eepromRead(myEeAddr++, &myEeTmpBuf[myIndx]);	// get information from eeprom

#if VORT_BOARD
			DelayUs(20);
#else
			delayMs(5);
#endif

			while(myEeTmpBuf[myIndx] != 0 && (myIndx < MAX_SW_ERR_LEN))
			{
				myIndx++;
				eepromRead(myEeAddr++, &myEeTmpBuf[myIndx]);	 // get information from eeprom

#if VORT_BOARD
				DelayUs(20);
#else
				delayMs(5);
#endif
			}

			myEeTmpBuf[myIndx] = 0;	// force end of string

			putString(myOutputCommPort, (char*)myEeTmpBuf);
		}
	}
	#endif

	displayPrompt(myOutputCommPort);

	return 0;
}


////////////////////////
// Calibration Commands
////////////////////////


///////////////////////////////////////////////////
// Calibration Version - manufacturing command
// Format #.##
ushort calCalVerHandler(void)
{
	float myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atof(arg1);

		if(myTmp >= 0 && myTmp <= 9.99)
		{
	 		calShadowRegs.calVersion = myTmp;

			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nCV=%1.2f", calShadowRegs.calVersion);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Set Calibration DAC Value Code Handler
ushort calDacHandler(void)
{
	ushort myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp <= 4095)
		{
			calShadowRegs.calDacValue = myTmp;

			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nCDAC=%d", calShadowRegs.calDacValue);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Set Calibration DFB DAC Value Code Handler
// May be used by calibration software for recalibration
// only if DFB current can be specified using DFBLC
ushort calDacDfbHandler(void)
{
	ushort myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp <= 4095)
		{
			calShadowRegs.calDacDfbValue = myTmp;

			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nCDACDFB=%d", calShadowRegs.calDacDfbValue);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

ushort calDfblcDacCntHandler()
{
	calDfblcDacCntHandlerProcess(VORT_PARSER);
	return 0;
}

void calDfblcDacCntHandlerProcess(ushort myMode)
{
	ushort myVal;

	if(myMode == VORT_PARSER)
		myVal = atoi(arg1);
	else
		myVal = sysCfgShadowRegs.dfblcDacCnt;


	if(myVal <= 4095)
	{
		if(!isErrLogStatGrtr16() || powerUp == TRUE) //Set DACs always at power-up
		{
			extQuadD2aDfbSetCur(myVal);

			if(myMode == VORT_PARSER)
			{
				sysCfgShadowRegs.dfblcDacCnt = myVal;
				saveSysCfgDelta(EE_AREA_SYS_CFG);
				sprintf(myBuf, "\n%s=%s", myCmdToken, arg1);
				putString(myOutputCommPort, myBuf);
			}
		}
		else 	// In a fault condition, the query will return the measured laser current.
		{
			if(myMode == VORT_PARSER)
			{
				errLogFlagSet(ERR_INVALID_COMMAND);	// error

				sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.dfblcDacCnt);
				putString(myOutputCommPort, myBuf);
			}
		}
	}
	else if(myMode == VORT_PARSER)
	{
		errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);	// error

		sprintf(myBuf, "\n%s=%d", myCmdToken, sysCfgShadowRegs.dfblcDacCnt);
		putString(myOutputCommPort, myBuf);
	}

	if(myMode == VORT_PARSER)
		displayPrompt(myOutputCommPort);
}

////////////////////////////////////////////////
//Calibration DFB Current Set Dac - manufacturing command
ushort calDfblcsdHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 4095)
		{
			extQuadD2aDfbSetCur(myTmp);
			delayMs(2);
			myTmp=extQuadD2aDfbGetCur(); //Confirm Setting

			sprintf(myResponseBuf, "\nDFBLCSD=%d", myTmp);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}





///////////////////////////////////////////////////
//Calibration DFB Current Set Pot- manufacturing command
ushort calDfblcspHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 255)
		{
		 	extDualPotsDfbSetCur((uchar)myTmp);
			delayMs(200);
			myTmp=extDualPotsDfbGetCur(); //Confirm Setting

			sprintf(myResponseBuf, "\nDFBLCSP=%d", myTmp);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}



///////////////////////////////////////////////////
// Calibration Maximum Power Laser - manufacturing command
ushort calMaxPwrLsrHandler(void)
{
	float myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atof(arg1);

		if(myTmp >= 0 && myTmp <= 1000)
		{
			calShadowRegs.maxPwrLsr = myTmp;

			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nMPL=%4f", calShadowRegs.maxPwrLsr);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Calibration Set Heater DAC - manufacturing command
// Sets heater DAC only, does not store value in EEPROM as HEAT does
ushort calHtrSetDacHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 4095)
		{
			extQuadD2aSetHeater(myTmp);
			delayMs(2);
			myTmp=extQuadD2aGetHeater();

			sprintf(myResponseBuf, "\nHSD=%d", myTmp);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Calibration Initial Laser Current - manufacturing command
ushort calInitLsrCurHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 1000)
		{
			calShadowRegs.initLsrCur = myTmp;

			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nILC=%4d", calShadowRegs.initLsrCur);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Calibration DFB Initial Laser Current  - manufacturing command
ushort calInitDfbCurHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 1000)
		{
			calShadowRegs.initDfbCur = myTmp;

			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nILCDFB=%4d", calShadowRegs.initDfbCur);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

	///////////////////////////////////////////////////
// Calibration Nominal Laser Power - manufacturing command
ushort calNomLsrPwrHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 1000)
		{
		 	calShadowRegs.nominalLsrPwr = myTmp;

			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nNLP=%4d", calShadowRegs.nominalLsrPwr);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Calibration Maximum Laser Current - manufacturing command
ushort calMaxLsrCurHandler(void)
{
 	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 1000)
		{
		 	calShadowRegs.maxLsrCur = myTmp;

			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nMLC=%4d", calShadowRegs.maxLsrCur);
			putString(myOutputCommPort, myResponseBuf);


		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Response: Calibration Maximum DFB Current - manufacturing command
ushort calMaxLsrCurDfbHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 1000)
		{
		 	calShadowRegs.maxCurrDfb = myTmp;
			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nMLCDFB=%4d", calShadowRegs.maxCurrDfb);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Response: Calibration Maximum Heater Current
ushort calMaxHtrCurHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 1000)
		{
		 	calShadowRegs.maxCurrHtr = myTmp;
			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nMHC=%4d", calShadowRegs.maxCurrHtr);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	return 0;
}

///////////////////////////////////////////////////
// Response: Calibration Maximum Heater Current
ushort calMaxOverCurHandler(void)
{
	ushort myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp <= 1000)
		{
		 	calShadowRegs.maxOverCurr = myTmp;
			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nMOC=%4d", calShadowRegs.maxOverCurr);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}
	displayPrompt(myOutputCommPort);
	return 0;
}




///////////////////////////////////////////////////
// Calibration Nominal Wave Length - manufacturing command
ushort calNomWaveLenHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 250 && myTmp <= 2000)
		{
		 	calShadowRegs.nominalWaveLen = myTmp;

			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nNWL=%4d", calShadowRegs.nominalWaveLen);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Calibration Measure Wave Length - manufacturing command
ushort calMeasWaveLenHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 1000)
		{
		 	calShadowRegs.measWaveLen = myTmp;

			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nMWL=%4d", calShadowRegs.measWaveLen);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}
///////////////////////////////////////////////////
// Calibration Laser Power Set DAC - manufacturing command
ushort calLsrPwrSetDacHandler(void)
{

	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 4095)
		{
			extQuadD2aSoaSetLight(myTmp);
			delayMs(2);
			myTmp=extQuadD2aSoaGetLight(); //Confirm Setting
			delayMs(2);

			sprintf(myResponseBuf, "\nLPSD=%d", myTmp);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Calibration Laser Power Set POT - manufacturing command
ushort calLsrPwrSetPotHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 255)
		{
		 	extSinglePotSetLtfb((uchar)myTmp);
			delayMs(200);
			myTmp=extSinglePotGetLtfb(); //Confirm Setting

			sprintf(myResponseBuf, "\nLPSP=%d", myTmp);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Calibration Laser Current Set DAC - manufacturing command
ushort calLsrCurSetDacHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 4095)
		{
			extQuadD2aSoaSetCur(myTmp);
			delayMs(2);
			myTmp=extQuadD2aSoaGetCur();

			sprintf(myResponseBuf, "\nLCSD=%d", myTmp);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Calibration Laser Current Set POT - manufacturing command
ushort calLsrCurSetPotHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 255)
		{
			extDualPotsSoaSetCur((uchar)myTmp);
			delayMs(200);
			myTmp=extDualPotsSoaGetCur();

			sprintf(myResponseBuf, "\nLCSP=%d", myTmp);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Calibration Laser Part Number - manufacturing command
ushort calLsrPartNumHandler(void)
{
	//char		lsrPartNum[6];

	if(isManufModeActive == TRUE)
	{
		if(strlen(arg1) <= 5 && strlen(arg1) > 0)
		{
		 	strcpy(calShadowRegs.lsrPartNum, arg1);

			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nLPN=%s", calShadowRegs.lsrPartNum);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Calibration Laser Serial Number - manufacturing command
ushort calLsrSerialNumHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		if(strlen(arg1) == 10)
		{
		 	strcpy(calShadowRegs.lsrSerialNum, arg1);

			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nLSN=%s", calShadowRegs.lsrSerialNum);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Calibration Laser Beam Shape - manufacturing command
ushort calLsrBeamShapeHandler(void)
{
	//char		lsrBeamShape[4];

	if(isManufModeActive == TRUE)
	{
		if(strlen(arg1) == 1)
		{
			if(*arg1 == 'C' || *arg1 == 'E')
			{
			 	strcpy(calShadowRegs.lsrBeamShape, arg1);

				saveSysCfgDelta(EE_AREA_CAL);

				sprintf(myResponseBuf, "\nLBS=%s", calShadowRegs.lsrBeamShape);
				putString(myOutputCommPort, myResponseBuf);
			}
			else
				errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Calibration OBT Deviation - manufacturing command
ushort calObtDeviationHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 255)
		{
		 	calShadowRegs.obtDeviation = atoi(arg1);

			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nOBTD=%d", calShadowRegs.obtDeviation);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Calibration Laser Current Scaling Factor - manufacturing command
ushort calLcScaleFactorHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp > 0 && myTmp <= 10000)
		{
		 	calShadowRegs.lcScaleFactor = atoi(arg1);
			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nLCSF=%d", calShadowRegs.lcScaleFactor);
			putString(myOutputCommPort, myResponseBuf);


		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;

}

///////////////////////////////////////////////////
// Calibration External Pwr Control Scaling Factor (Coarse Setting)
ushort calEpcScaleFactorHandler(void)
{
	short myTmp;


		myTmp = atoi(arg1);

		if(myTmp > 0 && myTmp <= 10000)
		{
		 	calShadowRegs.epcScaleFactor = atoi(arg1);
		    saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nEPCSF=%d", calShadowRegs.epcScaleFactor);
			putString(myOutputCommPort, myResponseBuf);

		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);


	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Calibration Laser Power Offset Adjust - manufacturing command
ushort calLpoaHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 100)
		{
		 	calShadowRegs.lpOffsetAdjust = atoi(arg1);
		    saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nLPOA=%d", calShadowRegs.lpOffsetAdjust);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Calibration End of Life Fault Enable/Disable - manufacturing command
ushort calEOLHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		if(*arg1 =='1')
		{
		 	calShadowRegs.endOfLifeEnable = 1;
			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nEOL=%d", calShadowRegs.endOfLifeEnable);
			putString(myOutputCommPort, myResponseBuf);

			
		}
		else
			if(*arg1 == '0')
			{
				calShadowRegs.endOfLifeEnable = 0;
				errLogFlagClear(ERR_END_OF_LIFE);

				saveSysCfgDelta(EE_AREA_CAL);

				sprintf(myResponseBuf, "\nEOL=%d", calShadowRegs.endOfLifeEnable);
				putString(myOutputCommPort, myResponseBuf);
			}
			else
			{
				errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
				putString(myOutputCommPort, "\n");
				sprintf(myResponseBuf, "EOL=%d", calShadowRegs.endOfLifeEnable);
				putString(myOutputCommPort, myResponseBuf);
			}
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Calibration End of Life Toggles Fault Line Enabled/Disabled - special non-published command
ushort calEOL_FaultHandler(void)
{
	if(*arg1 =='1')
	{
		calShadowRegs.endOfLifeFaultLine = 1;

		saveSysCfgDelta(EE_AREA_CAL);
		sprintf(myResponseBuf, "\nEOLFL=%d", calShadowRegs.endOfLifeFaultLine);
		putString(myOutputCommPort, myResponseBuf);		
	}
	else
		if(*arg1 == '0')
		{
			calShadowRegs.endOfLifeFaultLine = 0;
			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nEOLFL=%d", calShadowRegs.endOfLifeFaultLine);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
		{
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
			sprintf(myResponseBuf, "\nEOLFL=%d", calShadowRegs.endOfLifeFaultLine);
			putString(myOutputCommPort, myResponseBuf);
		}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Calibration Non-Volatile Pulse Mode (PUL) Enable/Disable - manufacturing command
ushort calNVPULHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		if(*arg1 =='1')
		{
			calShadowRegs.nonVolPUL = 1;
			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nNVPUL=%d", calShadowRegs.nonVolPUL);
			putString(myOutputCommPort, myResponseBuf);


		}
		else
			if(*arg1 == '0')
			{
				calShadowRegs.nonVolPUL=0;
				saveSysCfgDelta(EE_AREA_CAL);

				sprintf(myResponseBuf, "\nNVPUL=%d", calShadowRegs.nonVolPUL);
				putString(myOutputCommPort, myResponseBuf);
			}
			else
			{
				errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
				sprintf(myResponseBuf, "\nNVPUL=%d", calShadowRegs.nonVolPUL);
				putString(myOutputCommPort, myResponseBuf);
			}
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// 	External Power Control Scale Factor 2 (Fine Setting)
ushort calEpcScaleFactor2Handler(void)
{
	short myTmp;


		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 10000)
		{
		 	calShadowRegs.epcScaleFactor2 = atoi(arg1);
			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nEPCSF2=%d", calShadowRegs.epcScaleFactor2);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);


	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Calibration : Laser Operating Mode - manufacturing command
//Allows laser features to be enabled/disabled depending on exact product type, particular hardware, etc
ushort calModeHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp = atoi(arg1);

		if(myTmp >= 0 && myTmp <= 63)
		{
		 	calShadowRegs.mode = atoi(arg1);
			saveSysCfgDelta(EE_AREA_CAL);

			sprintf(myResponseBuf, "\nMODE=%d", calShadowRegs.mode);
			putString(myOutputCommPort, myResponseBuf);
		}
		else
			errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
	}
	else
	{
	 	 errLogFlagSet(ERR_INVALID_COMMAND);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Calibration - RF Injection Enable - manufacturing command
// Toggles port on Ext PSA On/Off to switch RF Injection feature On/Off
ushort calRfInjHandler(void)
{
	rfInjHandlerProcess(VORT_PARSER);
	return 0;
}

void rfInjHandlerProcess(ushort myMode)
{
	ushort myArg;

	if(myMode == VORT_PARSER)
		myArg = atoi(arg1);
	else
		myArg = calShadowRegs.rfInj;

 	if(myMode==VORT_PARSER)
 	{
		if(isManufModeActive == TRUE)
		{
			if(myArg== 1)
			{
			 	calShadowRegs.rfInj = 1;
				saveSysCfgDelta(EE_AREA_CAL);

				extPsaSet(1,1);

				sprintf(myResponseBuf, "\nRFINJ=%d", calShadowRegs.rfInj);
				putString(myOutputCommPort, myResponseBuf);
			}
			else
				if(myArg==0)
				{
					calShadowRegs.rfInj = 0;
					saveSysCfgDelta(EE_AREA_CAL);

					extPsaSet(1,0);

					sprintf(myResponseBuf, "\nRFINJ=%d", calShadowRegs.rfInj);
					putString(myOutputCommPort, myResponseBuf);
				}
				else
					errLogFlagSet(ERR_VALUE_OUT_OF_RANGE);
		}
		else
		{
		 	 errLogFlagSet(ERR_INVALID_COMMAND);
		}

		displayPrompt(myOutputCommPort);
 	}
	else
	{
		if(myArg==1)
		{
			calShadowRegs.rfInj = 1;
			saveSysCfgDelta(EE_AREA_CAL);
			extPsaSet(1,1);
		}
		else
			if(myArg==0)
			{
				calShadowRegs.rfInj = 0;
				saveSysCfgDelta(EE_AREA_CAL);
				extPsaSet(1,0);
			}
	}
}


////////////////////////////////
// Calibration Command Responses
////////////////////////////////


///////////////////////////////////////////////////
// Get Calibration DAC Value Code Handler
ushort rcalDacHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?CDAC=%d", calShadowRegs.calDacValue);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Get Calibration DFB DAC Value Code Handler
ushort rcalDacDfbHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?CDACDFB=%d", calShadowRegs.calDacDfbValue);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Response: Calibration Version - manufacturing command
ushort rcalCalVersionHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?CV=%1.2f", calShadowRegs.calVersion);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}



////////////////////////////////
///////////////////////////////////////////////////
//Calibration DFB Current Set Pot- manufacturing command
ushort rcalDfblcspHandler(void)
{
	short myTmp;

	if(isManufModeActive == TRUE)
	{
		myTmp=extDualPotsDfbGetCur(); //Confirm Setting

		sprintf(myResponseBuf, "\n?DFBLCSP=%d", myTmp);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
//Request Calibration DFB Current Set DAC - manufacturing command
//Non-volatile setting of DFB DAC counts
//which are applied at power-up
ushort rcalDfblcDacCntHandler()
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n%s=%d", myCmdToken,sysCfgShadowRegs.dfblcDacCnt);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Response: Calibration Help Information - manufacturing command
ushort rcalHelpHandler(void)
{
	if(myOutputCommPort==USB_PORT)
	{
	 	myOutputCommPort = USB_PORT_INIT;
	}

	putString(myOutputCommPort, "\n\nHelp Commands: Calibration\n");

	if(isManufModeActive == TRUE)	// the following commands are only active in manufacturing mode
	{
		// Calibration Commands
		putString(myOutputCommPort, "cdac=[0 - 4095] - Curr Cal DAC Value\n");
		putString(myOutputCommPort, "cdacdfb=[0 - 4095] - DFB Curr Cal DAC Value\n");
		putString(myOutputCommPort, "cv=[0 - 1000] - Calibration Version\n");
		putString(myOutputCommPort, "dfblcsd=[0 - 4095] - Laser DFB Curr Set DAC\n");
		putString(myOutputCommPort, "dfblcsp=[0 - 255] - Laser DFB Curr Set Pot\n");
		putString(myOutputCommPort, "eol=[0/1] - End of Life Fault Enable\n");
		putString(myOutputCommPort, "eolfl=[0/1] - End of Life Fault Line Enable\n");
		putString(myOutputCommPort, "epcsf=[0-10000] - Cal EPC Scale Factor (Coarse)\n");
		putString(myOutputCommPort, "epcsf2=[0-10000] - Cal EPC Scale Factor 2 (Fine)\n");
		putString(myOutputCommPort, "heat=[0-4095] - Heat DAC Count Value\n");
		putString(myOutputCommPort, "ilc=[0 - 1000] - Initial Laser Current\n");
		putString(myOutputCommPort, "ilcdfb=[0 - 1000] - Initial Laser Current\n");
		putString(myOutputCommPort, "lcsf=[0-10000] - Cal Laser Current Scale Factor\n");
		putString(myOutputCommPort, "lcsp=[0 - 255] - Laser Current Set Pot\n");
		putString(myOutputCommPort, "lbs=[C,E] - Laser Beam Shape\n");
		putString(myOutputCommPort, "lpn=[#####] - Laser Part Number\n");
		putString(myOutputCommPort, "lpoa=[0-100] - Cal Laser Power Offset Adjust in Percent\n");
		putString(myOutputCommPort, "lpsp=[0 - 255] - Laser Power Set Pot\n");
		putString(myOutputCommPort, "lsn=[VLmmyy####] - Laser Serial Number\n");
		putString(myOutputCommPort, "mhc=[0 - 1000] - Maximum Heater Current\n");
		putString(myOutputCommPort, "mlc=[0 - 1000] - Maximum Laser Current\n");
		putString(myOutputCommPort, "mlcdfb=[0 - 1000] - Maximum DFB Current\n");
		putString(myOutputCommPort, "mode=[0 - 63] - Laser Operating Mode\n");
		putString(myOutputCommPort, "mpl=[0 - 1000] - Maximum Power Laser\n");
		putString(myOutputCommPort, "mwl=[250 - 2000] - Measured Wavelength\n");
		putString(myOutputCommPort, "nlp=[0 - 1000] - Nominal Laser Power\n");
		putString(myOutputCommPort, "nvpul=[0/1] - Non-Volatile PUL Enable\n");
		putString(myOutputCommPort, "nwl=[250 - 2000] - Nominal Wavelength\n");
		putString(myOutputCommPort, "rfinj=[0-1] - Cal RF Injection Mode Enable\n");

		putString(myOutputCommPort, "?cdac - Request Calibration DAC Value\n");
		putString(myOutputCommPort, "?cdacdfb - Request DFB Curr Cal DAC Value\n");
		putString(myOutputCommPort, "?cv - Request Calibration Version\n");
		putString(myOutputCommPort, "?dfblc - Request DFB Current\n");
		putString(myOutputCommPort, "?dfblcsd - [0 - 4095] - Request DFB Curr Set DAC\n");
		putString(myOutputCommPort, "?dfblcsp - [0 - 255] - Request DFB Curr Set Pot\n");
		putString(myOutputCommPort, "?eol - Request End of Life Fault Enable\n");
		putString(myOutputCommPort, "?eolfl - Request End of Life Fault Line Enable\n");
		putString(myOutputCommPort, "?epcsf - Request Cal External Power Control Scale Factor (Coarse)\n");
		putString(myOutputCommPort, "?epcsf2 - Request Cal EPC Scale Factor 2 (Fine)\n");
		putString(myOutputCommPort, "?heat - Request Heater DAC Value\n");
		putString(myOutputCommPort, "?ilc - Request Initial Laser Current\n");
		putString(myOutputCommPort, "?ilcdfb - Request Initial DFB Current\n");
		putString(myOutputCommPort, "?lbs - Request Laser Beam Shape\n");
		putString(myOutputCommPort, "?lcsf - Request Cal Laser Current Scale Factor\n");
		putString(myOutputCommPort, "?lcsp - Request Laser Current Set Pot\n");
		putString(myOutputCommPort, "?lpoa - Request Cal Laser Power Offset Adjust in Percent\n");
		putString(myOutputCommPort, "?lpn - Request Laser Part Number\n");
		putString(myOutputCommPort, "?lpsp - Request Laser Power Set Pot\n");
		putString(myOutputCommPort, "?lsn - Request Laser Serial Number\n");
		putString(myOutputCommPort, "?mhc - Request Maximum Heater Current\n");
		putString(myOutputCommPort, "?mlc - Request Maximum Laser Current\n");
		putString(myOutputCommPort, "?mlcdfb - Request Maximum DFB Current\n");
		putString(myOutputCommPort, "?mode - Request Laser Operating Mode\n");
		putString(myOutputCommPort, "?mpl - Request Maximum Power Laser\n");
		putString(myOutputCommPort, "?mwl - Request Measured Wavelength\n");
		putString(myOutputCommPort, "?nlp - Request Nominal Laser Power\n");
		putString(myOutputCommPort, "?nvpul - Request Non-Volatile PUL Enable\n");
		putString(myOutputCommPort, "?nwl - Request Nominal Wavelength\n");
		putString(myOutputCommPort, "?rfinj - Request Cal RF Injection Enable Status\n");
	}

	displayPrompt(myOutputCommPort);
	return 0;
}


///////////////////////////////////////////////////
// Response: Calibration Maximum Power Laser - manufacturing command
ushort rcalMaxPwrLsrHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?MPL=%3.1f", calShadowRegs.maxPwrLsr);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Response: Calibration Initial Laser Current - manufacturing command
ushort rcalInitLsrCurHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?ILC=%4d", calShadowRegs.initLsrCur);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Response: Calibration Initial Laser Current - manufacturing command
ushort rcalInitDfbCurHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?ILCDFB=%4d", calShadowRegs.initDfbCur);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


	///////////////////////////////////////////////////
// Response: Calibration Nominal Laser Power - manufacturing command
ushort rcalNomLsrPwrHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?NLP=%4d", calShadowRegs.nominalLsrPwr);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Response: Calibration Maximum Laser Current - manufacturing command
ushort rcalMaxLsrCurHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?MLC=%4d", calShadowRegs.maxLsrCur);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Response: Calibration Maximum Laser Current - manufacturing command
ushort rcalMaxLsrCurDfbHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?MLCDFB=%4d", calShadowRegs.maxCurrDfb);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Response: Calibration Maximum Heater Current
ushort rcalMaxHtrCurHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?MHC=%4d", calShadowRegs.maxCurrHtr);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Response: Calibration Maximum Over Current Limit- manufacturing command
ushort rcalMaxOverCurHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?MOC=%4d", calShadowRegs.maxOverCurr);
		putString(myOutputCommPort, myResponseBuf);
	}
	displayPrompt(myOutputCommPort);
	return 0;
}


///////////////////////////////////////////////////
// Response: Calibration Nominal Wave Length - manufacturing command
ushort rcalNomWaveLenHandler(void)
{
	//short		nominalWaveLen;

	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?NWL=%4d", calShadowRegs.nominalWaveLen);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Response: Calibration Measure Wave Length - manufacturing command
ushort rcalMeasWaveLenHandler(void)
{
	//short		measWaveLen;

	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?MWL=%4d", calShadowRegs.measWaveLen);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Response: Calibration Laser Power Set POT - manufacturing command
ushort rcalLsrPwrSetPotHandler(void)
{
	uchar myVal=0;

	if(isManufModeActive == TRUE)
	{
		myVal=extSinglePotGetLtfb();
		sprintf(myResponseBuf, "\n?LPSP=%d", myVal);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Response: Calibration Laser Current Set POT - manufacturing command
ushort rcalLsrCurSetPotHandler(void)
{
	uchar myVal=0;

	if(isManufModeActive == TRUE)
	{
		myVal=extDualPotsSoaGetCur();
		sprintf(myResponseBuf, "\n?LCSP=%d", myVal);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Response: Calibration Laser Part Number - manufacturing command
ushort rcalLsrPartNumHandler(void)
{
	//char		lsrPartNum[6];

	if(isManufModeActive == TRUE)
	{
		putString(myOutputCommPort, "\n");
		sprintf(myResponseBuf, "?LPN=%s", calShadowRegs.lsrPartNum);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Response: OBT Deviation - manufacturing command
ushort rcalObtDeviationHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		putString(myOutputCommPort, "\n");
		sprintf(myResponseBuf, "?OBTD=%d", calShadowRegs.obtDeviation);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}



///////////////////////////////////////////////////
// Response: Calibration Laser Serial Number - manufacturing command
ushort rcalLsrSerialNumHandler(void)
{
	//char		lsrSerialNum[11];

	if(isManufModeActive == TRUE)
	{
		putString(myOutputCommPort, "\n");
		sprintf(myResponseBuf, "?LSN=%s", calShadowRegs.lsrSerialNum);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}


///////////////////////////////////////////////////
// Response: Calibration Laser Beam Shape - manufacturing command
ushort rcalLsrBeamShapeHandler(void)
{
	//char		lsrBeamShape[4];

	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?LBS=%s", calShadowRegs.lsrBeamShape);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Response: Laser Current Scaling Factor - manufacturing command
ushort rcalLcScaleFactorHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?LCSF=%d", calShadowRegs.lcScaleFactor);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;
}

///////////////////////////////////////////////////
// Response: Calibration External Pwr Control Scaling Factor (Coarse Setting)
ushort rcalEpcScaleFactorHandler(void)
{
	sprintf(myResponseBuf, "\n?EPCSF=%d", calShadowRegs.epcScaleFactor);
	putString(myOutputCommPort, myResponseBuf);

	displayPrompt(myOutputCommPort);

	return 0;

}

///////////////////////////////////////////////////
// Response: Laser Power Offset Adjust - manufacturing command
ushort rcalLpoaHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		sprintf(myResponseBuf, "\n?LPOA=%d", calShadowRegs.lpOffsetAdjust);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;

}

///////////////////////////////////////////////////
// Response: End of Life Faults Enabled/Disabled - special (non-published) query
ushort rcalEOLHandler(void)
{

	putString(myOutputCommPort, "\n");
	sprintf(myResponseBuf, "?EOL=%d", calShadowRegs.endOfLifeEnable);
	putString(myOutputCommPort, myResponseBuf);


	displayPrompt(myOutputCommPort);

	return 0;

}

///////////////////////////////////////////////////
// Response: End of Life Faults Toggle Fault Line Enabled/Disabled - special (non-published) query
ushort rcalEOL_FaultHandler(void)
{
	sprintf(myResponseBuf, "\n?EOLFL=%d", calShadowRegs.endOfLifeFaultLine);
	putString(myOutputCommPort, myResponseBuf);


	displayPrompt(myOutputCommPort);

	return 0;

}

///////////////////////////////////////////////////
// Response: Non-Volatile Pulse Mode (PUL) Enable/Disable - manufacturing command
ushort rcalNVPULHandler(void)
{
	if(isManufModeActive == TRUE)
	{
		putString(myOutputCommPort, "\n");
		sprintf(myResponseBuf, "?NVPUL=%d", calShadowRegs.nonVolPUL);
		putString(myOutputCommPort, myResponseBuf);
	}

	displayPrompt(myOutputCommPort);

	return 0;

}

///////////////////////////////////////////////////
// Calibration External Pwr Control Scaling Factor (Fine Setting)
ushort rcalEpcScaleFactor2Handler(void)
{
	sprintf(myResponseBuf, "\n?EPCSF2=%d", calShadowRegs.epcScaleFactor2);
	putString(myOutputCommPort, myResponseBuf);

	displayPrompt(myOutputCommPort);

	return 0;

}

///////////////////////////////////////////////////
// Mode: Laser Operating Mode
ushort rcalModeHandler(void)
{
	sprintf(myResponseBuf, "\n?MODE=%d", calShadowRegs.mode);
	putString(myOutputCommPort, myResponseBuf);

	displayPrompt(myOutputCommPort);

	return 0;

}

///////////////////////////////////////////////////
// Response: RF Injection Enable - manufacturing command
ushort rcalRfInjHandler(void)
{
	sprintf(myResponseBuf, "\n?RFINJ=%d", calShadowRegs.rfInj);
	putString(myOutputCommPort, myResponseBuf);

	displayPrompt(myOutputCommPort);

	return 0;

}
///////////////////////////////////////////////////
// Response: RF Injection Enable - manufacturing command
ushort rcalHtrCntHandler(void)
{
	sprintf(myResponseBuf, "\n?HEAT=%d", calShadowRegs.htrCnt);
	putString(myOutputCommPort, myResponseBuf);

	displayPrompt(myOutputCommPort);

	return 0;

}

void displayPrompt(uchar myPort)
{
	ulong myErrFlags = 0;
	ulong myBrandXErrFlags = 0;
	ulong myBitPos = 1;
	uchar myErrPos = 0;

	if(myPort==USB_PORT)
	{
	 	myPort = USB_PORT_INIT;
	}

	if(sysCfgShadowRegs.prompt)
	{
		myErrFlags = errLogFlagGet();

		if(sysCfgShadowRegs.protoMode == EE_DEF_PROTO_VORT)
		{
	 		// display error code on prompt with Brand name
			sprintf(myBuf, "\nStradus:%d> ", myErrFlags);
			putString(myPort, myBuf);
		}
		else
		{
			// convert the error flags to BrandX error flags
			while(myBitPos)
			{
				if(myErrFlags & myBitPos)
				{
					if(vort2BrandX[myErrPos + 1] != INVALID_ERR)
						myBrandXErrFlags |= (1 << (vort2BrandX[myErrPos + 1] - 1));
				}

				myBitPos = myBitPos << 1;
				myErrPos++;
			}

			// display error code on prompt
			sprintf(myBuf, "\nCube:%d> ", myBrandXErrFlags);
			putString(myPort, myBuf);
		}
	}
	else
	{
			putString(myPort, "\n");
	}

	//errLogFlagClearAll();	// clear the errors Commented Out by Vortran-DMS 11/19/09
	errLogFlagClear(ERR_VALUE_OUT_OF_RANGE); //Clear command line errors Vortran-DMS 11/19/09
	errLogFlagClear(ERR_INVALID_COMMAND);
}



