/*///////////////////////////////////////////////////////////////////////////
//
//						queue.c
//
//	Circular queues for communication to/from the USB, Serial 0, and Serial 1
//
//	(C) SoftwareFlair Inc. 2008
//
//
//
/////////////////////////////////////////////////////////////////////////////
*/
#include <LPC214x.H>
#include "system.h"
#include "queue.h"

typedef struct		// Queues for communication to/from the USB, Serial 0, and Serial 1
{
	uchar *qPtrHead[2];
	uchar *qPtrTail[2];
	uchar queue[2][MAX_Q_BUF_SIZE];		// input Queue and an output queue
} Q_CONFIG;

Q_CONFIG qHandle[3];	// these queue handles are for the MCU, Sub PIC A, and Sub PIC B

typedef enum {
	Q_READ = 0,
	Q_WRITE
} Q_ACCESS;

// prototypes
void qErrorLog(Q_STREAM myQ, Q_TYPE myQType, Q_ACCESS myQAccess);

// globals
#if 0
uchar inQTmp[MAX_Q_BUF_SIZE];
uchar outQTmp[MAX_Q_BUF_SIZE];
#endif


// initialize all of the queues
void qInit(void)
{
	ushort myCnt;	
	uchar myIndx;

	//swTrace(__FILE__, __LINE__);		// log Trace: we have been here!

	// initialize the head and tail pointers to the first element of the queue, for all queues
	for(myIndx = 0; myIndx < END_OF_QUEUES; myIndx++)
	{
		qHandle[myIndx].qPtrHead[IN_Q] = &qHandle[myIndx].queue[IN_Q][0];
		qHandle[myIndx].qPtrTail[IN_Q] = &qHandle[myIndx].queue[IN_Q][0];
		
		qHandle[myIndx].qPtrHead[OUT_Q] = &qHandle[myIndx].queue[OUT_Q][0];
		qHandle[myIndx].qPtrTail[OUT_Q] = &qHandle[myIndx].queue[OUT_Q][0];
		
		// initialize ALL of the in/out queues to zero
		for(myCnt = 0; myCnt < MAX_Q_BUF_SIZE; myCnt++)	// init MCU/sPicA/sPicB
		{
			qHandle[myIndx].queue[IN_Q][myCnt] = 0;	
			qHandle[myIndx].queue[OUT_Q][myCnt] = 0;		
		}		
	}				
}

// Reset a specific queue: head and tail, and queue data buffer 
void qReset(Q_STREAM myStream, Q_TYPE myType)
{
	uchar myCnt;

	wasHere = 0x10;

	//swTrace(__FILE__, __LINE__);		// log Trace: we have been here!
	
	//allInterruptsOff();
	VICIntEnClr = 1 << 6;       /* Disable UART0 Interrupt */
#if ACTIVATE_COMM1
	VICIntEnClr = 1 << 7;       /* Disable UART1 Interrupt */
#endif

	// initialize the head and tail pointers to the first element of the queue, for this specific queue
	qHandle[myStream].qPtrHead[myType] = &qHandle[myStream].queue[myType][0];
	qHandle[myStream].qPtrTail[myType] = &qHandle[myStream].queue[myType][0];
		
	// initialize ALL of the in/out queues to zero
	for(myCnt = 0; myCnt < MAX_Q_BUF_SIZE; myCnt++)	// init queue to zero
		qHandle[myStream].queue[myType][myCnt] = 0;	

  VICIntEnable = 1 << 6;       /* Enable UART0 Interrupt */
#if ACTIVATE_COMM1
  VICIntEnable = 1 << 7;       /* Enable UART1 Interrupt */
#endif		
	//allInterruptsOn();								
}

// Store a bytes to the specified queue if there is room in the queue
void writeQ(Q_STREAM myQ, Q_TYPE myQType, uchar *myData, uchar mySize)
{
	wasHere = 0x11;
	//allInterruptsOff();	// FRG: Optimize later!!
	//VICIntEnClr = 1 << 6;       /* Disable UART0 Interrupt */
#if ACTIVATE_COMM1
	VICIntEnClr = 1 << 7;       /* Disable UART1 Interrupt */
#endif

	//swTrace(__FILE__, __LINE__);		// log Trace: we have been here!
	
	if(mySize >= MAX_Q_BUF_SIZE)	// is the size requested bigger then the max buffer size?
	{
		swError(__FILE__, __LINE__);
		return;	
	}

	while(mySize--)	// save the data, based on size
	{
		// is there any free room in Q? The head pointer must not be 1 byte behind the tail pointer
		// first check the wrap condition 
		if(qHandle[myQ].qPtrHead[myQType] == &qHandle[myQ].queue[myQType][MAX_Q_BUF_SIZE - 1])	// is this the last position in Q?
		{
			if(qHandle[myQ].qPtrTail[myQType] == &qHandle[myQ].queue[myQType][0])	// pointers shall collide!!! Queue Full!!
			{
				swError(__FILE__, __LINE__); 
				qErrorLog(myQ, myQType, Q_WRITE);			// yes!!!			
				break;
			}
			else
			{
				// FRG DEBUG
				if(qHandle[myQ].qPtrHead[myQType] < (uchar*) 0x40000000 || qHandle[myQ].qPtrHead[myQType] > (uchar*)0x40002000)
					swError(__FILE__, __LINE__);

				*qHandle[myQ].qPtrHead[myQType] = *myData++;	// save the data
				qHandle[myQ].qPtrHead[myQType] = &qHandle[myQ].queue[myQType][0];		// wrap to the first start of the queue						
			}	
		}
		else
		{
			if(qHandle[myQ].qPtrHead[myQType] == (qHandle[myQ].qPtrTail[myQType] - 1)) 	// pointers shall collide!!! Queue Full!!
			{
				qErrorLog(myQ, myQType, Q_WRITE);	// yes!!!
				break;
			}
			else
			{
				// FRG DEBUG
				if(qHandle[myQ].qPtrHead[myQType] < (uchar*) 0x40000000 || qHandle[myQ].qPtrHead[myQType] > (uchar*) 0x40002000)
					while(1);

				*qHandle[myQ].qPtrHead[myQType] = *myData++;	// save the data
				qHandle[myQ].qPtrHead[myQType]++;		// go to the next free element of the queue						
			}			
		}
	}

  //VICIntEnable = 1 << 6;       /* Enable UART0 Interrupt */
#if ACTIVATE_COMM1
  VICIntEnable = 1 << 7;       /* Enable UART1 Interrupt */
#endif
	//allInterruptsOn();	
}


// read bytes from the specified queue if there is data in the queue, This does NOT Flush the data from  the queue
void readQ(Q_STREAM myQ, Q_TYPE myQType, uchar *myData, uchar mySize)
{
	uchar *myCurQHandle;

	wasHere = 0x12;

	//allInterruptsOff();
	//VICIntEnClr = 1 << 6;       /* Disable UART0 Interrupt */
#if ACTIVATE_COMM1
	VICIntEnClr = 1 << 7;       /* Disable UART1 Interrupt */
#endif

	//swTrace(__FILE__, __LINE__);		// log Trace: we have been here!

	myCurQHandle = qHandle[myQ].qPtrTail[myQType];	// save the "current" address
	
	if(mySize >= MAX_Q_BUF_SIZE)	// is the size requested bigger then the max buffer size?
	{
		swError(__FILE__, __LINE__);
		return;	
	}
	
	while(mySize--)
	{
		// is queue empty? The tail pointer must not be equal to the head pointer
		if(qHandle[myQ].qPtrTail[myQType] == qHandle[myQ].qPtrHead[myQType])
		{
			qErrorLog(myQ, myQType, Q_READ);	// yes!!! Queue empty
			qHandle[myQ].qPtrTail[myQType] = myCurQHandle;	 // return to the "current" address
			break;
		}
		else
		{
			*myData++ = *qHandle[myQ].qPtrTail[myQType];	// get the data
			
			// make sure the tail is not about to wrap!!
			if(qHandle[myQ].qPtrTail[myQType] == &qHandle[myQ].queue[myQType][MAX_Q_BUF_SIZE - 1]) 	// is this the last position in Q?
				qHandle[myQ].qPtrTail[myQType] = &qHandle[myQ].queue[myQType][0];		// yes, go to start of queue
			else
				qHandle[myQ].qPtrTail[myQType]++;
		}	
	}
	
	qHandle[myQ].qPtrTail[myQType] = myCurQHandle;	 // return to the "current" address	

  //VICIntEnable = 1 << 6;       /* Enable UART0 Interrupt */
#if ACTIVATE_COMM1
  VICIntEnable = 1 << 7;       /* Enable UART1 Interrupt */
#endif
	//allInterruptsOn();
}

// flush bytes from the specified queue if there is data in the queue
void flushQ(Q_STREAM myQ, Q_TYPE myQType, uchar mySize)
{
	uchar *myCurQHandle;

	wasHere = 0x13;

	//if(myQ == SER0_Q)			// FRG DEBUG!!
		//return;

	//allInterruptsOff();
	//VICIntEnClr = 1 << 6;       /* Disable UART0 Interrupt */
#if ACTIVATE_COMM1
	VICIntEnClr = 1 << 7;       /* Disable UART1 Interrupt */
#endif

	//swTrace(__FILE__, __LINE__);		// log Trace: we have been here!

	myCurQHandle = qHandle[myQ].qPtrTail[myQType];	// save the "current" address
	
	if(mySize >= MAX_Q_BUF_SIZE)	// is the size requested bigger then the max buffer size?
	{
		swError(__FILE__, __LINE__);
		return;	
	}
	
	while(mySize--)
	{
		// is queue empty? The tail pointer must not be equal to the head pointer
		if(qHandle[myQ].qPtrTail[myQType] == qHandle[myQ].qPtrHead[myQType])
		{
			qErrorLog(myQ, myQType, Q_READ);	// yes!!! Queue empty
			qHandle[myQ].qPtrTail[myQType] = myCurQHandle;	 // return to the "current" address
			break;
		}
		else
		{	
			// make sure the tail is not about to wrap!!
			if(qHandle[myQ].qPtrTail[myQType] == &qHandle[myQ].queue[myQType][MAX_Q_BUF_SIZE - 1]) 	// is this the last position in Q?
				qHandle[myQ].qPtrTail[myQType] = &qHandle[myQ].queue[myQType][0];		// yes, go to start of queue
			else
				qHandle[myQ].qPtrTail[myQType]++;
		}	
	}	

  //VICIntEnable = 1 << 6;       /* Enable UART0 Interrupt */
#if ACTIVATE_COMM1
  VICIntEnable = 1 << 7;       /* Enable UART1 Interrupt */
#endif
	//allInterruptsOn();
}

short prevWasHere = 0;

// the number of bytes that have been written and can be read from the queue
uchar qFullIndicator(Q_STREAM myQ, Q_TYPE myQType)
{
	uchar myVal;

	prevWasHere	= wasHere;
	wasHere = 0x14;

	//allInterruptsOff();
	//VICIntEnClr = 1 << 6;       /* Disable UART0 Interrupt */
#if ACTIVATE_COMM1
	VICIntEnClr = 1 << 7;       /* Disable UART1 Interrupt */
#endif
	//swTrace(__FILE__, __LINE__);		// log Trace: we have been here!

	wasHere = 0x15;
	
	if((uchar *) qHandle[myQ].qPtrHead[myQType] > (uchar *) qHandle[myQ].qPtrTail[myQType])
	{
		wasHere = 0x16;
		myVal =  (qHandle[myQ].qPtrHead[myQType] - qHandle[myQ].qPtrTail[myQType]);
	}
	else if((uchar *) qHandle[myQ].qPtrHead[myQType] == (uchar *) qHandle[myQ].qPtrTail[myQType])
	{
		wasHere = 0x17;
		myVal = 0;
		//swTrace(__FILE__, __LINE__);		// log Trace: we have been here!
	}
	else
	{		// calculate the tail proximity from the end of the queue
		//swTrace(__FILE__, __LINE__);		// log Trace: we have been here!
		wasHere = 0x18;
		myVal =  &qHandle[myQ].queue[myQType][MAX_Q_BUF_SIZE] -  qHandle[myQ].qPtrTail[myQType];
		wasHere = 0x19;
		//swTrace(__FILE__, __LINE__);		// log Trace: we have been here!
		myVal += ( qHandle[myQ].qPtrHead[myQType] - &qHandle[myQ].queue[myQType][0]); // calculate the head proximity from the start of the queue				
	}

	wasHere = 0x1a;

  //VICIntEnable = 1 << 6;       /* Enable UART0 Interrupt */
#if ACTIVATE_COMM1
  VICIntEnable = 1 << 7;       /* Enable UART1 Interrupt */
#endif
	//allInterruptsOn();
	
	wasHere = 0x1b;	

	return(myVal);	
}

// the number of bytes that have been written and can be read from the queue
uchar qEmptyIndicator(Q_STREAM myQ, Q_TYPE myQType)
{
	uchar myFullSize;
	
	myFullSize = qFullIndicator(myQ, myQType);
	
	return(MAX_Q_BUF_SIZE - myFullSize);
}


void qErrorLog(Q_STREAM myQ, Q_TYPE myQType, Q_ACCESS myQAccess)
{
	//swTrace(__FILE__, __LINE__);		// log Trace: we have been here!

	if(myQAccess == Q_WRITE)
	{	
		switch(myQ)
		{
			case USB_Q:
				if(myQType == IN_Q)
					swError(__FILE__, __LINE__);		
				else
					swError(__FILE__, __LINE__);
				break;
				
			case SER0_Q: 
				if(myQType == IN_Q)
					swError(__FILE__, __LINE__);
				else
					swError(__FILE__, __LINE__);
					
				break;
				
			case SER1_Q: 
				if(myQType == IN_Q)
					swError(__FILE__, __LINE__);
				else
					swError(__FILE__, __LINE__);
					
				break;
				
			default:
				swError(__FILE__, __LINE__);		// log error
				break;	
		}
	}
	else
	{
		switch(myQ)
		{
			case USB_Q:
				if(myQType == IN_Q)
					swError(__FILE__, __LINE__);
				else
					swError(__FILE__, __LINE__);
				break;
				
			case SER0_Q: 
				if(myQType == IN_Q)
					swError(__FILE__, __LINE__);
				else
					swError(__FILE__, __LINE__);
					
				break;
				
			case SER1_Q: 
				if(myQType == IN_Q)
					swError(__FILE__, __LINE__);
				else
					swError(__FILE__, __LINE__);
					
				break;
				
			default:
				swError(__FILE__, __LINE__);		// log error
				break;	
		}
	}	
}


#if 0
void diagTestQueue(void)
{
	uchar myCnt;	
	
	qInit();
	
	for(myCnt = 0; myCnt < MAX_Q_BUF_SIZE; myCnt++)
	{
		inQTmp[myCnt] = 0;
		outQTmp[myCnt] = 0;		
	}
	
	// test the queue full and empty indicator	
	myCnt = qFullIndicator(USB_Q, IN_Q);
	myCnt = qEmptyIndicator(USB_Q, IN_Q);
	
	writeQ(USB_Q, IN_Q, inQTmp, 0x3F);
	
	myCnt = qFullIndicator(USB_Q, IN_Q);
	myCnt = qEmptyIndicator(USB_Q, IN_Q);
	
	writeQ(USB_Q, IN_Q, inQTmp, 0x1);
	
	myCnt = qFullIndicator(USB_Q, IN_Q);
	myCnt = qEmptyIndicator(USB_Q, IN_Q);
	
	writeQ(USB_Q, IN_Q, inQTmp, 0x1);
	myCnt = qFullIndicator(USB_Q, IN_Q);
	myCnt = qEmptyIndicator(USB_Q, IN_Q);
	
	
	// debug test" writes to queue to make queue full
	for(myCnt = 0; myCnt < MAX_Q_BUF_SIZE; myCnt++)
		inQTmp[myCnt] = myCnt;
		
	writeQ(USB_Q, IN_Q, inQTmp, 0xF0);
	writeQ(USB_Q, IN_Q, &inQTmp[0x20], 0x10);	

	
	// debug test roll over (wrap around)
	for(myCnt = 0; myCnt < MAX_Q_BUF_SIZE; myCnt++)
		inQTmp[myCnt] = myCnt;
		
	writeQ(USB_Q, IN_Q, inQTmp, 0xF3);
	readQ(USB_Q, IN_Q, outQTmp, 0xF3);
	writeQ(USB_Q, IN_Q, &inQTmp[0x20], 0x10);
	readQ(USB_Q, IN_Q, outQTmp, 0x10);
}
#endif


