/*----------------------------------------------------------------------------
 *      Name:    sysCfg.c
 *      Purpose: SystemConfiguration
 *
 *	SystemCfg  the system configuration is stored in EEPROM. User parameters
 *	and system parameters are assigned static physical addresses, and a Checksum is
 *	performed on the memory area. The first byte is the system configuration version
 *	number. The last 2 bytes of EEPROM are the diagnostic test area (DTA), and the
 *	2 byte preceding the DTA  are the Checksum value. Help information may also be
 *	stored in a unique area of eeprom, with its own Checksum.
 *
 *	EEPROM Image
 *		Vortran 2144 board	(64K Bits = 8KBytes)
 *		0x0 - 0xFD		system configuration Attributes
 *		0xFE - 0xFF		system configuration Attributes checksum (2 Bytes)
 *		0x100 - 0x1FD		system calibration Attributes
 *		0x1FE - 0x1FF		system calibration Attributes checksum (2 Bytes)
 *		0x200 - 0xFFD		help command description
 *		0xFFE - 0xFFF		help command description checksum (2 Bytes)
 *		0x1000 - 0x1FFD		Free
 *		0x1FFE - 0x1FFF		DIAGS test word (16 bits)
 *
 *		Keil 2148 Development board with add on eeprom (4KBits = 512 bytes)
 *		0x0 - 0xAD		system configuration Attributes
 *		0xAE - 0xAF		system configuration Attributes checksum (2 Bytes)
 *		0xB0 - 0xFD		system calibration Attributes
 *		0xFE - 0xFF		system calibration Attributes checksum (2 Bytes)
 *		0x100 - 0x1FC		help command description
 *		0x1FE - 0x1FF		help command description checksum (2 Bytes)
 *
 *      Copyright (c) 2008 SoftwareFlair Inc.
 *---------------------------------------------------------------------------*/

#include <string.h>
#include "system.h"
#include "sysCfg.h"
#include "extEeprom.h"
#include "type.h"
#include "errLog.h"
#include "serial.h"



extern ushort chkSumCalc(ushort myStartAddr, ushort myEndAddr);
extern char verifySysCfg(EE_AREA_TYPE);
extern void getSysCfg(EE_AREA_TYPE);
extern void saveCfgDefaults(EE_AREA_TYPE);
extern void setShadowRegsToDef(EE_AREA_TYPE);
extern void saveSwErrDefaults(void);



SYS_CFG_SHADOW_REG sysCfgShadowRegs;	// these shadow registers are modified by the code, when a change is required
SYS_CFG_SHADOW_REG sysCfgEeprom;			// this structure contains what is in eeprom
CAL_SHADOW_REG calShadowRegs;	// these shadow registers are modified by the code, when a change is required
CAL_SHADOW_REG calEeprom;			// this structure contains what is in eeprom
HELP_SHADOW_REG helpShadowRegs;	// these shadow registers are modified by the code, when a change is required
HELP_SHADOW_REG helpEeprom;			// this structure contains what is in eeprom
static ulong myStartAddr[EE_AREA_END];
static ulong myChkSmAddr[EE_AREA_END];
static ulong myShadowRegSize[EE_AREA_END] = {0,0,0};
static uchar* myShadowPtr[EE_AREA_END] = {0,0,0};
static uchar* myEePtr[EE_AREA_END] = {0,0,0};

short isCalImageGood = FALSE;		 // if the Calibration Image is corrupted then do not allow the laser to be turned on!
short isSwErrImageGood = FALSE;
short isSysCfgImageGood = FALSE;
ulong currentSysCfgBank = EE_SYSCFG0_START; 	// current System Configuration Bank: 0 or 1 - double buffered

void sysCfgInit(void)
{
	// initailize System Configuration Image Areas
	myStartAddr[EE_AREA_SYS_CFG] = (ulong) currentSysCfgBank;
	myChkSmAddr[EE_AREA_SYS_CFG] = currentSysCfgBank + EE_SYSCFG0_CHKSUM;
	myShadowRegSize[EE_AREA_SYS_CFG] = sizeof(sysCfgShadowRegs);
	myShadowPtr[EE_AREA_SYS_CFG] = (uchar*) &sysCfgShadowRegs;
	myEePtr[EE_AREA_SYS_CFG] = (uchar*) &sysCfgEeprom;

	myStartAddr[EE_AREA_CAL] = (ulong) EE_CAL_START;
	myChkSmAddr[EE_AREA_CAL] = EE_CAL_CHKSUM;
	myShadowRegSize[EE_AREA_CAL] = sizeof(calShadowRegs);
	myShadowPtr[EE_AREA_CAL] = (uchar*) &calShadowRegs;
	myEePtr[EE_AREA_CAL] = (uchar*) &calEeprom;

#if VORT_BOARD
	myStartAddr[EE_AREA_HELP] = (ulong) EE_HELP_START;
	myChkSmAddr[EE_AREA_HELP] = EE_HELP_CHKSUM;
	myShadowRegSize[EE_AREA_HELP] = sizeof(helpShadowRegs);
	myShadowPtr[EE_AREA_HELP] = (uchar*) &helpShadowRegs;
	myEePtr[EE_AREA_HELP] = (uchar*) &helpEeprom;


	myStartAddr[EE_AREA_SW_ERR] = (ulong) EE_SW_ERR_START;
	myChkSmAddr[EE_AREA_SW_ERR] = EE_SW_ERR_CHKSUM;
	myShadowRegSize[EE_AREA_SW_ERR] = 0;
	myShadowPtr[EE_AREA_SW_ERR] = (uchar*) 0;
	myEePtr[EE_AREA_SW_ERR] = (uchar*) 0;
#endif


	// is eeprom alive and well?
	if(isEepromGood() == TRUE)
	{
			////////////////
			// SYSTEM CONFIG
			if(verifySysCfg(EE_AREA_SYS_CFG) == TRUE)	// is this system config image/bank valid?
			{
				getSysCfg(EE_AREA_SYS_CFG);	// yes, get the image
				isSysCfgImageGood = TRUE;
			}
			else
			{
				// check the "other" System Configuration bank!
				if(currentSysCfgBank == EE_SYSCFG0_START)
					currentSysCfgBank = EE_SYSCFG1_START;
				else
					currentSysCfgBank = EE_SYSCFG0_START;

				myStartAddr[EE_AREA_SYS_CFG] = (ulong) currentSysCfgBank;
				myChkSmAddr[EE_AREA_SYS_CFG] = currentSysCfgBank + EE_SYSCFG0_CHKSUM;

				if(verifySysCfg(EE_AREA_SYS_CFG) == TRUE)	// is this 2nd system config image/bank valid?
				{
					getSysCfg(EE_AREA_SYS_CFG);	// yes, get the image
					isSysCfgImageGood = TRUE;
				}
				else	// both banks are bad!!
				{
					// FRG: Remove Later!!
					putString(COMM_0, "*Init Sys Cfg!!\n");

					saveCfgDefaults(EE_AREA_SYS_CFG);	// no, save the system config

					if(verifySysCfg(EE_AREA_SYS_CFG) == TRUE)	// is the system config image valid?
					{
						getSysCfg(EE_AREA_SYS_CFG);	// yes, get the image
						isSysCfgImageGood = TRUE;
					}
					else
					{
						setShadowRegsToDef(EE_AREA_SYS_CFG);
						errLogFlagSet(ERR_EEPROM_ERROR);

						// FRG: Remove Later!!
						putString(COMM_0, "*Sys Cfg Failed!!\n");
					}
				}
			}

			/////////////
			// CAL
			if(verifySysCfg(EE_AREA_CAL) == TRUE)	// is the calibration image valid?
			{
				getSysCfg(EE_AREA_CAL);	// yes, get the image
				isCalImageGood = TRUE;
			}
			else
			{
				// FRG: Remove Later!!
				putString(COMM_0, "*Cal Failed!!\n");
				swError(__FILE__, __LINE__);

				isCalImageGood = FALSE;

#if 0
				// FRG: Remove Later!!
				putString(COMM_0, "*Init Cal Image!!\n");
				swError(__FILE__, __LINE__);

				saveCfgDefaults(EE_AREA_CAL);	// no, save the system config

				if(verifySysCfg(EE_AREA_CAL) == TRUE)	// is the system config image valid?
				{
					getSysCfg(EE_AREA_CAL);	// yes, get the image
				}
				else
				{
					setShadowRegsToDef(EE_AREA_CAL);
					errLogFlagSet(ERR_EEPROM_ERROR);

					// FRG: Remove Later!!
					putString(COMM_0, "*Cal Failed!!\n");
					swError(__FILE__, __LINE__);
				}
#endif
			}

			/////////////
			// Software Error
			if(verifySysCfg(EE_AREA_SW_ERR) == TRUE)	// is the Software Error image valid?
			{
				isSwErrImageGood = TRUE;
			}
			else
			{
				// FRG: Remove Later!!
				putString(COMM_0, "*SwErr Failed!!\n");
				swError(__FILE__, __LINE__);

				isSwErrImageGood = FALSE;

				// attempt to initialize the Software Error image
				saveSwErrDefaults();

				if(verifySysCfg(EE_AREA_SW_ERR) == TRUE)
					isSwErrImageGood = TRUE;

			}

			/////////////
			// HELP
	}
	else		// eeprom issues!
	{
		setShadowRegsToDef(EE_AREA_SYS_CFG);
		setShadowRegsToDef(EE_AREA_CAL);

#if VORT_BOARD
		setShadowRegsToDef(EE_AREA_HELP);
#endif

		errLogFlagSet(ERR_EEPROM_ERROR);

		//putString(COMM_0, "\nEeprom Failed!!!\n");
		//putString(COMM_1, "\nEeprom Failed!!!\n");
		swError(__FILE__, __LINE__);
 	}


 	if(isSysCfgImageGood == TRUE && isCalImageGood == TRUE)
 	{
		//If the EEPROM DEF version in the EEPROM is from a prior firmware version set any
		//variables added to the cfg and cal to their respective default values and then overwrite
		//the EE_DEF_VERSION with the current value for this firmware build. If the EEPROM DEF version
		//in the EEPROM is newer than this version then don't change any values. Only the values supported
		//this firmware build will be used.
		if(sysCfgShadowRegs.version < EE_DEF_SYSCFG_VERSION)
		{			
			//if(sysCfgShadowRegs.version < 0x01)
			//{
				 //Set default values to new members of SYS_CFG_SHADOW_REG struct
			//}
			sysCfgShadowRegs.version = EE_DEF_SYSCFG_VERSION; //Update the EEPROM DEFINITION version to make current.
			saveSysCfgDelta(EE_AREA_SYS_CFG);
		}
		if(calShadowRegs.version < EE_DEF_CAL_VERSION)
		{
		    //if(calShadowRegs.version < 0x01)
			//{
				 //Set default values to new members of CAL_SHADOW_REG struct
			//}
			if(calShadowRegs.version < 0x03)
			{
				
			 	calShadowRegs.maxCurrDfb = EE_DFB_MAX_CURR; //Change default to 200mA
			}

			calShadowRegs.version = EE_DEF_CAL_VERSION;
			saveSysCfgDelta(EE_AREA_CAL);
		}
	}
}

// force the system configuration values to default in the eeprom
void saveCfgDefaults(EE_AREA_TYPE myArea)
{
	ushort myCnt;
	uchar* myPtr = 0;
	ushort myCheckSum;

	setShadowRegsToDef(myArea);

	myPtr = (uchar*) myShadowPtr[myArea];

	for(myCnt = myStartAddr[myArea]; myCnt < (myStartAddr[myArea] + myShadowRegSize[myArea]); myCnt++)
	{
		// write all of the system configuration values from shadow registers into EEPROM
		eepromWrite(myCnt, *myPtr++);
#if VORT_BOARD
	//delayMs(1);
#else
	delayMs(5);
#endif
	}

	// calculate checksum
	myCheckSum = chkSumCalc(myStartAddr[myArea], myChkSmAddr[myArea] - 1);

	// save checksum
	eepromWrite(myChkSmAddr[myArea], myCheckSum >> 8);
#if VORT_BOARD
	//delayMs(1);
#else
	delayMs(5);
#endif
	eepromWrite(myChkSmAddr[myArea] + 1, myCheckSum & 0xFF);
#if VORT_BOARD
	//delayMs(1);
#else
	delayMs(5);
#endif
}

// save the system config shadow registers to eeprom
void saveSysCfg(EE_AREA_TYPE myArea)
{
	ushort myCnt;
	uchar* myPtr;
	uchar myData;
	ushort myCheckSum = 0;
	ulong tmpCurrentSysCfgBank;

	// double banking is used for the system config area
	// start to write the new transaction, when a full bank write is complete with checksum, then switch banks, and destroy the previous image
	if(myArea == EE_AREA_SYS_CFG)
	{
		// check the "other" System Configuration bank!
		if(currentSysCfgBank == EE_SYSCFG0_START)
			tmpCurrentSysCfgBank = EE_SYSCFG1_START;
		else
			tmpCurrentSysCfgBank = EE_SYSCFG0_START;

		myStartAddr[EE_AREA_SYS_CFG] = (ulong) tmpCurrentSysCfgBank;
		myChkSmAddr[EE_AREA_SYS_CFG] = (ulong) tmpCurrentSysCfgBank + EE_SYSCFG0_CHKSUM;
	}

	myPtr =  myShadowPtr[myArea];

	for(myCnt = myStartAddr[myArea]; myCnt < (myStartAddr[myArea] + myShadowRegSize[myArea]); myCnt++)
	{
		// write all of the system configuration values from shadow registers into EEPROM
		eepromWrite(myCnt, *myPtr++);
	}

	// calculate checksum
	myCheckSum = chkSumCalc(myStartAddr[myArea], myChkSmAddr[myArea] - 1);

	// save checksum
	eepromWrite(myChkSmAddr[myArea], myCheckSum >> 8);
#if VORT_BOARD
	//delayMs(1);
#else
	delayMs(5);
#endif
	eepromWrite(myChkSmAddr[myArea] + 1, myCheckSum & 0xFF);
#if VORT_BOARD
	//delayMs(1);
#else
	delayMs(5);
#endif

 	// double banking is used for the system config area
	if(myArea == EE_AREA_SYS_CFG)
	{
		// destroy the previous bank.
		eepromRead(tmpCurrentSysCfgBank + EE_SYSCFG0_CHKSUM + 1, &myData);	 // read the previous "correct" checksum LSB value
		// destroy the image area: save correct checksum + 1. This allows the image area to be analyzed in the future
		eepromWrite(tmpCurrentSysCfgBank + EE_SYSCFG0_CHKSUM + 1, myData + 1);

		currentSysCfgBank = tmpCurrentSysCfgBank;	// switch to the new bank.
	}

}

// save the system config shadow register(s) that do not match what is in the eeprom
void saveSysCfgDelta(EE_AREA_TYPE myArea)
{
	ushort myCnt;
	uchar* myPtrShReg;
	uchar* myPtrEe;
	uchar myData;
	//short eePromChangeMade = FALSE;
	//ushort myLen;
	ushort myCheckSum = 0;
	ulong tmpCurrentSysCfgBank;

	// double banking is used for the system config area
	// start to write the new transaction, when a full bank write is complete with checksum, then switch banks, and destroy the previous image
	if(myArea == EE_AREA_SYS_CFG)
	{
		// check the "other" System Configuration bank!
		if(currentSysCfgBank == EE_SYSCFG0_START)
			tmpCurrentSysCfgBank = EE_SYSCFG1_START;
		else
			tmpCurrentSysCfgBank = EE_SYSCFG0_START;

		myStartAddr[EE_AREA_SYS_CFG] = (ulong) tmpCurrentSysCfgBank;
		myChkSmAddr[EE_AREA_SYS_CFG] = (ulong) tmpCurrentSysCfgBank + EE_SYSCFG0_CHKSUM;
	}


	myPtrShReg = myShadowPtr[myArea];
	myPtrEe = myEePtr[myArea];

	// read in the eeprom contents for the sys cfg new bank.
	for(myCnt = myStartAddr[myArea]; myCnt < (myStartAddr[myArea] + myShadowRegSize[myArea]); myCnt++)
	{
		// read all of the system configuration values from EEPROM into shadow registers
		eepromRead(myCnt, myPtrEe++);

#if VORT_BOARD
	DelayUs(20);
#else
	delayMs(5);
#endif
	}

	myPtrEe = myEePtr[myArea];

	for(myCnt = myStartAddr[myArea]; myCnt < ( myStartAddr[myArea] + myShadowRegSize[myArea]); myCnt++)
	{
		// if the shadow register has changed (does not match what is in eeprom), write the "new" correct value to the EEProm
		if(*myPtrShReg	!= *myPtrEe)
		{
			// yes, write the system configuration value from shadow registers into EEPROM
			eepromWrite(myCnt, *myPtrShReg);
#if VORT_BOARD
	//delayMs(1);
#else
	delayMs(5);
#endif
			*myPtrEe = *myPtrShReg;		// save the new value in the eeprom variable

			//eePromChangeMade = TRUE;
		}

		// move onto the next byte/attribute for comparison
		myPtrShReg++;
		myPtrEe++;
	}

	// calculate checksum
	myCheckSum = chkSumCalc(myStartAddr[myArea], myChkSmAddr[myArea] - 1);

	// save checksum
	eepromWrite(myChkSmAddr[myArea], myCheckSum >> 8);

#if VORT_BOARD
	//delayMs(1);
#else
	delayMs(5);
#endif

	eepromWrite(myChkSmAddr[myArea] + 1, myCheckSum & 0xFF);
#if VORT_BOARD
	//delayMs(1);
#else
	delayMs(5);
#endif

 	// double banking is used for the system config area
	if(myArea == EE_AREA_SYS_CFG)
	{
		// destroy the previous bank.
		eepromRead(currentSysCfgBank + EE_SYSCFG0_CHKSUM + 1, &myData);	 // read the previous "correct" checksum LSB value

#if VORT_BOARD
		DelayUs(20);
#else
		delayMs(5);
#endif

		// destroy the image area: save correct checksum + 1. This allows the image area to be analyzed in the future
		eepromWrite(currentSysCfgBank + EE_SYSCFG0_CHKSUM + 1, myData + 1);

		currentSysCfgBank = tmpCurrentSysCfgBank;	// switch to the new bank.
	}
}

// verify the saved system configuration image
char verifySysCfg(EE_AREA_TYPE myArea)
{
	ushort myCheckSum = 0;
	uchar myData;
	ushort myCalcCheckSum = 0;


	// calculate checksum
	myCalcCheckSum = chkSumCalc(myStartAddr[myArea], myChkSmAddr[myArea] - 1);

	// get checksum
	eepromRead(myChkSmAddr[myArea], &myData);

#if VORT_BOARD
	DelayUs(20);
#else
	delayMs(5);
#endif

	myCheckSum = myData << 8;
	eepromRead(myChkSmAddr[myArea] + 1, &myData);

#if VORT_BOARD
	DelayUs(20);
#else
	delayMs(5);
#endif

	myCheckSum |= myData;

	if(myCalcCheckSum != myCheckSum)
		return FALSE;

	return TRUE;
}

// get system configuration values from the eeprom.
void getSysCfg(EE_AREA_TYPE myArea)
{
	ushort myCnt;
	uchar* myPtr;
	//ushort myLen;

	myPtr = myEePtr[myArea];


	for(myCnt = myStartAddr[myArea]; myCnt < (myStartAddr[myArea] + myShadowRegSize[myArea]); myCnt++)
	{
		// read all of the system configuration values from EEPROM into shadow registers
		eepromRead(myCnt, myPtr++);

#if VORT_BOARD
	DelayUs(20);
#else
	delayMs(5);
#endif
	}

	memcpy(myShadowPtr[myArea], myEePtr[myArea], myShadowRegSize[myArea]);	// copy the eeprom image into the shadow registers
}

void setShadowRegsToDef(EE_AREA_TYPE myArea)
{
	if(myArea == EE_AREA_SYS_CFG)
	{
		sysCfgShadowRegs.version = EE_DEF_SYSCFG_VERSION;
		sysCfgShadowRegs.c = EE_DEF_C;
		sysCfgShadowRegs.delay = EE_DEF_DELAY;
		sysCfgShadowRegs.echo = EE_DEF_ECHO;
		sysCfgShadowRegs.epc = EE_DEF_EPC;
		sysCfgShadowRegs.protoMode = EE_DEF_PROTO_VORT;
		sysCfgShadowRegs.laserHours = EE_DEF_LSR_HOURS;
		sysCfgShadowRegs.lc = EE_DEF_LC;
		sysCfgShadowRegs.lp = EE_DEF_LP;
		sysCfgShadowRegs.prompt = EE_DEF_PROMPT;
		sysCfgShadowRegs.peakPower = EE_DEF_PEAK_PWR;
		sysCfgShadowRegs.pulseMode = EE_DEF_PULSE_MODE;
		sysCfgShadowRegs.isTecOn = EE_DEF_TEC_ON;
		sysCfgShadowRegs.vobts = EE_DEF_VOBTS;
		sysCfgShadowRegs.currDrvSetting = EE_DEF_CURR_DRV;
		sysCfgShadowRegs.dfblc = EE_DEF_DFBLC;
		sysCfgShadowRegs.dfblcDacCnt = EE_DEF_DFBLC_DAC;
	}
	else if(myArea == EE_AREA_CAL)
	{
		calShadowRegs.version =  EE_DEF_CAL_VERSION;
		calShadowRegs.calVersion = EE_CAL_VERSION;
		calShadowRegs.maxPwrLsr = EE_MAX_PWR_LSR;
		calShadowRegs.initLsrCur = EE_INIT_LSR_CUR;
		calShadowRegs.initDfbCur = EE_INIT_DFB_CUR;
		calShadowRegs.nominalLsrPwr = EE_NOMINAL_LSR_PWR;
		calShadowRegs.maxLsrCur = EE_MAX_LSR_CUR;
		calShadowRegs.nominalWaveLen = EE_NOMINAL_WAVE_LEN;
		calShadowRegs.measWaveLen = EE_MEAS_WAVE_LEN;
		calShadowRegs.lsrPwrSetPot = EE_LSR_PWR_SET_POT;
		calShadowRegs.lsrCurSetPot = EE_LSR_CUR_SET_POT;
		calShadowRegs.obtDeviation = EE_DEF_OBT_DEV;
		calShadowRegs.calDacValue = EE_DEF_CAL_DAC;
		calShadowRegs.lcScaleFactor = EE_LC_SCALE_FACTOR;
		calShadowRegs.epcScaleFactor = EE_EPC_SCALE_FACTOR;
		calShadowRegs.lpOffsetAdjust = EE_LP_OFFSET_ADJUST;

		calShadowRegs.endOfLifeEnable = EE_END_OF_LIFE;
		calShadowRegs.endOfLifeFaultLine = EE_EOL_FAULT_LINE;
		calShadowRegs.nonVolPUL = EE_DEF_NVPUL;
		calShadowRegs.epcScaleFactor2 = EE_EPC_SCALE_FACTOR2;
		calShadowRegs.mode = EE_MODE;
		calShadowRegs.rfInj = EE_RFINJ;
		calShadowRegs.maxCurrDfb = EE_DFB_MAX_CURR;
		calShadowRegs.maxCurrHtr = EE_HTR_MAX_CURR;
		calShadowRegs.htrCnt = EE_DEF_HTR_CNT;
		calShadowRegs.calDacDfbValue = EE_DEF_CAL_DAC_DFB;
		calShadowRegs.maxOverCurr = EE_DEF_MAX_OVER_CURR;
		
		strcpy(calShadowRegs.lsrPartNum, EE_LSR_PART_NUM);
		strcpy(calShadowRegs.lsrSerialNum, EE_LSR_SERIAL_NUM);
		strcpy(calShadowRegs.lsrBeamShape, EE_LSR_BEAM_SHAPE);
	}
}

void forceSysCfgDefaults(void)
{
	saveCfgDefaults(EE_AREA_SYS_CFG);	// save the system config
	saveCfgDefaults(EE_AREA_CAL);			// save the calibration information
	saveSwErrDefaults();
}


