//
//						Timer.c
//
//	Timer interrupt Service Routines
//
//	(C) SoftwareFlair Inc. 2007
//
#include <LPC214X.H> 	// LPC214x definitions 
#include "type.h"                       
#include "system.h"
#include "usbcfg.h"

ulong execTicks = 0;		// Exec Tick 10ms resolution, fixed at 10ms
ulong prevExecTicks = 0;		// Previous Exec Tick used to obtain 1 second resolution
ulong prevExecTicksDebug = 0;	//Used to produce ticks of less than 1 second resolution
ulong oneSecTicks = 0;
ulong debugTicks = 0;
ulong swTicks = 0;		// sw Tick 10ms resolution, period is specified by software

// Clock Definitions 
#define CPU_CLOCK 60000000              // CPU Clock
#define VPB_CLOCK (CPU_CLOCK/1)         // VPB Clock 

//#define TIMER0_FREQ 100000				// Timer0 frequency  for 100us
#define TIMER0_FREQ 1					// Timer0 frequency  for 10ms
#define TIMER1_FREQ 1					// Timer1 frequency (system tick count) 

// TODO: use scope to calibrate time
#define TICKS_PER_SECOND  (TIMER1_FREQ)	// number of system execTicks per second 


#define MSECS_TO_SAMPLES(msecs)  (msecs * TIMER0_FREQ / 1000)
#define FREQ_TO_HALF_SAMPLES(freq)  (TIMER0_FREQ / (freq * 2))


#define STAGE_COUNT   (sizeof (clickStages) / sizeof (clickStages[0]))

BOOL triggerSound = FALSE;				// Set to TRUE to trigger click sound


// Timer 0 Interrupt Service Routine
//   executed by software, for specified period in ms
void tc0_isr (void) __irq
{
	wasHere = 0x130;
	swTicks++;						// update sw tick counter 

	T0IR = 1;                                 // Clear Interrupt Flag 
	VICVectAddr = 0;                          // Acknowledge Interrupt 
}

// Timer 1 Interrupt Service Routine
//  Calibrated for 12MHz clock: executes every 10ms (100 Hz frequency)
//   Handles executive tick count
void tc1_isr (void) __irq
{
	wasHere = 0x140;

	#if 0		  // SW DEBUG
	if(IO0PIN & PIN_FAULT_INDC)
		IO0CLR = PIN_FAULT_INDC;
	else
		IO0SET = PIN_FAULT_INDC;
	#endif

	execTicks++;						// update exec tick counter 

	// has 100 exec ticks occured (which equals 1 second)?
	if((execTicks - prevExecTicks) >= 100)
	{
		oneSecTicks++;
		prevExecTicks = execTicks;
	}

	if((execTicks - prevExecTicksDebug) >= 50)
	{
		debugTicks++;
		prevExecTicksDebug = execTicks;
	}

	cdrhStartTime++;	// another 10 ms has elapsed, initialized to 0 when laser enable command occurs
	
	T1IR = 1;						// Clear Interrupt Flag 
	VICVectAddr = 0;				// Acknowledge Interrupt 
}

void timerInit (void)
{
	timer0Init(10);

	timer1Init(10);		// Executive clock
}

void timer0Init (ushort myPeriodInMs)
{
	swTicks = 0;		// sw Tick 10ms resolution, period is specified by software
	
	// Setup Timer Counter 0 Interrupt 
	VICVectAddr2 = (unsigned long)tc0_isr;	// TC0 Interrupt -> Vector 2
	VICVectCntl2 = 0x20 | 4;				// TC0 Interrupt -> IRQ Slot 2 

	// Setup Timer Counter 0: Periodic Timer with Interrupt at TIMER0_FREQ Rate
	//T0MR0 = ((VPB_CLOCK) * 10 * myPeriodInMs) - 1;     // TC1 Match Value 0
	T0MR0 = ((VPB_CLOCK)/(1000/(myPeriodInMs * 1))) - 1;
	T0MCR = 3;                               	// TCO Interrupt and Reset on MR0
	T0TCR = 0;                               	// TC0 Disable

 	VICIntEnable = 1 << 4;                    	// Enable TC0 Interrupt

	swTicks = 0;
}

// Executive Timer
void timer1Init (ushort myPeriodInMs)
{

 	execTicks = 0;		// Exec Tick 10ms resolution, fixed at 10ms
 	prevExecTicks = 0;		// Previous Exec Tick used to obtain 1 second resolution
	prevExecTicksDebug = 0;
	
	// Setup Timer Counter 1 Interrupt 
	VICVectAddr3 = (unsigned long)tc1_isr;	// TC1 Interrupt -> Vector 3
	VICVectCntl3 = 0x20 | 5;				// TC1 Interrupt -> IRQ Slot 3

	// Setup Timer Counter 1: Periodic Timer with Interrupt at TIMER1_FREQ Rate
	if(myPeriodInMs == 0)
		T1MR0 = VPB_CLOCK/TIMER1_FREQ - 1;       	// TC1 Match Value 0
	else
	{
		if(myPeriodInMs == 10)
			T1MR0 = ((VPB_CLOCK)/(1000/(myPeriodInMs * 1))) - 1 ;       	// 10ms
		else if(myPeriodInMs == 100)
			T1MR0 = ((VPB_CLOCK)/10) - 1;       	// 100ms
		else if(myPeriodInMs == 500)
			T1MR0 = ((VPB_CLOCK)/2) - 1;       	// 500ms
		else if(myPeriodInMs == 1000)
			T1MR0 = (VPB_CLOCK) - 1;       	// 1 second
		else if(myPeriodInMs == 2000)
			T1MR0 = (VPB_CLOCK * 2) - 1;       	// TC1 Match Value 0
		else
			T1MR0 = ((VPB_CLOCK)/(1000/(myPeriodInMs * 2))) - 1;       	// TC1 Match Value 0

	}

	T1MCR = 3;                               	// TC1 Interrupt and Reset on MR0

	VICIntEnable = 1 << 5;						// Enable TC1 Interrupt
}

void timer0Start(void)
{
	T0TCR = 1;                               	// TC1 Enable 
}

void timer0Stop(void)
{
	T0TCR = 0;                               	// TC1 Disable  
}

void timer1Start(void)
{
	T1TCR = 1;                               	// TC1 Enable 
}

void timer1Stop(void)
{
	T1TCR = 0;                               	// TC1 Disable  
}

long timer0ReadTicks(void)
{
 	return swTicks;
} 


// wait for specified number of system execTicks (TICKS_PER_SECOND execTicks/sec)
void waitExecTicks (int myExecTickPeriod)
{
	//long startExecTicks = myExecTickPeriod;

	//while (myExecTickPeriod - startExecTicks < execTicks);		// Handles wrap around
}
